@@ -331,6 +331,7 @@ static void dai_diplomat_city(struct ai_type *ait, struct unit *punit,
   struct player *pplayer = unit_owner(punit);
   struct player *tplayer = city_owner(ctarget);
   int count_tech = count_stealable_techs(pplayer, tplayer);
+  const struct civ_map *nmap = &(wld.map);
 
   fc_assert_ret(is_ai(pplayer));
 
@@ -357,7 +358,7 @@ static void dai_diplomat_city(struct ai_type *ait, struct unit *punit,
     }
 
     if (!action_prob_possible(
-            action_prob_vs_city(punit, action_number(paction),ctarget))) {
+            action_prob_vs_city(nmap, punit, action_number(paction),ctarget))) {
       /* Not possible. */
       continue;
     }
@@ -386,7 +387,7 @@ static void dai_diplomat_city(struct ai_type *ait, struct unit *punit,
     /* An action has been selected. */
 
     if (action_prob_possible(
-          action_prob_vs_city(punit, action_number(chosen_action),
+          action_prob_vs_city(nmap, punit, action_number(chosen_action),
                               ctarget))) {
       log_base(LOG_DIPLOMAT, "%s %s[%d] does %s at %s",
                nation_rule_name(nation_of_unit(punit)),
@@ -587,6 +588,7 @@ static bool dai_diplomat_bribe_nearby(struct ai_type *ait,
                                       struct unit *punit, struct pf_map *pfm)
 {
   int gold_avail, expenses;
+  const struct civ_map *nmap = &(wld.map);
 
   dai_calc_data(pplayer, NULL, &expenses, NULL);
   gold_avail = pplayer->economic.gold - expenses;
@@ -608,7 +610,7 @@ static bool dai_diplomat_bribe_nearby(struct ai_type *ait,
         || !POTENTIALLY_HOSTILE_PLAYER(ait, pplayer, unit_owner(pvictim))
         || unit_list_size(ptile->units) > 1
         || tile_city(ptile)
-        || !is_action_enabled_unit_on_unit(ACTION_SPY_BRIBE_UNIT,
+        || !is_action_enabled_unit_on_unit(nmap, ACTION_SPY_BRIBE_UNIT,
                                            punit, pvictim)) {
       continue;
     }
@@ -659,7 +661,7 @@ static bool dai_diplomat_bribe_nearby(struct ai_type *ait,
       struct tile *bribee_tile;
       struct pf_path *path;
 
-      bribee_tile = mapstep(&(wld.map), pos.tile, DIR_REVERSE(pos.dir_to_here));
+      bribee_tile = mapstep(nmap, pos.tile, DIR_REVERSE(pos.dir_to_here));
       path = pf_map_path(pfm, bribee_tile);
       if (!path || !adv_unit_execute_path(punit, path) 
           || punit->moves_left <= 0) {
@@ -669,7 +671,7 @@ static bool dai_diplomat_bribe_nearby(struct ai_type *ait,
       pf_path_destroy(path);
     }
 
-    if (action_prob_possible(action_prob_vs_unit(punit,
+    if (action_prob_possible(action_prob_vs_unit(nmap, punit,
                                  ACTION_SPY_BRIBE_UNIT,
                                  pvictim))) {
       unit_do_action(pplayer, punit->id,
@@ -681,7 +683,7 @@ static bool dai_diplomat_bribe_nearby(struct ai_type *ait,
       } else {
         return FALSE;
       }
-    } else if (action_prob_possible(action_prob_vs_unit(punit,
+    } else if (action_prob_possible(action_prob_vs_unit(nmap, punit,
                                         ACTION_SPY_SABOTAGE_UNIT_ESC,
                                         pvictim))
                && threat) {
