@@ -295,7 +295,7 @@ static void aiferry_request_boat(struct ai_type *ait, struct unit *punit)
   struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
 
   /* First clear the previous assignments (just in case there are). 
-   * Substract virtual units or already counted */
+   * Subtract virtual units or already counted */
   if ((punit->id == 0) || 
       ((ai->stats.passengers > 0) && 
        (unit_data->ferryboat == FERRY_WANTED))) {
@@ -840,7 +840,7 @@ bool aiferry_gobyboat(struct ai_type *ait, struct player *pplayer,
     }
 
     action_by_result_iterate(paction, ACTRES_TRANSPORT_BOARD) {
-      if (action_prob_possible(action_prob_vs_unit(punit, paction->id,
+      if (action_prob_possible(action_prob_vs_unit(nmap, punit, paction->id,
                                                    ferryboat))) {
         if (unit_perform_action(pplayer,
                                 punit->id, ferryboat->id, 0, "",
@@ -904,7 +904,7 @@ bool aiferry_gobyboat(struct ai_type *ait, struct player *pplayer,
 
         /* Bodyguard either uses the same boat or none at all. */
         action_by_result_iterate(paction, ACTRES_TRANSPORT_BOARD) {
-          if (action_prob_possible(action_prob_vs_unit(bodyguard,
+          if (action_prob_possible(action_prob_vs_unit(nmap, bodyguard,
                                                        paction->id,
                                                        ferryboat))) {
             if (unit_perform_action(pplayer,
@@ -1118,6 +1118,7 @@ void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
   struct unit_ai *unit_data;
   int bossid;
   const struct unit_type *ptype;
+  const struct civ_map *nmap = &(wld.map);
 
   CHECK_UNIT(punit);
 
@@ -1247,7 +1248,7 @@ void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
       * -- Syela */
     dai_unit_new_task(ait, punit, AIUNIT_NONE, NULL);
     UNIT_LOG(LOGLEVEL_FERRY, punit, "passing ferry over to attack code");
-    dai_manage_military(ait, pplayer, punit);
+    dai_manage_military(ait, nmap, pplayer, punit);
     return;
   }
 
