@@ -205,13 +205,14 @@ static void dai_barbarian_choose_build(struct player *pplayer,
 {
   struct unit_type *bestunit = NULL;
   int i, bestattack = 0;
+  const struct civ_map *nmap = &(wld.map);
 
   /* Choose the best unit among the basic ones */
   for (i = 0; i < num_role_units(L_BARBARIAN_BUILD); i++) {
     struct unit_type *iunit = get_role_unit(L_BARBARIAN_BUILD, i);
 
     if (iunit->attack_strength > bestattack
-        && can_city_build_unit_now(pcity, iunit)) {
+        && can_city_build_unit_now(nmap, pcity, iunit)) {
       bestunit = iunit;
       bestattack = iunit->attack_strength;
     }
@@ -222,7 +223,7 @@ static void dai_barbarian_choose_build(struct player *pplayer,
     struct unit_type *iunit = get_role_unit(L_BARBARIAN_BUILD_TECH, i);
 
     if (iunit->attack_strength > bestattack
-        && can_city_build_unit_now(pcity, iunit)) {
+        && can_city_build_unit_now(nmap, pcity, iunit)) {
       bestunit = iunit;
       bestattack = iunit->attack_strength;
     }
@@ -254,6 +255,7 @@ static void dai_city_choose_build(struct ai_type *ait, struct player *pplayer,
   struct adv_choice *newchoice;
   struct adv_data *adv = adv_data_get(pplayer, NULL);
   struct ai_city *city_data = def_ai_city_data(pcity, ait);
+  const struct civ_map *nmap = &(wld.map);
 
   if (has_handicap(pplayer, H_AWAY)
       && !city_built_last_turn(pcity)
@@ -290,7 +292,7 @@ static void dai_city_choose_build(struct ai_type *ait, struct player *pplayer,
       unsigned int our_def = assess_defense_quadratic(ait, pcity);
 
       if (our_def == 0
-          && dai_process_defender_want(ait, pplayer, pcity, 1,
+          && dai_process_defender_want(ait, nmap, pplayer, pcity, 1,
                                        &(city_data->choice), 0)) {
         adv_choice_set_use(&(city_data->choice), "fallback defender");
         CITY_LOG(LOG_DEBUG, pcity, "Building fallback defender");
@@ -453,6 +455,7 @@ static void unit_do_disband_trad(struct player *owner, struct unit *punit,
                                  const enum action_requester requester)
 {
   const int punit_id_stored = punit->id;
+  const struct civ_map *nmap = &(wld.map);
 
   fc_assert_ret(owner == unit_owner(punit));
 
@@ -465,7 +468,7 @@ static void unit_do_disband_trad(struct player *owner, struct unit *punit,
     tgt_city = tile_city(unit_tile(punit));
 
     if (tgt_city
-        && is_action_enabled_unit_on_city(ACTION_HELP_WONDER,
+        && is_action_enabled_unit_on_city(nmap, ACTION_HELP_WONDER,
                                           punit, tgt_city)) {
       if (unit_perform_action(owner, punit->id, tgt_city->id,
                               0, NULL, ACTION_HELP_WONDER, requester)) {
@@ -489,7 +492,7 @@ static void unit_do_disband_trad(struct player *owner, struct unit *punit,
     tgt_city = tile_city(unit_tile(punit));
 
     if (tgt_city
-        && is_action_enabled_unit_on_city(ACTION_DISBAND_UNIT_RECOVER,
+        && is_action_enabled_unit_on_city(nmap, ACTION_DISBAND_UNIT_RECOVER,
                                           punit, tgt_city)) {
       if (unit_perform_action(owner, punit->id, tgt_city->id,
                               0, NULL, ACTION_DISBAND_UNIT_RECOVER, requester)) {
@@ -506,7 +509,7 @@ static void unit_do_disband_trad(struct player *owner, struct unit *punit,
 
   /* Try to disband even if all shields will be wasted. */
   if (unit_can_do_action(punit, ACTION_DISBAND_UNIT)) {
-    if (is_action_enabled_unit_on_self(ACTION_DISBAND_UNIT, punit)) {
+    if (is_action_enabled_unit_on_self(nmap, ACTION_DISBAND_UNIT, punit)) {
       if (unit_perform_action(owner, punit->id, punit->id,
                               0, NULL, ACTION_DISBAND_UNIT, requester)) {
         /* All shields wasted. The unit did Disband Unit. */
@@ -907,7 +910,7 @@ void dai_manage_cities(struct ai_type *ait, struct player *pplayer)
     if (city_data->choice.want <= 0) {
       /* Note that this function mungs the seamap, but we don't care */
       TIMING_LOG(AIT_CITY_MILITARY, TIMER_START);
-      choice = military_advisor_choose_build(ait, pplayer, pcity, &(wld.map), NULL);
+      choice = military_advisor_choose_build(ait, &(wld.map), pplayer, pcity, NULL);
       adv_choice_copy(&(city_data->choice), choice);
       adv_free_choice(choice);
       TIMING_LOG(AIT_CITY_MILITARY, TIMER_STOP);
@@ -1030,6 +1033,7 @@ static void resolve_city_emergency(struct ai_type *ait, struct player *pplayer,
                                    struct city *pcity)
 {
   struct tile *pcenter = city_tile(pcity);
+  const struct civ_map *nmap = &(wld.map);
 
   log_base(LOG_EMERGENCY,
            "Emergency in %s (%s, angry%d, unhap%d food%d, prod%d)",
@@ -1040,7 +1044,7 @@ static void resolve_city_emergency(struct ai_type *ait, struct player *pplayer,
            pcity->surplus[O_FOOD],
            pcity->surplus[O_SHIELD]);
 
-  city_tile_iterate(city_map_radius_sq_get(pcity), pcenter, atile) {
+  city_tile_iterate(nmap, city_map_radius_sq_get(pcity), pcenter, atile) {
     struct city *acity = tile_worked(atile);
 
     if (acity && acity != pcity && city_owner(acity) == city_owner(pcity))  {
@@ -1074,10 +1078,10 @@ static void resolve_city_emergency(struct ai_type *ait, struct player *pplayer,
   unit_list_iterate_safe(pcity->units_supported, punit) {
     if (city_unhappy(pcity)
         && (utype_happy_cost(unit_type_get(punit), pplayer) > 0
-            && (unit_being_aggressive(punit) || is_field_unit(punit)))
+            && (unit_being_aggressive(nmap, punit) || is_field_unit(punit)))
         && def_ai_unit_data(punit, ait)->passenger == 0) {
       UNIT_LOG(LOG_EMERGENCY, punit, "is causing unrest, disbanded");
-      /* TODO: if Help Wonder stops blocking Disband Unit there may be
+      /* TODO: If Help Wonder stops blocking Disband Unit there may be
        * cases where Disband Unit should be selected. Example: Field unit
        * in allied city that is building a wonder that makes the ally win
        * without sharing the victory. */
@@ -1417,6 +1421,7 @@ adv_want dai_city_want(struct player *pplayer, struct city *acity,
 {
   adv_want want = 0;
   int prod[O_LAST], bonus[O_LAST], waste[O_LAST];
+  const struct civ_map *nmap = &(wld.map);
 
   memset(prod, 0, O_LAST * sizeof(*prod));
   if (NULL != pimprove
@@ -1427,7 +1432,7 @@ adv_want dai_city_want(struct player *pplayer, struct city *acity,
     /* The below calculation mostly duplicates get_worked_tile_output().
      * We do this only for buildings that we know may change tile
      * outputs. */
-    city_tile_iterate(city_map_radius_sq_get(acity), acenter, ptile) {
+    city_tile_iterate(nmap, city_map_radius_sq_get(acity), acenter, ptile) {
       if (tile_worked(ptile) == acity) {
         output_type_iterate(o) {
           prod[o] += city_tile_output(acity, ptile, celebrating, o);
