@@ -76,8 +76,9 @@
 #define CITY_CONQUEST_WORTH(_city_, _data_) \
   (_data_->worth * 0.9 + (city_size_get(_city_) - 0.5) * 10)
 
-static unsigned int assess_danger(struct ai_type *ait, struct city *pcity,
-                                  const struct civ_map *dmap,
+static unsigned int assess_danger(struct ai_type *ait,
+                                  const struct civ_map *nmap,
+                                  struct city *pcity,
                                   player_unit_list_getter ul_cb);
 
 static adv_want dai_unit_attack_desirability(struct ai_type *ait,
@@ -98,7 +99,7 @@ struct unit_type *dai_choose_defender_versus(struct city *pcity,
   struct civ_map *nmap = &(wld.map);
 
   simple_ai_unit_type_iterate(punittype) {
-    if (can_city_build_unit_now(pcity, punittype)) {
+    if (can_city_build_unit_now(nmap, pcity, punittype)) {
       int fpatt, fpdef, defense, attack;
       double want, loss, cost = utype_build_shield_cost(pcity, NULL, punittype);
       struct unit *defender;
@@ -151,6 +152,7 @@ static struct unit_type *dai_choose_attacker(struct ai_type *ait,
   adv_want best = -1;
   adv_want cur;
   struct player *pplayer = city_owner(pcity);
+  const struct civ_map *nmap = &(wld.map);
 
   simple_ai_unit_type_iterate(putype) {
     if (!allow_gold_upkeep && utype_upkeep_cost(putype, pplayer, O_GOLD) > 0) {
@@ -161,7 +163,7 @@ static struct unit_type *dai_choose_attacker(struct ai_type *ait,
     if ((tc == TC_LAND && utype_class(putype)->adv.land_move != MOVE_NONE)
         || (tc == TC_OCEAN
             && utype_class(putype)->adv.sea_move != MOVE_NONE)) {
-      if (can_city_build_unit_now(pcity, putype)
+      if (can_city_build_unit_now(nmap, pcity, putype)
           && (cur > best
               || (ADV_WANTS_EQ(cur, best)
                   && utype_build_shield_cost(pcity, NULL, putype)
@@ -184,6 +186,7 @@ static struct unit_type *dai_choose_attacker(struct ai_type *ait,
   is the only role being considered worthy of bodyguarding in findjob.
 **************************************************************************/
 static struct unit_type *dai_choose_bodyguard(struct ai_type *ait,
+                                              const struct civ_map *nmap,
                                               struct city *pcity,
                                               enum terrain_class tc,
                                               enum unit_role_id role,
@@ -213,7 +216,7 @@ static struct unit_type *dai_choose_bodyguard(struct ai_type *ait,
     }
 
     /* Now find best */
-    if (can_city_build_unit_now(pcity, putype)) {
+    if (can_city_build_unit_now(nmap, pcity, putype)) {
       const adv_want desire = dai_unit_defense_desirability(ait, putype);
 
       if (desire > best
@@ -507,7 +510,8 @@ action_may_happen_unit_on_city(const action_id wanted_action,
 /**********************************************************************//**
   How dangerous and far a unit is for a city?
 **************************************************************************/
-static unsigned int assess_danger_unit(const struct city *pcity,
+static unsigned int assess_danger_unit(const struct civ_map *nmap,
+                                       const struct city *pcity,
                                        struct pf_reverse_map *pcity_map,
                                        const struct unit *punit,
                                        int *move_time)
@@ -519,7 +523,6 @@ static unsigned int assess_danger_unit(const struct city *pcity,
   const struct unit *ferry;
   unsigned int danger;
   int amod = -99, dmod;
-  struct civ_map *nmap = &(wld.map);
   bool attack_danger = FALSE;
 
   *move_time = PF_IMPOSSIBLE_MC;
@@ -610,13 +613,14 @@ static unsigned int assess_danger_unit(const struct city *pcity,
 
   This is necessary to initialize some ai data before some ai calculations.
 **************************************************************************/
-void dai_assess_danger_player(struct ai_type *ait, struct player *pplayer,
-                              const struct civ_map *dmap)
+void dai_assess_danger_player(struct ai_type *ait,
+                              const struct civ_map *nmap,
+                              struct player *pplayer)
 {
   /* Do nothing if game is not running */
   if (S_S_RUNNING == server_state()) {
     city_list_iterate(pplayer->cities, pcity) {
-      (void) assess_danger(ait, pcity, dmap, NULL);
+      (void) assess_danger(ait, nmap, pcity, NULL);
     } city_list_iterate_end;
   }
 }
@@ -675,8 +679,9 @@ static void dai_reevaluate_building(struct city *pcity, adv_want *value,
   afraid of a boat laden with enemies if it stands on the coast (i.e.
   is directly reachable by this boat).
 **************************************************************************/
-static unsigned int assess_danger(struct ai_type *ait, struct city *pcity,
-                                  const struct civ_map *dmap,
+static unsigned int assess_danger(struct ai_type *ait,
+                                  const struct civ_map *nmap,
+                                  struct city *pcity,
                                   player_unit_list_getter ul_cb)
 {
   struct player *pplayer = city_owner(pcity);
@@ -802,8 +807,8 @@ static unsigned int assess_danger(struct ai_type *ait, struct city *pcity,
     /* Note that we still consider the units of players we are not (yet)
      * at war with. */
 
-    pcity_map = pf_reverse_map_new_for_city(pcity, aplayer, assess_turns,
-                                            omnimap, dmap);
+    pcity_map = pf_reverse_map_new_for_city(nmap, pcity, aplayer, assess_turns,
+                                            omnimap);
 
     if (ul_cb != NULL) {
       units = ul_cb(aplayer);
@@ -839,7 +844,7 @@ static unsigned int assess_danger(struct ai_type *ait, struct city *pcity,
       }
 
       /* Defender unspecific vulnerability and potential move time */
-      vulnerability = assess_danger_unit(pcity, pcity_map,
+      vulnerability = assess_danger_unit(nmap, pcity, pcity_map,
                                          punit, &move_time);
 
       if (PF_IMPOSSIBLE_MC == move_time) {
@@ -866,7 +871,7 @@ static unsigned int assess_danger(struct ai_type *ait, struct city *pcity,
       vulnerability = vulnerability * 100 / (defbonus_pct + 100);
       /* Pass the order for a new defender type against it
        * to the scientific advisor, urgency considered */
-      (void) dai_wants_defender_against(ait, pplayer, pcity, utype,
+      (void) dai_wants_defender_against(ait, nmap, pplayer, pcity, utype,
                                         vulnerability / MAX(move_time, 1));
 
       if (utype_acts_hostile(unit_type_get(punit)) && 2 >= move_time) {
@@ -1031,7 +1036,8 @@ static adv_want dai_unit_attack_desirability(struct ai_type *ait,
   type in choice. Also sets the technology want for the units we can't
   build yet.
 **************************************************************************/
-bool dai_process_defender_want(struct ai_type *ait, struct player *pplayer,
+bool dai_process_defender_want(struct ai_type *ait, const struct civ_map *nmap,
+                               struct player *pplayer,
                                struct city *pcity, unsigned int danger,
                                struct adv_choice *choice, adv_want extra_want)
 {
@@ -1085,7 +1091,7 @@ bool dai_process_defender_want(struct ai_type *ait, struct player *pplayer,
       desire /= POWER_DIVIDER / 2; /* Good enough, no rounding errors. */
       desire *= desire;
 
-      if (can_city_build_unit_now(pcity, punittype)) {
+      if (can_city_build_unit_now(nmap, pcity, punittype)) {
         /* We can build the unit now... */
 
         int build_cost = utype_build_shield_cost(pcity, NULL, punittype);
@@ -1109,7 +1115,7 @@ bool dai_process_defender_want(struct ai_type *ait, struct player *pplayer,
           best_unit_type = punittype;
           best_unit_cost = build_cost;
         }
-      } else if (can_city_build_unit_later(pcity, punittype)) {
+      } else if (can_city_build_unit_later(nmap, pcity, punittype)) {
         /* We first need to develop the tech required by the unit... */
 
         /* Cost (shield equivalent) of gaining these techs. */
@@ -1224,13 +1230,15 @@ static void process_attacker_want(struct ai_type *ait,
   const struct unit_type *orig_utype = best_choice->value.utype;
   int victim_count = 1;
   int needferry = 0;
-  bool unhap = dai_assess_military_unhappiness(pcity);
+  bool unhap;
   struct ai_plr *plr_data = def_ai_player_data(pplayer, ait);
   const struct civ_map *nmap = &(wld.map);
 
   /* Has to be initialized to make gcc happy */
   struct ai_city *acity_data = NULL;
 
+  unhap = dai_assess_military_unhappiness(nmap, pcity);
+
   if (acity != NULL) {
     acity_data = def_ai_city_data(acity, ait);
   }
@@ -1254,8 +1262,8 @@ static void process_attacker_want(struct ai_type *ait,
     if (dai_can_unit_type_follow_unit_type(punittype, orig_utype, ait)
         && is_native_near_tile(&(wld.map), utype_class(punittype), ptile)
         && (U_NOT_OBSOLETED == punittype->obsoleted_by
-            || !can_city_build_unit_direct(pcity, punittype->obsoleted_by))
-        && punittype->attack_strength > 0 /* or we'll get SIGFPE */) {
+            || !can_city_build_unit_direct(nmap, pcity, punittype->obsoleted_by))
+        && punittype->attack_strength > 0 /* Or we'll get SIGFPE */) {
       /* Values to be computed */
       adv_want desire;
       adv_want want;
@@ -1416,7 +1424,7 @@ static void process_attacker_want(struct ai_type *ait,
         } else if (want > best_choice->want) {
           const struct impr_type *impr_req;
 
-          if (can_city_build_unit_now(pcity, punittype)) {
+          if (can_city_build_unit_now(nmap, pcity, punittype)) {
             /* This is a real unit and we really want it */
 
             CITY_LOG(LOG_DEBUG, pcity, "overriding %s(" ADV_WANT_PRINTF
@@ -1668,6 +1676,7 @@ cleanup:
   if want is 0 this advisor doesn't want anything
 **************************************************************************/
 static void dai_unit_consider_bodyguard(struct ai_type *ait,
+                                        const struct civ_map *nmap,
                                         struct city *pcity,
                                         struct unit_type *punittype,
                                         struct adv_choice *choice)
@@ -1680,7 +1689,7 @@ static void dai_unit_consider_bodyguard(struct ai_type *ait,
       = unit_virtual_create(pplayer, pcity, punittype,
                             city_production_unit_veteran_level(pcity,
                                                                punittype));
-    const adv_want want = look_for_charge(ait, pplayer, virtualunit,
+    const adv_want want = look_for_charge(ait, nmap, pplayer, virtualunit,
                                           &aunit, &acity);
 
     if (want > choice->want) {
@@ -1733,9 +1742,9 @@ static void adjust_ai_unit_choice(struct city *pcity,
   If 'choice->want' is 0 this advisor doesn't want anything.
 **************************************************************************/
 struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
+                                                 const struct civ_map *nmap,
                                                  struct player *pplayer,
                                                  struct city *pcity,
-                                                 const struct civ_map *mamap,
                                                  player_unit_list_getter ul_cb)
 {
   struct adv_data *ai = adv_data_get(pplayer, NULL);
@@ -1749,10 +1758,10 @@ struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
   struct adv_choice *choice = adv_new_choice();
   bool allow_gold_upkeep;
 
-  urgency = assess_danger(ait, pcity, mamap, ul_cb);
+  urgency = assess_danger(ait, nmap, pcity, ul_cb);
   /* Changing to quadratic to stop AI from building piles 
    * of small units -- Syela */
-  /* It has to be AFTER assess_danger thanks to wallvalue. */
+  /* It has to be AFTER assess_danger() thanks to wallvalue. */
   our_def = assess_defense_quadratic(ait, pcity); 
 
   dai_choose_diplomat_defensive(ait, pplayer, pcity, choice, our_def);
@@ -1820,7 +1829,7 @@ struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
     if (our_def == 0 && danger > 0) {
       /* Build defensive unit first! Walls will help nothing if there's
        * nobody behind them. */
-      if (dai_process_defender_want(ait, pplayer, pcity, danger, choice,
+      if (dai_process_defender_want(ait, nmap, pplayer, pcity, danger, choice,
                                     martial_value)) {
         choice->want = DAI_WANT_BELOW_MIL_EMERGENCY + danger;
         adv_choice_set_use(choice, "first defender");
@@ -1853,7 +1862,7 @@ struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
         if (pcity->server.adv->building_want[wall_id] > 0) {
           /* NB: great wall is under domestic */
           choice->value.building = pimprove;
-          /* building_want is hacked by assess_danger */
+          /* building_want is hacked by assess_danger() */
           choice->want = pcity->server.adv->building_want[wall_id];
           choice->want = choice->want * (0.5 + (ai_trait_get_value(TRAIT_BUILDER, pplayer)
                                                 / TRAIT_DEFAULT_VALUE / 2));
@@ -1881,8 +1890,8 @@ struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
         adv_init_choice(&uchoice);
 
         /* Consider building defensive units */
-        if (dai_process_defender_want(ait, pplayer, pcity, danger, &uchoice,
-                                      martial_value)) {
+        if (dai_process_defender_want(ait, nmap, pplayer, pcity, danger,
+                                      &uchoice, martial_value)) {
           /* Potential defender found */
           if (urgency == 0
               && uchoice.value.utype->defense_strength == 1) {
@@ -1938,10 +1947,10 @@ struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
   }
 
   /* Consider making a land bodyguard */
-  punittype = dai_choose_bodyguard(ait, pcity, TC_LAND, L_DEFEND_GOOD,
+  punittype = dai_choose_bodyguard(ait, nmap, pcity, TC_LAND, L_DEFEND_GOOD,
                                    allow_gold_upkeep);
   if (punittype) {
-    dai_unit_consider_bodyguard(ait, pcity, punittype, choice);
+    dai_unit_consider_bodyguard(ait, nmap, pcity, punittype, choice);
   }
 
   /* If we are in severe danger, don't consider attackers. This is probably
@@ -1959,10 +1968,10 @@ struct adv_choice *military_advisor_choose_build(struct ai_type *ait,
   dai_choose_diplomat_offensive(ait, pplayer, pcity, choice);
 
   /* Consider making a sea bodyguard */
-  punittype = dai_choose_bodyguard(ait, pcity, TC_OCEAN, L_DEFEND_GOOD,
+  punittype = dai_choose_bodyguard(ait, nmap, pcity, TC_OCEAN, L_DEFEND_GOOD,
                                    allow_gold_upkeep);
   if (punittype) {
-    dai_unit_consider_bodyguard(ait, pcity, punittype, choice);
+    dai_unit_consider_bodyguard(ait, nmap, pcity, punittype, choice);
   }
 
   /* Consider making an airplane */
