@@ -105,6 +105,7 @@ static void tai_tile_worker_task_select(struct player *pplayer,
 {
   int orig_value;
   bool potential_worst_worked = FALSE;
+  const struct civ_map *nmap = &(wld.map);
 
   if (!city_can_work_tile(pcity, ptile)) {
     return;
@@ -151,7 +152,7 @@ static void tai_tile_worker_task_select(struct player *pplayer,
       }
 
       if (action_prob_possible(
-            action_speculate_unit_on_tile(act,
+            action_speculate_unit_on_tile(nmap, act,
                                           punit, unit_home(punit), ptile,
                                           TRUE,
                                           ptile, tgt))) {
@@ -174,7 +175,7 @@ static void tai_tile_worker_task_select(struct player *pplayer,
           if (limit == TWTL_BUILDABLE_UNITS) {
             unit_list_iterate(units, punit) {
               if (action_prob_possible(
-                    action_speculate_unit_on_tile(act,
+                    action_speculate_unit_on_tile(nmap, act,
                                                   punit, unit_home(punit),
                                                   ptile,
                                                   TRUE,
@@ -201,7 +202,7 @@ static void tai_tile_worker_task_select(struct player *pplayer,
           if (limit == TWTL_BUILDABLE_UNITS) {
             unit_list_iterate(units, punit) {
               if (action_prob_possible(
-                    action_speculate_unit_on_tile(act,
+                    action_speculate_unit_on_tile(nmap, act,
                                                   punit, unit_home(punit),
                                                   ptile,
                                                   TRUE,
@@ -223,9 +224,10 @@ static void tai_tile_worker_task_select(struct player *pplayer,
       if (removing) {
         as_rmextra_action_iterate(try_act) {
           struct action *taction = action_by_number(try_act);
+
           if (is_extra_removed_by_action(tgt, taction)
               && action_prob_possible(
-                action_speculate_unit_on_tile(try_act,
+                action_speculate_unit_on_tile(nmap, try_act,
                                               punit,
                                               unit_home(punit), ptile,
                                               TRUE,
@@ -237,9 +239,10 @@ static void tai_tile_worker_task_select(struct player *pplayer,
       } else {
         as_extra_action_iterate(try_act) {
           struct action *taction = action_by_number(try_act);
+
           if (is_extra_caused_by_action(tgt, taction)
               && action_prob_possible(
-                action_speculate_unit_on_tile(try_act,
+                action_speculate_unit_on_tile(nmap, try_act,
                                               punit,
                                               unit_home(punit), ptile,
                                               TRUE,
@@ -335,7 +338,7 @@ static void tai_tile_worker_task_select(struct player *pplayer,
               fc_assert_action(action_get_target_kind(paction) == ATK_TILE,
                                break);
               if (action_prob_possible(action_speculate_unit_on_tile(
-                                         paction->id,
+                                         nmap, paction->id,
                                          punit, unit_home(punit), ptile,
                                          TRUE,
                                          ptile, tgt))) {
@@ -363,7 +366,7 @@ static void tai_tile_worker_task_select(struct player *pplayer,
               fc_assert_action(action_get_target_kind(paction) == ATK_TILE,
                                break);
               if (action_prob_possible(action_speculate_unit_on_tile(
-                                         paction->id,
+                                         nmap, paction->id,
                                          punit, unit_home(punit), ptile,
                                          TRUE,
                                          ptile, tgt))) {
