@@ -44,7 +44,7 @@
 #include "packhand.h"
 #include "text.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-3.0 */
 #include "citydlg.h"
 #include "dialogs.h"
 #include "unitselextradlg.h"
@@ -583,10 +583,13 @@ static void create_advances_list(struct player *pplayer,
   struct unit *actor_unit = game_unit_by_number(args->actor_unit_id);
 
   spy_tech_shell = gtk_dialog_new_with_buttons(_("Steal Technology"),
-                                               NULL, 0,
-                                               _("_Cancel"), GTK_RESPONSE_CANCEL,
-                                               _("_Steal"), GTK_RESPONSE_ACCEPT,
-                                               NULL);
+    NULL,
+    0,
+    GTK_STOCK_CANCEL,
+    GTK_RESPONSE_CANCEL,
+    _("_Steal"),
+    GTK_RESPONSE_ACCEPT,
+    NULL);
   setup_dialog(spy_tech_shell, toplevel);
   gtk_window_set_position(GTK_WINDOW(spy_tech_shell), GTK_WIN_POS_MOUSE);
 
@@ -771,10 +774,13 @@ static void create_improvements_list(struct player *pplayer,
   struct unit *actor_unit = game_unit_by_number(args->actor_unit_id);
   
   spy_sabotage_shell = gtk_dialog_new_with_buttons(_("Sabotage Improvements"),
-                                                   NULL, 0,
-                                                   _("_Cancel"), GTK_RESPONSE_CANCEL,
-                                                   _("_Sabotage"), GTK_RESPONSE_ACCEPT,
-                                                   NULL);
+    NULL,
+    0,
+    GTK_STOCK_CANCEL,
+    GTK_RESPONSE_CANCEL,
+    _("_Sabotage"), 
+    GTK_RESPONSE_ACCEPT,
+    NULL);
   setup_dialog(spy_sabotage_shell, toplevel);
   gtk_window_set_position(GTK_WINDOW(spy_sabotage_shell), GTK_WIN_POS_MOUSE);
 
@@ -1317,7 +1323,7 @@ static void action_entry_update(GtkWidget *shl,
   const gchar *tooltip;
 
   /* An action that just became impossible has its button disabled.
-   * An action that became possible again must be reenabled. */
+   * An action that became possible again must be re-enabled. */
   choice_dialog_button_set_sensitive(act_sel_dialog,
       action_button_map[act_id],
       action_prob_possible(act_probs[act_id]));
@@ -1552,7 +1558,7 @@ void popup_action_selection(struct unit *actor_unit,
 
   action_button_map[BUTTON_CANCEL] =
       choice_dialog_get_number_of_buttons(shl);
-  choice_dialog_add(shl, _("_Cancel"),
+  choice_dialog_add(shl, GTK_STOCK_CANCEL,
                     (GCallback)act_sel_cancel_callback,
                     GINT_TO_POINTER(ACTION_NONE),
                     FALSE, NULL);
