@@ -590,7 +590,7 @@ static void diplomacy_main_response(struct gui_dialog *dlg, int response,
   case GTK_RESPONSE_DELETE_EVENT:   /* GTK: delete the widget. */
   case RESPONSE_CANCEL_MEETING_ALL: /* Cancel all meetings. */
     dialog_list_iterate(dialog_list, adialog) {
-      /* This will do a round trip to the server ans close the diolag in the
+      /* This will do a round trip to the server ans close the dialog in the
        * client. Closing the last dialog will also close the main tab.*/
       dsend_packet_diplomacy_cancel_meeting_req(&client.conn,
                                                 player_number(
@@ -612,7 +612,6 @@ static void diplomacy_destroy(struct Diplomacy_dialog* pdialog)
     pdialog->dialog = NULL;
   }
   dialog_list_remove(dialog_list, pdialog);
-  free(pdialog);
 
   if (dialog_list) {
     /* Diplomatic meetings in one main tab. */
@@ -629,6 +628,9 @@ static void diplomacy_destroy(struct Diplomacy_dialog* pdialog)
       diplomacy_main_destroy();
     }
   }
+
+  /* Last sub-tab must not be freed before diplomacy_main_destroy() call. */
+  free(pdialog);
 }
 
 /************************************************************************//**
@@ -850,7 +852,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_widget_set_halign(label, GTK_ALIGN_CENTER);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   buf = g_strdup_printf("<span size=\"large\" weight=\"bold\">%s</span>",
-                        ruler_title_for_player(plr1, plr_buf, sizeof(plr_buf)));
+                        title_for_player(plr1, plr_buf, sizeof(plr_buf)));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   g_free(buf);
   gtk_container_add(GTK_CONTAINER(hbox), label);
