@@ -73,7 +73,7 @@
 #include "tilespec.h"
 #include "zoom.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-3.0 */
 #include "chatline.h"
 #include "citizensinfo.h"
 #include "connectdlg.h"
@@ -186,8 +186,6 @@ static bool gui_up = FALSE;
 
 static struct video_mode vmode = { -1, -1 };
 
-static void set_g_log_callbacks(void);
-
 static gboolean show_info_button_release(GtkWidget *w, GdkEventButton *ev, gpointer data);
 static gboolean show_info_popup(GtkWidget *w, GdkEventButton *ev, gpointer data);
 
@@ -246,7 +244,7 @@ static gboolean timer_callback(gpointer data)
 
 /**********************************************************************//**
   Print extra usage information, including one line help on each option,
-  to stderr. 
+  to stderr.
 **************************************************************************/
 static void print_usage(void)
 {
@@ -258,6 +256,10 @@ static void print_usage(void)
   fc_fprintf(stderr,
              _("Other gui-specific options are:\n"));
 
+  fc_fprintf(stderr,
+             _("-g, --gtk-warnings\tAllow Gtk+ to print warnings "
+               "to console\n"));
+
   fc_fprintf(stderr,
              _("-r, --resolution WIDTHxHEIGHT\tAssume given resolution "
                "screen\n"));
@@ -268,12 +270,23 @@ static void print_usage(void)
   fc_fprintf(stderr, _("Report bugs at %s\n"), BUG_URL);
 }
 
+/**********************************************************************//**
+  Dummy gtk error printer
+**************************************************************************/
+static void log_gtk_warns(const gchar *log_domain, GLogLevelFlags log_level,
+                          const gchar *message,
+                          gpointer user_data)
+{
+  log_verbose("%s", message);
+}
+
 /**********************************************************************//**
   Search for gui-specific command-line options.
 **************************************************************************/
 static bool parse_options(int argc, char **argv)
 {
   int i = 1;
+  bool gtk_warns_enabled = FALSE;
 
   while (i < argc) {
     char *option = NULL;
@@ -282,6 +295,8 @@ static bool parse_options(int argc, char **argv)
       print_usage();
 
       return FALSE;
+    } else if (is_option("--gtk-warnings", argv[i])) {
+      gtk_warns_enabled = TRUE;
     } else if ((option = get_option_malloc("--resolution", argv, &i, argc, FALSE))) {
       if (!string_to_video_mode(option, &vmode)) {
         fc_fprintf(stderr, _("Illegal video mode '%s'\n"), option);
@@ -294,6 +309,10 @@ static bool parse_options(int argc, char **argv)
     i++;
   }
 
+  if (!gtk_warns_enabled) {
+    g_log_set_handler("Gtk", G_LOG_LEVEL_WARNING, log_gtk_warns, NULL);
+  }
+
   return TRUE;
 }
 
@@ -723,21 +742,14 @@ static gboolean mouse_scroll_mapcanvas(GtkWidget *w, GdkEventScroll *ev)
 static void tearoff_destroy(GtkWidget *w, gpointer data)
 {
   GtkWidget *p, *b, *box;
-  GtkWidget *old_parent;
 
   box = GTK_WIDGET(data);
-  old_parent = gtk_widget_get_parent(box);
   p = g_object_get_data(G_OBJECT(w), "parent");
   b = g_object_get_data(G_OBJECT(w), "toggle");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), FALSE);
 
   gtk_widget_hide(w);
-
-  g_object_ref(box); /* Make sure reference count stays above 0
-                      * during the transition to new parent. */
-  gtk_container_remove(GTK_CONTAINER(old_parent), box);
-  gtk_container_add(GTK_CONTAINER(p), box);
-  g_object_unref(box);
+  gtk_widget_reparent(box, p);
 }
 
 /**********************************************************************//**
@@ -757,13 +769,11 @@ static gboolean propagate_keypress(GtkWidget *w, GdkEvent *ev)
 static void tearoff_callback(GtkWidget *b, gpointer data)
 {
   GtkWidget *box = GTK_WIDGET(data);
+  GtkWidget *w;
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b))) {
-    GtkWidget *old_parent;
-    GtkWidget *w;
     GtkWidget *temp_hide;
 
-    old_parent = gtk_widget_get_parent(box);
     w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     setup_dialog(w, toplevel);
     gtk_widget_set_name(w, "Freeciv");
@@ -775,19 +785,12 @@ static void tearoff_callback(GtkWidget *b, gpointer data)
 
     g_object_set_data(G_OBJECT(w), "parent", gtk_widget_get_parent(box));
     g_object_set_data(G_OBJECT(w), "toggle", b);
-
     temp_hide = g_object_get_data(G_OBJECT(box), "hide-over-reparent");
     if (temp_hide != NULL) {
       gtk_widget_hide(temp_hide);
     }
-
-    g_object_ref(box); /* Make sure reference count stays above 0
-                        * during the transition to new parent. */
-    gtk_container_remove(GTK_CONTAINER(old_parent), box);
-    gtk_container_add(GTK_CONTAINER(w), box);
-    g_object_unref(box);
+    gtk_widget_reparent(box, w);
     gtk_widget_show(w);
-
     if (temp_hide != NULL) {
       gtk_widget_show(temp_hide);
     }
@@ -802,6 +805,7 @@ static void tearoff_callback(GtkWidget *b, gpointer data)
 static GtkWidget *detached_widget_new(void)
 {
   GtkWidget *hgrid = gtk_grid_new();
+
   gtk_grid_set_column_spacing(GTK_GRID(hgrid), 2);
   return hgrid;
 }
@@ -813,30 +817,11 @@ static GtkWidget *detached_widget_new(void)
 static GtkWidget *detached_widget_fill(GtkWidget *tearbox)
 {
   GtkWidget *b, *fillbox;
-  static GtkCssProvider *detach_button_provider = NULL;
-
-  if (detach_button_provider == NULL) {
-    detach_button_provider = gtk_css_provider_new();
-
-    /* These toggle buttons run vertically down the side of many UI
-     * elements, so they need to be thin horizontally. */
-    gtk_css_provider_load_from_data(detach_button_provider,
-                                    ".detach_button {\n"
-                                    "  padding: 0px 0px 0px 0px;\n"
-                                    "  min-width: 6px;\n"
-                                    "}",
-                                    -1, NULL);
-  }
 
   b = gtk_toggle_button_new();
-  gtk_style_context_add_provider(gtk_widget_get_style_context(b),
-                                 GTK_STYLE_PROVIDER(detach_button_provider),
-                                 GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-  gtk_style_context_add_class(gtk_widget_get_style_context(b),
-                              "detach_button");
-
   gtk_container_add(GTK_CONTAINER(tearbox), b);
   g_signal_connect(b, "toggled", G_CALLBACK(tearoff_callback), tearbox);
+  gtk_widget_set_tooltip_text(b, _("Detach/Attach the pane."));
 
   fillbox = gtk_grid_new();
   gtk_orientable_set_orientation(GTK_ORIENTABLE(fillbox),
@@ -932,10 +917,7 @@ static void populate_unit_image_table(void)
                    G_CALLBACK(select_more_arrow_pixmap_callback), NULL);
   /* An extra layer so that we can hide the clickable button but keep
    * an explicit size request to avoid the layout jumping around */
-  more_arrow_pixmap_container = gtk_frame_new(NULL);
-  gtk_frame_set_shadow_type(GTK_FRAME(more_arrow_pixmap_container), GTK_SHADOW_NONE);
-  gtk_widget_set_halign(more_arrow_pixmap_container, GTK_ALIGN_CENTER);
-  gtk_widget_set_valign(more_arrow_pixmap_container, GTK_ALIGN_CENTER);
+  more_arrow_pixmap_container = gtk_alignment_new(0.5, 0.5, 0, 0);
   g_object_ref(more_arrow_pixmap_container);
   gtk_container_add(GTK_CONTAINER(more_arrow_pixmap_container),
                     more_arrow_pixmap_button);
@@ -1059,7 +1041,6 @@ static gboolean right_notebook_button_release(GtkWidget *widget,
 /**********************************************************************//**
   Override background color for canvases
 **************************************************************************/
-#if 0
 static void setup_canvas_color_for_state(GtkStateFlags state)
 {
   gtk_widget_override_background_color(GTK_WIDGET(overview_canvas), state,
@@ -1067,7 +1048,6 @@ static void setup_canvas_color_for_state(GtkStateFlags state)
   gtk_widget_override_background_color(GTK_WIDGET(map_canvas), state,
                                        &get_color(tileset, COLOR_OVERVIEW_UNKNOWN)->color);
 }
-#endif
 
 /**********************************************************************//**
   Update tooltip of the Turn Done button
@@ -1097,9 +1077,9 @@ static void setup_widgets(void)
   GtkWidget *frame, *table, *table2, *paned, *sw, *text;
   GtkWidget *button, *view, *vgrid, *right_vbox = NULL;
   int i;
+  struct sprite *spr;
   GtkWidget *notebook, *statusbar;
   GtkWidget *dtach_lowbox = NULL;
-  struct sprite *spr;
 
   message_buffer = gtk_text_buffer_new(NULL);
 
@@ -1182,7 +1162,7 @@ static void setup_widgets(void)
     overview_canvas_store_height = OVERVIEW_CANVAS_STORE_HEIGHT;
   }
 
-  /* this holds the overview canvas, production info, etc. */
+  /* This holds the overview canvas, production info, etc. */
   vgrid = gtk_grid_new();
   gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid),
                                  GTK_ORIENTATION_VERTICAL);
@@ -1202,15 +1182,15 @@ static void setup_widgets(void)
   overview_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_container_set_border_width(GTK_CONTAINER (overview_scrolled_window), 1);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (overview_scrolled_window),
-                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   overview_canvas = gtk_drawing_area_new();
   gtk_widget_set_halign(overview_canvas, GTK_ALIGN_CENTER);
   gtk_widget_set_valign(overview_canvas, GTK_ALIGN_CENTER);
   gtk_widget_set_size_request(overview_canvas, overview_canvas_store_width,
-		              overview_canvas_store_height);
+                              overview_canvas_store_height);
   gtk_widget_set_size_request(overview_scrolled_window, overview_canvas_store_width,
-		              overview_canvas_store_height);
+                              overview_canvas_store_height);
   gtk_widget_set_hexpand(overview_canvas, TRUE);
   gtk_widget_set_vexpand(overview_canvas, TRUE);
 
@@ -1223,13 +1203,13 @@ static void setup_widgets(void)
                     overview_canvas);
  
   g_signal_connect(overview_canvas, "draw",
-        	   G_CALLBACK(overview_canvas_draw), NULL);
+                   G_CALLBACK(overview_canvas_draw), NULL);
 
   g_signal_connect(overview_canvas, "motion_notify_event",
-        	   G_CALLBACK(move_overviewcanvas), NULL);
+                   G_CALLBACK(move_overviewcanvas), NULL);
 
   g_signal_connect(overview_canvas, "button_press_event",
-        	   G_CALLBACK(butt_down_overviewcanvas), NULL);
+                   G_CALLBACK(butt_down_overviewcanvas), NULL);
 
   /* The rest */
 
@@ -1262,8 +1242,8 @@ static void setup_widgets(void)
   label = gtk_label_new(NULL);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_widget_set_margin_start(label, 2);
-  gtk_widget_set_margin_end(label, 2);
+  gtk_widget_set_margin_left(label, 2);
+  gtk_widget_set_margin_right(label, 2);
   gtk_widget_set_margin_top(label, 2);
   gtk_widget_set_margin_bottom(label, 2);
   gtk_container_add(GTK_CONTAINER(ebox), label);
@@ -1280,10 +1260,10 @@ static void setup_widgets(void)
   gtk_event_box_set_visible_window(GTK_EVENT_BOX(ebox), FALSE);
   gtk_grid_attach(GTK_GRID(table), ebox, 0, 0, 10, 1);
   econ_ebox = ebox;
-
+  
   table2 = gtk_grid_new();
   gtk_container_add(GTK_CONTAINER(ebox), table2);
-
+  
   for (i = 0; i < 10; i++) {
     ebox = gtk_event_box_new();
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(ebox), FALSE);
@@ -1303,34 +1283,23 @@ static void setup_widgets(void)
   spr = client_research_sprite();
   if (spr != NULL) {
     bulb_label = gtk_image_new_from_surface(spr->surface);
-  } else {
-    bulb_label = gtk_image_new();
   }
-
   spr = client_warming_sprite();
   if (spr != NULL) {
     sun_label = gtk_image_new_from_surface(spr->surface);
-  } else {
-    sun_label = gtk_image_new();
   }
-
   spr = client_cooling_sprite();
   if (spr != NULL) {
     flake_label = gtk_image_new_from_surface(spr->surface);
-  } else {
-    flake_label = gtk_image_new();
   }
-
   spr = client_government_sprite();
   if (spr != NULL) {
     government_label = gtk_image_new_from_surface(spr->surface);
-  } else {
-    government_label = gtk_image_new();
   }
 
   for (i = 0; i < 4; i++) {
     GtkWidget *w;
-
+    
     ebox = gtk_event_box_new();
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(ebox), FALSE);
 
@@ -1356,8 +1325,8 @@ static void setup_widgets(void)
 
     gtk_widget_set_halign(w, GTK_ALIGN_START);
     gtk_widget_set_valign(w, GTK_ALIGN_START);
-    gtk_widget_set_margin_start(w, 0);
-    gtk_widget_set_margin_end(w, 0);
+    gtk_widget_set_margin_left(w, 0);
+    gtk_widget_set_margin_right(w, 0);
     gtk_widget_set_margin_top(w, 0);
     gtk_widget_set_margin_bottom(w, 0);
     gtk_container_add(GTK_CONTAINER(ebox), w);
@@ -1378,6 +1347,7 @@ static void setup_widgets(void)
 
   g_signal_connect(turn_done_button, "clicked",
                    G_CALLBACK(end_turn_callback), NULL);
+
   update_turn_done_tooltip();
 
   /* Selected unit status */
@@ -1408,8 +1378,8 @@ static void setup_widgets(void)
   gtk_widget_set_hexpand(label, TRUE);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_widget_set_margin_start(label, 2);
-  gtk_widget_set_margin_end(label, 2);
+  gtk_widget_set_margin_left(label, 2);
+  gtk_widget_set_margin_right(label, 2);
   gtk_widget_set_margin_top(label, 2);
   gtk_widget_set_margin_bottom(label, 2);
   gtk_container_add(GTK_CONTAINER(sw), label);
@@ -1472,7 +1442,6 @@ static void setup_widgets(void)
   gtk_widget_set_size_request(map_canvas, 300, 300);
   gtk_widget_set_can_focus(map_canvas, TRUE);
 
-#if 0
   setup_canvas_color_for_state(GTK_STATE_FLAG_NORMAL);
   setup_canvas_color_for_state(GTK_STATE_FLAG_ACTIVE);
   setup_canvas_color_for_state(GTK_STATE_FLAG_PRELIGHT);
@@ -1481,7 +1450,6 @@ static void setup_widgets(void)
   setup_canvas_color_for_state(GTK_STATE_FLAG_INCONSISTENT);
   setup_canvas_color_for_state(GTK_STATE_FLAG_FOCUSED);
   setup_canvas_color_for_state(GTK_STATE_FLAG_BACKDROP);
-#endif /* 0 */
 
   gtk_widget_add_events(map_canvas, GDK_EXPOSURE_MASK
                                    |GDK_BUTTON_PRESS_MASK
@@ -1614,7 +1582,7 @@ static void setup_widgets(void)
   g_object_set(view, "margin", 3, NULL);
 
   button = gtk_check_button_new_with_label(_("Allies Only"));
-  gtk_widget_set_focus_on_click(button, FALSE);
+  gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
                                GUI_GTK_OPTION(allied_chat_only));
   g_signal_connect(button, "toggled",
@@ -1644,85 +1612,12 @@ static void setup_widgets(void)
   }
 }
 
-/**********************************************************************//**
-  g_log callback to log with freelog
-**************************************************************************/
-static void g_log_to_freelog_cb(const gchar *log_domain,
-                                GLogLevelFlags log_level,
-                                const gchar *message,
-                                gpointer user_data)
-{
-  enum log_level fllvl = LOG_ERROR;
-
-  switch (log_level) {
-  case G_LOG_LEVEL_DEBUG:
-    fllvl = LOG_DEBUG;
-    break;
-  case G_LOG_LEVEL_WARNING:
-    fllvl = LOG_WARN;
-    break;
-  default:
-    break;
-  }
-
-  if (log_domain != NULL) {
-    log_base(fllvl, "%s: %s", log_domain, message);
-  } else {
-    log_base(fllvl, "%s", message);
-  }
-}
-
-/**********************************************************************//**
-  g_log callback to log with freelog
-**************************************************************************/
-static GLogWriterOutput g_log_writer_to_freelog_cb(GLogLevelFlags log_level,
-                                                   const GLogField *fields,
-                                                   gsize n_fields,
-                                                   gpointer user_data)
-{
-  /* No need to have formatter of our own - let's use glib's default one. */
-  gchar *out = g_log_writer_format_fields(log_level, fields, n_fields, FALSE);
-
-  g_log_to_freelog_cb(NULL, log_level, out, NULL);
-
-  return G_LOG_WRITER_HANDLED;
-}
-
-/**********************************************************************//**
-  Set up g_log callback for a single domain.
-**************************************************************************/
-static void set_g_log_callback_domain(const char *domain)
-{
-  g_log_set_handler(domain,
-                    G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_LEVEL_WARNING
-                    | G_LOG_LEVEL_MASK
-                    | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                    g_log_to_freelog_cb, NULL);
-}
-
-/**********************************************************************//**
-  Set up g_log callbacks.
-**************************************************************************/
-static void set_g_log_callbacks(void)
-{
-  /* Old API, still used by many log producers */
-  g_log_set_default_handler(g_log_to_freelog_cb, NULL);
-
-  set_g_log_callback_domain("Gtk");
-  set_g_log_callback_domain("Gdk");
-  set_g_log_callback_domain("Glib");
-
-  /* glib >= 2.50 API */
-  g_log_set_writer_func(g_log_writer_to_freelog_cb, NULL, NULL);
-}
-
 /**********************************************************************//**
   Called from main().
 **************************************************************************/
 void ui_init(void)
 {
   log_set_callback(log_callback_utf8);
-  set_g_log_callbacks();
   set_frame_by_frame_animation();
 }
 
@@ -1741,10 +1636,10 @@ static void migrate_options_from_gtk2(void)
 {
   log_normal(_("Migrating options from gtk2 to gtk3 client"));
 
-#define MIGRATE_OPTION(opt) gui_options.gui_gtk3_##opt = gui_options.gui_gtk2_##opt;
+#define MIGRATE_OPTION(opt) GUI_GTK_OPTION(opt) = gui_options.gui_gtk2_##opt;
 #define MIGRATE_STR_OPTION(opt) \
-  strncpy(gui_options.gui_gtk3_##opt, gui_options.gui_gtk2_##opt,      \
-          sizeof(gui_options.gui_gtk3_##opt));
+  strncpy(GUI_GTK_OPTION(opt), gui_options.gui_gtk2_##opt,      \
+          sizeof(GUI_GTK_OPTION(opt)));
 
   /* Default theme name is never migrated */
   /* 'fullscreen', 'small_display_layout', and 'message_chat_location'
@@ -1786,66 +1681,19 @@ static void migrate_options_from_gtk2(void)
 #undef MIGRATE_OPTION
 #undef MIGRATE_STR_OPTION
 
-  gui_options.gui_gtk3_migrated_from_gtk2 = TRUE;
+  GUI_GTK_OPTION(migrated_from_gtk2) = TRUE;
 }
 
 /**********************************************************************//**
-  Migrate gtk3.22 client specific options from gtk3 client options.
+  Migrate gtk3 client specific options from freeciv-2.5 options
 **************************************************************************/
-static void migrate_options_from_gtk3(void)
+static void migrate_options_from_2_5(void)
 {
-  log_normal(_("Migrating options from gtk3 to gtk3.22 client"));
+  log_normal(_("Migrating gtk3-client options from freeciv-2.5 options."));
 
-#define MIGRATE_OPTION(opt) GUI_GTK_OPTION(opt) = gui_options.gui_gtk3_##opt;
-#define MIGRATE_STR_OPTION(opt) \
-  strncpy(GUI_GTK_OPTION(opt), gui_options.gui_gtk3_##opt,      \
-          sizeof(GUI_GTK_OPTION(opt)));
+  GUI_GTK_OPTION(fullscreen) = gui_options.migrate_fullscreen;
 
-  /* Default theme name is never migrated */
-
-  /* Simulate gui-gtk3's migrate_options_from_2_5() */
-  if (!gui_options.gui_gtk3_migrated_from_2_5) {
-    log_normal(_("Migrating gtk3-client options from freeciv-2.5 options."));
-    gui_options.gui_gtk3_fullscreen = gui_options.migrate_fullscreen;
-    gui_options.gui_gtk3_migrated_from_2_5 = TRUE;
-  }
-
-  MIGRATE_OPTION(fullscreen);
-  MIGRATE_OPTION(map_scrollbars);
-  MIGRATE_OPTION(dialogs_on_top);
-  MIGRATE_OPTION(show_task_icons);
-  MIGRATE_OPTION(enable_tabs);
-  MIGRATE_OPTION(show_chat_message_time);
-  MIGRATE_OPTION(new_messages_go_to_top);
-  MIGRATE_OPTION(show_message_window_buttons);
-  MIGRATE_OPTION(metaserver_tab_first);
-  MIGRATE_OPTION(allied_chat_only);
-  MIGRATE_OPTION(message_chat_location);
-  MIGRATE_OPTION(small_display_layout);
-  MIGRATE_OPTION(mouse_over_map_focus);
-  MIGRATE_OPTION(chatline_autocompletion);
-  MIGRATE_OPTION(citydlg_xsize);
-  MIGRATE_OPTION(citydlg_ysize);
-  MIGRATE_OPTION(popup_tech_help);
-
-  MIGRATE_STR_OPTION(font_city_label);
-  MIGRATE_STR_OPTION(font_notify_label);
-  MIGRATE_STR_OPTION(font_spaceship_label);
-  MIGRATE_STR_OPTION(font_help_label);
-  MIGRATE_STR_OPTION(font_help_link);
-  MIGRATE_STR_OPTION(font_help_text);
-  MIGRATE_STR_OPTION(font_chatline);
-  MIGRATE_STR_OPTION(font_beta_label);
-  MIGRATE_STR_OPTION(font_small);
-  MIGRATE_STR_OPTION(font_comment_label);
-  MIGRATE_STR_OPTION(font_city_names);
-  MIGRATE_STR_OPTION(font_city_productions);
-  MIGRATE_STR_OPTION(font_reqtree_text);
-
-#undef MIGRATE_OPTION
-#undef MIGRATE_STR_OPTION
-
-  GUI_GTK_OPTION(migrated_from_gtk3) = TRUE;
+  GUI_GTK_OPTION(migrated_from_2_5) = TRUE;
 }
 
 /**********************************************************************//**
@@ -1871,8 +1719,6 @@ int ui_main(int argc, char **argv)
 
     help_system_init();
 
-    dlg_tab_provider_prepare();
-
     toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_position(GTK_WINDOW(toplevel), GTK_WIN_POS_CENTER);
     if (vmode.width > 0 && vmode.height > 0) {
@@ -1890,21 +1736,19 @@ int ui_main(int argc, char **argv)
       adjust_default_options();
       /* We're using fresh defaults for this version of this client,
        * so prevent any future migrations from other clients / versions */
-      GUI_GTK_OPTION(migrated_from_gtk3) = TRUE;
-      /* Avoid also marking gtk3 as migrated, so that it can have its own
-       * run of its adjust_default_options() if it is ever run (as a
-       * side effect of Gtk2->Gtk3 migration). */
+      GUI_GTK_OPTION(migrated_from_gtk2) = TRUE;
+      GUI_GTK_OPTION(migrated_from_2_5) = TRUE;
     } else {
-      if (!GUI_GTK_OPTION(migrated_from_gtk3)) {
-        if (!gui_options.gui_gtk3_migrated_from_gtk2) {
-          migrate_options_from_gtk2();
-          /* We want a fresh look at screen-size-related options after Gtk2 */
-          adjust_default_options();
-          /* We don't ever want to consider pre-2.6 fullscreen option again
-           * (even for gui-gtk3) */
-          gui_options.gui_gtk3_migrated_from_2_5 = TRUE;
-        }
-        migrate_options_from_gtk3();
+      if (!GUI_GTK_OPTION(migrated_from_gtk2)) {
+        migrate_options_from_gtk2();
+        /* We want a fresh look at screen-size-related defaults */
+        adjust_default_options();
+        /* We don't ever want to consider pre-2.6 fullscreen option again */
+        GUI_GTK_OPTION(migrated_from_2_5) = TRUE;
+      } else if (!GUI_GTK_OPTION(migrated_from_2_5)) {
+        /* This only affects the fullscreen option, which we don't want
+         * to touch if adjust_default_options() just adjusted it. */
+        migrate_options_from_2_5();
       }
     }
 
@@ -2038,11 +1882,11 @@ void ui_exit(void)
 **************************************************************************/
 enum gui_type get_gui_type(void)
 {
-  return GUI_GTK3_22;
+  return GUI_GTK3;
 }
 
 /**********************************************************************//**
- obvious...
+  Obvious...
 **************************************************************************/
 void sound_bell(void)
 {
@@ -2117,7 +1961,7 @@ void real_focus_units_changed(void)
   Callback for clicking a unit icon underneath unit info box.
   these are the units on the same tile as the focus unit.
 **************************************************************************/
-static gboolean select_unit_image_callback(GtkWidget *w, GdkEvent *ev,
+static gboolean select_unit_image_callback(GtkWidget *w, GdkEvent *ev, 
                                            gpointer data)
 {
   int i = GPOINTER_TO_INT(data);
@@ -2163,13 +2007,11 @@ static gboolean select_more_arrow_pixmap_callback(GtkWidget *w, GdkEvent *ev,
 /**********************************************************************//**
   Button released when showing info popup
 **************************************************************************/
-static gboolean show_info_button_release(GtkWidget *w, GdkEventButton *ev,
-                                         gpointer data)
+static gboolean show_info_button_release(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
   gtk_grab_remove(w);
-  gdk_seat_ungrab(gdk_device_get_seat(ev->device));
+  gdk_device_ungrab(ev->device, ev->time);
   gtk_widget_destroy(w);
-
   return FALSE;
 }
 
@@ -2192,9 +2034,9 @@ static gboolean show_info_popup(GtkWidget *w, GdkEventButton *ev, gpointer data)
                    NULL);
     gtk_widget_show(p);
 
-    gdk_seat_grab(gdk_device_get_seat(ev->device), gtk_widget_get_window(p),
-                  GDK_SEAT_CAPABILITY_ALL_POINTING,
-                  TRUE, NULL, (GdkEvent *)ev, NULL, NULL);
+    gdk_device_grab(ev->device, gtk_widget_get_window(p),
+                    GDK_OWNERSHIP_NONE, TRUE, GDK_BUTTON_RELEASE_MASK, NULL,
+                    ev->time);
     gtk_grab_add(p);
 
     g_signal_connect_after(p, "button_release_event",
@@ -2234,8 +2076,9 @@ static void set_wait_for_writable_socket(struct connection *pc,
 
   fc_assert_ret(pc == &client.conn);
 
-  if (previous_state == socket_writable)
+  if (previous_state == socket_writable) {
     return;
+  }
 
   log_debug("set_wait_for_writable_socket(%d)", socket_writable);
 
@@ -2435,12 +2278,11 @@ static void apply_reqtree_text_font(struct option *poption)
 }
 
 /**********************************************************************//**
-  Extra initializers for client options. Here we make set the callback
-  for the specific gui-gtk-3.22 options.
+  Extra initializers for client options.  Here we make set the callback
+  for the specific gui-gtk-3.0 options.
 **************************************************************************/
 void options_extra_init(void)
 {
-
   struct option *poption;
 
 #define option_var_set_callback(var, callback)                              \
@@ -2509,62 +2351,43 @@ void insert_client_build_info(char *outbuf, size_t outlen)
 }
 
 /**********************************************************************//**
-  Return dimensions of primary monitor, if any
-  (in 'application pixels')
-**************************************************************************/
-static bool monitor_size(GdkRectangle *rect_p)
-{
-  GdkDisplay *display;
-  GdkMonitor *monitor;
-
-  display = gdk_display_get_default();
-  if (!display) {
-    return FALSE;
-  }
-
-  monitor = gdk_display_get_primary_monitor(display);
-  if (!monitor) {
-    return FALSE;
-  }
-
-  gdk_monitor_get_geometry(monitor, rect_p);
-  return TRUE;
-}
-
-/**********************************************************************//**
-  Return width of the primary monitor
+  Return width of the default screen
 **************************************************************************/
 int screen_width(void)
 {
-  GdkRectangle rect;
+  GdkScreen *screen;
 
   if (vmode.width > 0) {
     return vmode.width;
   }
 
-  if (monitor_size(&rect)) {
-    return rect.width;
-  } else {
+  screen = gdk_screen_get_default();
+
+  if (screen == NULL) {
     return 0;
   }
+
+  return gdk_screen_get_width(screen);
 }
 
 /**********************************************************************//**
-  Return height of the primary monitor
+  Return height of the default screen
 **************************************************************************/
 int screen_height(void)
 {
-  GdkRectangle rect;
+  GdkScreen *screen;
 
   if (vmode.height > 0) {
     return vmode.height;
   }
 
-  if (monitor_size(&rect)) {
-    return rect.height;
-  } else {
+  screen = gdk_screen_get_default();
+
+  if (screen == NULL) {
     return 0;
   }
+
+  return gdk_screen_get_height(screen);
 }
 
 /**********************************************************************//**
