@@ -43,7 +43,7 @@
 #include "mapview_common.h"
 #include "tilespec.h"
 
-/* client/gui-gtk-4.0 */
+/* client/gui-gtk-3.22 */
 #include "canvas.h"
 #include "editgui.h"
 #include "editprop.h"
@@ -182,15 +182,14 @@ static void try_to_set_editor_tool(enum editor_tool_type ett)
 
   if (!editor_tool_is_usable(ett)) {
     GtkWidget *dialog;
-
     dialog = gtk_message_dialog_new(GTK_WINDOW(toplevel),
         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
         GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s",
         _("The current ruleset does not define any "
           "objects corresponding to this editor tool."));
     gtk_window_set_title(GTK_WINDOW(dialog), editor_tool_get_name(ett));
-    blocking_dialog(dialog);
-    gtk_window_destroy(GTK_WINDOW(dialog));
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
   } else {
     editor_set_tool(ett);
   }
@@ -331,7 +330,7 @@ static int tool_value_selector_run(struct tool_value_selector *tvs)
   }
 
   dialog = tvs->dialog;
-  res = blocking_dialog(dialog);
+  res = gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_hide(dialog);
 
   if (res != GTK_RESPONSE_ACCEPT) {
@@ -387,11 +386,18 @@ static bool editgui_run_tool_selection(enum editor_tool_type ett)
 /************************************************************************//**
   Handle a mouse click on any of the tool buttons.
 ****************************************************************************/
-static gboolean editbar_tool_right_button(GtkGestureClick *gesture,
-                                          int n_press,
-                                          double x, double y, gpointer data)
+static gboolean editbar_tool_button_mouse_click(GtkWidget *w,
+                                                GdkEventButton *ev,
+                                                gpointer userdata)
 {
-  return editgui_run_tool_selection(GPOINTER_TO_INT(data));
+  int ett;
+
+  if (ev->button != 3) {
+    return FALSE; /* Propagate event further. */
+  }
+
+  ett = GPOINTER_TO_INT(userdata);
+  return editgui_run_tool_selection(ett);
 }
 
 /************************************************************************//**
@@ -402,12 +408,10 @@ static void editbar_add_tool_button(struct editbar *eb,
                                     enum editor_tool_type ett)
 {
   GdkPixbuf *pixbuf;
-  GtkWidget *pic, *button, *hbox;
-  GtkToggleButton *parent = NULL;
+  GtkWidget *image, *button, *hbox;
+  GtkRadioButton *parent = NULL;
   struct sprite *sprite;
   int i;
-  GtkGesture *gesture;
-  GtkEventController *controller;
 
   if (!eb || !(ett < NUM_EDITOR_TOOL_TYPES)) {
     return;
@@ -415,46 +419,37 @@ static void editbar_add_tool_button(struct editbar *eb,
 
   for (i = 0; i < NUM_EDITOR_TOOL_TYPES; i++) {
     if (eb->tool_buttons[i] != NULL) {
-      parent = GTK_TOGGLE_BUTTON(eb->tool_buttons[i]);
+      parent = GTK_RADIO_BUTTON(eb->tool_buttons[i]);
       break;
     }
   }
 
   if (parent == NULL) {
-    button = gtk_toggle_button_new();
+    button = gtk_radio_button_new(NULL);
   } else {
-    button = gtk_toggle_button_new();
-    gtk_toggle_button_set_group(GTK_TOGGLE_BUTTON(button),
-                                GTK_TOGGLE_BUTTON(parent));
+    button = gtk_radio_button_new_from_widget(parent);
   }
 
   sprite = editor_tool_get_sprite(ett);
   fc_assert_ret(sprite != NULL);
   pixbuf = sprite_get_pixbuf(sprite);
-  pic = gtk_picture_new_for_pixbuf(pixbuf);
+  image = gtk_image_new_from_pixbuf(pixbuf);
   g_object_unref(G_OBJECT(pixbuf));
 
-  gtk_button_set_child(GTK_BUTTON(button), pic);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
+  gtk_container_add(GTK_CONTAINER(button), image);
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button), FALSE);
   gtk_widget_set_tooltip_text(button, editor_tool_get_tooltip(ett));
   gtk_size_group_add_widget(eb->size_group, button);
-  gtk_button_set_has_frame(GTK_BUTTON(button), FALSE);
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_widget_set_focus_on_click(button, FALSE);
 
   g_signal_connect(button, "toggled",
-                   G_CALLBACK(editbar_tool_button_toggled),
-                   GINT_TO_POINTER(ett));
-
-  gesture = gtk_gesture_click_new();
-  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 3);
-  controller = GTK_EVENT_CONTROLLER(gesture);
-  g_signal_connect(controller, "pressed",
-                   G_CALLBACK(editbar_tool_right_button),
-                   GINT_TO_POINTER(ett));
-  gtk_widget_add_controller(button, controller);
+      G_CALLBACK(editbar_tool_button_toggled), GINT_TO_POINTER(ett));
+  g_signal_connect(button, "button_press_event",
+      G_CALLBACK(editbar_tool_button_mouse_click), GINT_TO_POINTER(ett));
 
   hbox = eb->widget;
-  gtk_box_append(GTK_BOX(hbox), button);
+  gtk_container_add(GTK_CONTAINER(hbox), button);
   eb->tool_buttons[ett] = button;
 
   if (editor_tool_has_value(ett)) {
@@ -484,7 +479,7 @@ static void editbar_add_mode_button(struct editbar *eb,
                                     enum editor_tool_mode etm)
 {
   GdkPixbuf *pixbuf;
-  GtkWidget *pic, *button, *hbox;
+  GtkWidget *image, *button, *hbox;
   struct sprite *sprite;
   const char *tooltip;
 
@@ -497,25 +492,24 @@ static void editbar_add_mode_button(struct editbar *eb,
   sprite = editor_get_mode_sprite(etm);
   fc_assert_ret(sprite != NULL);
   pixbuf = sprite_get_pixbuf(sprite);
-  pic = gtk_picture_new_for_pixbuf(pixbuf);
+  image = gtk_image_new_from_pixbuf(pixbuf);
   g_object_unref(G_OBJECT(pixbuf));
 
-  gtk_button_set_child(GTK_BUTTON(button), pic);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
+  gtk_container_add(GTK_CONTAINER(button), image);
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button), FALSE);
   tooltip = editor_get_mode_tooltip(etm);
   if (tooltip != NULL) {
     gtk_widget_set_tooltip_text(button, tooltip);
   }
   gtk_size_group_add_widget(eb->size_group, button);
-  gtk_button_set_has_frame(GTK_BUTTON(button), FALSE);
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_widget_set_focus_on_click(button, FALSE);
 
   g_signal_connect(button, "toggled",
-                   G_CALLBACK(editbar_mode_button_toggled),
-                   GINT_TO_POINTER(etm));
+      G_CALLBACK(editbar_mode_button_toggled), GINT_TO_POINTER(etm));
 
   hbox = eb->widget;
-  gtk_box_append(GTK_BOX(hbox), button);
+  gtk_container_add(GTK_CONTAINER(hbox), button);
   eb->mode_buttons[etm] = button;
 }
 
@@ -525,16 +519,16 @@ static void editbar_add_mode_button(struct editbar *eb,
 static struct editbar *editbar_create(void)
 {
   struct editbar *eb;
-  GtkWidget *hbox, *button, *combo, *pic, *separator, *vgrid;
+  GtkWidget *hbox, *button, *combo, *image, *separator, *vbox;
   GtkListStore *store;
   GtkCellRenderer *cell;
   GdkPixbuf *pixbuf;
   const struct editor_sprites *sprites;
-  int grid_row = 0;
-
+  
   eb = fc_calloc(1, sizeof(struct editbar));
 
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+  hbox = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hbox), 4);
   eb->widget = hbox;
   eb->size_group = gtk_size_group_new(GTK_SIZE_GROUP_BOTH);
 
@@ -543,7 +537,7 @@ static struct editbar *editbar_create(void)
   editbar_add_mode_button(eb, ETM_ERASE);
 
   separator = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
-  gtk_box_append(GTK_BOX(hbox), separator);
+  gtk_container_add(GTK_CONTAINER(hbox), separator);
 
   editbar_add_tool_button(eb, ETT_TERRAIN);
   editbar_add_tool_button(eb, ETT_TERRAIN_RESOURCE);
@@ -557,13 +551,13 @@ static struct editbar *editbar_create(void)
   editbar_add_tool_button(eb, ETT_COPYPASTE);
 
   separator = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
-  gtk_box_append(GTK_BOX(hbox), separator);
+  gtk_container_add(GTK_CONTAINER(hbox), separator);
 
   /* Player POV indicator. */
-  vgrid = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid),
+  vbox = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
                                  GTK_ORIENTATION_VERTICAL);
-  gtk_box_append(GTK_BOX(hbox), vgrid);
+  gtk_container_add(GTK_CONTAINER(hbox), vbox);
 
   store = gtk_list_store_new(PPV_NUM_COLS,
                              GDK_TYPE_PIXBUF,
@@ -591,22 +585,22 @@ static struct editbar *editbar_create(void)
       _("Switch player point-of-view. Use this to edit "
         "from the perspective of different players, or "
         "even as a global observer."));
-  gtk_grid_attach(GTK_GRID(vgrid), combo, 0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox), combo);
   eb->player_pov_combobox = combo;
 
   /* Property editor button. */
   button = gtk_button_new();
   pixbuf = sprite_get_pixbuf(sprites->properties);
-  pic = gtk_picture_new_for_pixbuf(pixbuf);
+  image = gtk_image_new_from_pixbuf(pixbuf);
   g_object_unref(G_OBJECT(pixbuf));
-  gtk_button_set_child(GTK_BUTTON(button), pic);
+  gtk_container_add(GTK_CONTAINER(button), image);
   gtk_widget_set_tooltip_text(button, _("Show the property editor."));
   gtk_size_group_add_widget(eb->size_group, button);
-  gtk_button_set_has_frame(GTK_BUTTON(button), FALSE);
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_widget_set_focus_on_click(button, FALSE);
   g_signal_connect(button, "clicked",
                    G_CALLBACK(editbar_player_properties_button_clicked), eb);
-  gtk_box_append(GTK_BOX(hbox), button);
+  gtk_container_add(GTK_CONTAINER(hbox), button);
 
   return eb;
 }
@@ -687,7 +681,7 @@ static void editbar_refresh(struct editbar *eb)
   refresh_all_tool_value_selectors(eb);
   refresh_player_pov_indicator(eb);
 
-  gtk_widget_show(eb->widget);
+  gtk_widget_show_all(eb->widget);
 }
 
 /************************************************************************//**
@@ -910,26 +904,47 @@ static int convert_modifiers(int gdk_event_state)
   if (gdk_event_state & GDK_CONTROL_MASK) {
     modifiers |= EKM_CTRL;
   }
-  if (gdk_event_state & GDK_ALT_MASK) {
+  if (gdk_event_state & GDK_MOD1_MASK) {
     modifiers |= EKM_ALT;
   }
 
   return modifiers;
 }
 
+/************************************************************************//**
+  Convert gdk mouse button values to editor mouse button values.
+****************************************************************************/
+static int convert_mouse_button(int gdk_mouse_button)
+{
+  switch (gdk_mouse_button) {
+  case 1:
+    return MOUSE_BUTTON_LEFT;
+    break;
+  case 2:
+    return MOUSE_BUTTON_MIDDLE;
+    break;
+  case 3:
+    return MOUSE_BUTTON_RIGHT;
+    break;
+  default:
+    break;
+  }
+
+  return MOUSE_BUTTON_OTHER;
+}
+
 /************************************************************************//**
   Pass on the gdk mouse event to the editor's handler.
 ****************************************************************************/
-gboolean handle_edit_mouse_button_press(GtkGestureClick *gesture,
-                                        int editor_mouse_button,
-                                        double x, double y)
+gboolean handle_edit_mouse_button_press(GdkEventButton *ev)
 {
-  GdkModifierType state;
+  if (ev->type != GDK_BUTTON_PRESS) {
+    return TRUE;
+  }
 
-  state = gtk_event_controller_get_current_event_state(
-                            GTK_EVENT_CONTROLLER(gesture));
-  editor_mouse_button_press(x, y, editor_mouse_button,
-                            convert_modifiers(state));
+  editor_mouse_button_press(ev->x, ev->y,
+                            convert_mouse_button(ev->button),
+                            convert_modifiers(ev->state));
 
   return TRUE;
 }
@@ -937,30 +952,24 @@ gboolean handle_edit_mouse_button_press(GtkGestureClick *gesture,
 /************************************************************************//**
   Pass on the gdk mouse event to the editor's handler.
 ****************************************************************************/
-gboolean handle_edit_mouse_button_release(GtkGestureClick *gesture,
-                                          int editor_mouse_button,
-                                          double x, double y)
+gboolean handle_edit_mouse_button_release(GdkEventButton *ev)
 {
-  GdkModifierType state;
+  if (ev->type != GDK_BUTTON_RELEASE) {
+    return TRUE;
+  }
 
-  state = gtk_event_controller_get_current_event_state(GTK_EVENT_CONTROLLER(gesture));
-  editor_mouse_button_release(x, y,
-                              editor_mouse_button,
-                              convert_modifiers(state));
+  editor_mouse_button_release(ev->x, ev->y,
+                              convert_mouse_button(ev->button),
+                              convert_modifiers(ev->state));
   return TRUE;
 }
 
 /************************************************************************//**
   Pass on the gdk mouse event to the editor's handler.
 ****************************************************************************/
-gboolean handle_edit_mouse_move(GtkEventControllerMotion *controller,
-                                gdouble x, gdouble y)
+gboolean handle_edit_mouse_move(GdkEventMotion *ev)
 {
-  GdkModifierType state;
-
-  state = gtk_event_controller_get_current_event_state(GTK_EVENT_CONTROLLER(controller));
-  editor_mouse_move(x, y, convert_modifiers(state));
-
+  editor_mouse_move(ev->x, ev->y, convert_modifiers(ev->state));
   return TRUE;
 }
 
@@ -1008,14 +1017,15 @@ create_tool_value_selector(struct editbar *eb,
                              G_TYPE_STRING);
   tvs->store = store;
 
-  scrollwin = gtk_scrolled_window_new();
-  gtk_scrolled_window_set_has_frame(GTK_SCROLLED_WINDOW(scrollwin), TRUE);
+  scrollwin = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin),
+                                      GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scrollwin),
                                              10 * tileset_tile_height(tileset));
-  gtk_box_insert_child_after(GTK_BOX(vbox), scrollwin, NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), scrollwin, TRUE, TRUE, 0);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(tvs->store));
   gtk_widget_set_size_request(view, -1, 10 * tileset_tile_height(tileset));
@@ -1056,11 +1066,11 @@ create_tool_value_selector(struct editbar *eb,
   gtk_tree_view_column_set_sort_column_id(col, TVS_COL_NAME);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
 
-  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(scrollwin), view);
+  gtk_container_add(GTK_CONTAINER(scrollwin), view);
   tvs->view = view;
 
   /* Show everything but the window itself. */
-  gtk_widget_show(vbox);
+  gtk_widget_show_all(vbox);
 
   return tvs;
 }
@@ -1068,23 +1078,20 @@ create_tool_value_selector(struct editbar *eb,
 /************************************************************************//**
   Handle a mouse click on the tool image area in the editor info box.
 ****************************************************************************/
-static gboolean editinfobox_handle_tool_image_button_press(
-                                                    GtkGestureClick *gesture,
-                                                    int n_press,
-                                                    double x, double y)
+static gboolean editinfobox_handle_tool_image_button_press(GtkWidget *evbox,
+                                                           GdkEventButton *ev,
+                                                           gpointer data)
 {
   editgui_run_tool_selection(editor_get_tool());
-
   return TRUE;
 }
 
 /************************************************************************//**
   Handle a mouse click on the mode image area in the editor info box.
 ****************************************************************************/
-static gboolean editinfobox_handle_mode_image_button_press(
-                                                    GtkGestureClick *gesture,
-                                                    int n_press,
-                                                    double x, double y)
+static gboolean editinfobox_handle_mode_image_button_press(GtkWidget *evbox,
+                                                           GdkEventButton *ev,
+                                                           gpointer data)
 {
   editor_tool_cycle_mode(editor_get_tool());
   editgui_refresh();
@@ -1101,7 +1108,7 @@ static void editinfobox_spin_button_value_changed(GtkSpinButton *spinbutton,
   struct editinfobox *ei;
   int which, value;
   enum editor_tool_type ett;
-
+  
   ei = editgui_get_editinfobox();
 
   if (!ei) {
@@ -1158,97 +1165,79 @@ static void editinfobox_tool_applied_player_changed(GtkComboBox *combo,
 ****************************************************************************/
 static struct editinfobox *editinfobox_create(void)
 {
-  GtkWidget *label, *vgrid, *frame, *hgrid, *vgrid2, *pic;
+  GtkWidget *label, *vbox, *frame, *hbox, *vbox2, *image, *evbox;
   GtkWidget *spin, *combo;
   GtkListStore *store;
   GtkCellRenderer *cell;
-  GtkEventController *controller;
   struct editinfobox *ei;
   char buf[128];
-  int grid_row = 0;
-  int grid_col = 0;
-  int grid2_row = 0;
 
   ei = fc_calloc(1, sizeof(*ei));
 
-  /* If you turn this to something else than GtkFrame, also adjust
-   * editgui.c replace_widget() code that replaces widget in it. */
   frame = gtk_frame_new(_("Editor Tool"));
-  gtk_widget_set_margin_start(frame, 4);
-  gtk_widget_set_margin_end(frame, 4);
-  gtk_widget_set_margin_top(frame, 4);
-  gtk_widget_set_margin_bottom(frame, 4);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
   ei->widget = frame;
 
-  vgrid = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid),
+  vbox = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
                                  GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing(GTK_GRID(vgrid), 8);
-  gtk_widget_set_margin_start(vgrid, 4);
-  gtk_widget_set_margin_end(vgrid, 4);
-  gtk_widget_set_margin_top(vgrid, 4);
-  gtk_widget_set_margin_bottom(vgrid, 4);
-  gtk_frame_set_child(GTK_FRAME(frame), vgrid);
+  gtk_grid_set_row_spacing(GTK_GRID(vbox), 8);
+  gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
+  gtk_container_add(GTK_CONTAINER(frame), vbox);
 
   /* tool section */
-  hgrid = gtk_grid_new();
-  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 8);
-  gtk_grid_attach(GTK_GRID(vgrid), hgrid, 0, grid_row++, 1, 1);
+  hbox = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hbox), 8);
+  gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-  pic = gtk_picture_new();
-  gtk_widget_set_tooltip_text(pic, _("Click to change value if applicable."));
+  evbox = gtk_event_box_new();
+  gtk_widget_set_tooltip_text(evbox, _("Click to change value if applicable."));
+  g_signal_connect(evbox, "button_press_event",
+      G_CALLBACK(editinfobox_handle_tool_image_button_press), NULL);
+  gtk_container_add(GTK_CONTAINER(hbox), evbox);
 
-  controller = GTK_EVENT_CONTROLLER(gtk_gesture_click_new());
-  g_signal_connect(controller, "pressed",
-                   G_CALLBACK(editinfobox_handle_tool_image_button_press),
-                   NULL);
-  gtk_widget_add_controller(pic, controller);
+  image = gtk_image_new();
+  gtk_container_add(GTK_CONTAINER(evbox), image);
+  ei->tool_image = image;
 
-  gtk_grid_attach(GTK_GRID(hgrid), pic, grid_col++, 0, 1, 1);
-  ei->tool_pic = pic;
-
-  vgrid2 = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid2),
+  vbox2 = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox2),
                                  GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing(GTK_GRID(vgrid2), 4);
-  gtk_grid_attach(GTK_GRID(hgrid), vgrid2, grid_col++, 0, 1, 1);
+  gtk_grid_set_row_spacing(GTK_GRID(vbox2), 4);
+  gtk_container_add(GTK_CONTAINER(hbox), vbox2);
 
   label = gtk_label_new(NULL);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_grid_attach(GTK_GRID(vgrid2), label, 0, grid2_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox2), label);
   ei->tool_label = label;
 
   label = gtk_label_new(NULL);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_grid_attach(GTK_GRID(vgrid2), label, 0, grid2_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox2), label);
   ei->tool_value_label = label;
 
   /* mode section */
-  hgrid = gtk_grid_new();
-  grid_col = 0;
-  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 8);
-  gtk_grid_attach(GTK_GRID(vgrid), hgrid, 0, grid_row++, 1, 1);
+  hbox = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hbox), 8);
+  gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-  pic = gtk_picture_new();
-  gtk_widget_set_tooltip_text(pic, _("Click to change tool mode."));
+  evbox = gtk_event_box_new();
+  gtk_widget_set_tooltip_text(evbox, _("Click to change tool mode."));
+  g_signal_connect(evbox, "button_press_event",
+      G_CALLBACK(editinfobox_handle_mode_image_button_press), NULL);
+  gtk_container_add(GTK_CONTAINER(hbox), evbox);
 
-  controller = GTK_EVENT_CONTROLLER(gtk_gesture_click_new());
-  g_signal_connect(controller, "pressed",
-                   G_CALLBACK(editinfobox_handle_mode_image_button_press),
-                   NULL);
-  gtk_widget_add_controller(pic, controller);
+  image = gtk_image_new();
+  gtk_container_add(GTK_CONTAINER(evbox), image);
+  ei->mode_image = image;
 
-  gtk_grid_attach(GTK_GRID(hgrid), pic, grid_col++, 0, 1, 1);
-  ei->mode_pic = pic;
-
-  vgrid2 = gtk_grid_new();
-  grid2_row = 0;
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid2),
+  vbox2 = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox2),
                                  GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing(GTK_GRID(vgrid2), 4);
-  gtk_grid_attach(GTK_GRID(hgrid), vgrid2, grid_col++, 0, 1, 1);
+  gtk_grid_set_row_spacing(GTK_GRID(vbox2), 4);
+  gtk_container_add(GTK_CONTAINER(hbox), vbox2);
 
   label = gtk_label_new(NULL);
   fc_snprintf(buf, sizeof(buf), "<span weight=\"bold\">%s</span>",
@@ -1256,20 +1245,19 @@ static struct editinfobox *editinfobox_create(void)
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_grid_attach(GTK_GRID(vgrid2), label, 0, grid2_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox2), label);
 
   label = gtk_label_new(NULL);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_grid_attach(GTK_GRID(vgrid2), label, 0, grid2_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox2), label);
   ei->mode_label = label;
 
   /* spinner section */
-  hgrid = gtk_grid_new();
-  grid_col = 0;
-  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 8);
-  gtk_grid_attach(GTK_GRID(vgrid), hgrid, 0, grid_row++, 1, 1);
-  ei->size_hbox = hgrid;
+  hbox = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hbox), 8);
+  gtk_container_add(GTK_CONTAINER(vbox), hbox);
+  ei->size_hbox = hbox;
   spin = gtk_spin_button_new_with_range(1, 255, 1);
   gtk_widget_set_tooltip_text(spin,
       _("Use this to change the \"size\" parameter for the tool. "
@@ -1279,16 +1267,15 @@ static struct editinfobox *editinfobox_create(void)
   g_signal_connect(spin, "value-changed",
                    G_CALLBACK(editinfobox_spin_button_value_changed),
                    GINT_TO_POINTER(SPIN_BUTTON_SIZE));
-  gtk_grid_attach(GTK_GRID(hgrid), spin, grid_col++, 0, 1, 1);
+  gtk_container_add(GTK_CONTAINER(hbox), spin);
   ei->size_spin_button = spin;
   label = gtk_label_new(_("Size"));
-  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
-
-  hgrid = gtk_grid_new();
-  grid_col = 0;
-  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 8);
-  gtk_grid_attach(GTK_GRID(vgrid), hgrid, 0, grid_row++, 1, 1);
-  ei->count_hbox = hgrid;
+  gtk_container_add(GTK_CONTAINER(hbox), label);
+  
+  hbox = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hbox), 8);
+  gtk_container_add(GTK_CONTAINER(vbox), hbox);
+  ei->count_hbox = hbox;
   spin = gtk_spin_button_new_with_range(1, 255, 1);
   gtk_widget_set_tooltip_text(spin,
       _("Use this to change the tool's \"count\" parameter. "
@@ -1297,10 +1284,10 @@ static struct editinfobox *editinfobox_create(void)
   g_signal_connect(spin, "value-changed",
                    G_CALLBACK(editinfobox_spin_button_value_changed),
                    GINT_TO_POINTER(SPIN_BUTTON_COUNT));
-  gtk_grid_attach(GTK_GRID(hgrid), spin, grid_col++, 0, 1, 1);
+  gtk_container_add(GTK_CONTAINER(hbox), spin);
   ei->count_spin_button = spin;
   label = gtk_label_new(_("Count"));
-  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
+  gtk_container_add(GTK_CONTAINER(hbox), label);
 
   /* combo section */
   store = gtk_list_store_new(TAP_NUM_COLS,
@@ -1328,17 +1315,17 @@ static struct editinfobox *editinfobox_create(void)
       _("Use this to change the \"applied player\" tool parameter. "
         "This controls for example under which player units and cities "
         "are created."));
-  gtk_grid_attach(GTK_GRID(vgrid), combo, 0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox), combo);
   ei->tool_applied_player_combobox = combo;
 
   /* We add a ref to the editinfobox widget so that it is
    * not destroyed when replaced by the unit info box when
-   * we leave edit mode. See editinfobox_refresh() call to replace_widget() */
+   * we leave edit mode. See editinfobox_refresh(). */
   g_object_ref(ei->widget);
 
   /* The edit info box starts with no parent, so we have to
    * show its internal widgets manually. */
-  gtk_widget_show(ei->widget);
+  gtk_widget_show_all(ei->widget);
 
   return ei;
 }
@@ -1519,17 +1506,9 @@ static void replace_widget(GtkWidget *old, GtkWidget *new)
   }
 
   gtk_widget_hide(old);
-
-  if (GTK_IS_FRAME(parent)) {
-    gtk_frame_set_child(GTK_FRAME(parent), new);
-  } else {
-    fc_assert(GTK_IS_BOX(parent));
-
-    gtk_box_remove(GTK_BOX(parent), old);
-    gtk_box_append(GTK_BOX(parent), new);
-  }
-
-  gtk_widget_show(new);
+  gtk_container_remove(GTK_CONTAINER(parent), old);
+  gtk_container_add(GTK_CONTAINER(parent), new);
+  gtk_widget_show_all(new);
 }
 
 /************************************************************************//**
@@ -1561,7 +1540,7 @@ static void editinfobox_refresh(struct editinfobox *ei)
   gtk_label_set_text(label, editor_tool_get_mode_name(ett, etm));
 
   pixbuf = get_tool_mode_pixbuf(etm);
-  gtk_picture_set_pixbuf(GTK_PICTURE(ei->mode_pic), pixbuf);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(ei->mode_image), pixbuf);
   if (pixbuf) {
     g_object_unref(pixbuf);
     pixbuf = NULL;
@@ -1582,14 +1561,14 @@ static void editinfobox_refresh(struct editinfobox *ei)
 
     spr = editor_tool_get_sprite(ett);
     pixbuf = spr ? sprite_get_pixbuf(spr) : NULL;
-    gtk_picture_set_pixbuf(GTK_PICTURE(ei->tool_pic), pixbuf);
+    gtk_image_set_from_pixbuf(GTK_IMAGE(ei->tool_image), pixbuf);
     if (pixbuf) {
       g_object_unref(G_OBJECT(pixbuf));
       pixbuf = NULL;
     }
   } else {
     pixbuf = get_tool_value_pixbuf(ett, value);
-    gtk_picture_set_pixbuf(GTK_PICTURE(ei->tool_pic), pixbuf);
+    gtk_image_set_from_pixbuf(GTK_IMAGE(ei->tool_image), pixbuf);
     if (pixbuf) {
       g_object_unref(pixbuf);
       pixbuf = NULL;
@@ -1622,7 +1601,7 @@ static void editinfobox_refresh(struct editinfobox *ei)
 /************************************************************************//**
   Handle ctrl+[key] combinations.
 ****************************************************************************/
-static gboolean handle_edit_key_press_with_ctrl(guint keyval)
+static gboolean handle_edit_key_press_with_ctrl(GdkEventKey *ev)
 {
   return FALSE; /* Don't gobble. */
 }
@@ -1630,12 +1609,12 @@ static gboolean handle_edit_key_press_with_ctrl(guint keyval)
 /************************************************************************//**
   Handle shift+[key] combinations.
 ****************************************************************************/
-static gboolean handle_edit_key_press_with_shift(guint keyval)
+static gboolean handle_edit_key_press_with_shift(GdkEventKey *ev)
 {
   enum editor_tool_type ett;
 
   ett = editor_get_tool();
-  switch (keyval) {
+  switch (ev->keyval) {
   case GDK_KEY_D:
     editor_tool_toggle_mode(ett, ETM_ERASE);
     break;
@@ -1678,23 +1657,23 @@ static gboolean handle_edit_key_press_with_shift(guint keyval)
 /************************************************************************//**
   Handle any kind of key press event.
 ****************************************************************************/
-gboolean handle_edit_key_press(guint keyval, GdkModifierType state)
+gboolean handle_edit_key_press(GdkEventKey *ev)
 {
   enum editor_tool_type ett, new_ett = NUM_EDITOR_TOOL_TYPES;
 
   /* Check ctrl before shift - this is correct also for the case where
    * they are both active. */
-  if (state & GDK_CONTROL_MASK) {
-    return handle_edit_key_press_with_ctrl(keyval);
+  if (ev->state & GDK_CONTROL_MASK) {
+    return handle_edit_key_press_with_ctrl(ev);
   }
 
-  if (state & GDK_SHIFT_MASK) {
-    return handle_edit_key_press_with_shift(keyval);
+  if (ev->state & GDK_SHIFT_MASK) {
+    return handle_edit_key_press_with_shift(ev);
   }
 
   ett = editor_get_tool();
 
-  switch (keyval) {
+  switch (ev->keyval) {
   case GDK_KEY_t:
     new_ett = ETT_TERRAIN;
     break;
@@ -1727,11 +1706,9 @@ gboolean handle_edit_key_press(guint keyval, GdkModifierType state)
   case GDK_KEY_KP_Add:
     if (editor_tool_has_size(ett)) {
       int size = editor_tool_get_size(ett);
-
       editor_tool_set_size(ett, size + 1);
     } else if (editor_tool_has_count(ett)) {
       int count = editor_tool_get_count(ett);
-
       editor_tool_set_count(ett, count + 1);
     }
     break;
@@ -1740,11 +1717,9 @@ gboolean handle_edit_key_press(guint keyval, GdkModifierType state)
   case GDK_KEY_KP_Subtract:
     if (editor_tool_has_size(ett)) {
       int size = editor_tool_get_size(ett);
-
       editor_tool_set_size(ett, size - 1);
     } else if (editor_tool_has_count(ett)) {
       int count = editor_tool_get_count(ett);
-
       editor_tool_set_count(ett, count - 1);
     }
     break;
@@ -1758,9 +1733,9 @@ gboolean handle_edit_key_press(guint keyval, GdkModifierType state)
   case GDK_KEY_8:
   case GDK_KEY_9:
     if (editor_tool_has_size(ett)) {
-      editor_tool_set_size(ett, keyval - GDK_KEY_1 + 1);
+      editor_tool_set_size(ett, ev->keyval - GDK_KEY_1 + 1);
     } else if (editor_tool_has_count(ett)) {
-      editor_tool_set_count(ett, keyval - GDK_KEY_1 + 1);
+      editor_tool_set_count(ett, ev->keyval - GDK_KEY_1 + 1);
     }
     break;
   case GDK_KEY_space:
@@ -1797,6 +1772,14 @@ gboolean handle_edit_key_press(guint keyval, GdkModifierType state)
   return TRUE;
 }
 
+/************************************************************************//**
+  Key release handler.
+****************************************************************************/
+gboolean handle_edit_key_release(GdkEventKey *ev)
+{
+  return FALSE;
+}
+
 /************************************************************************//**
   Get the pointer for the editbar embedded in the client's GUI.
 ****************************************************************************/
