@@ -50,7 +50,7 @@
 #include "reqtree.h"
 #include "text.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "canvas.h"
 #include "cityrep.h"
 #include "dialogs.h"
@@ -330,14 +330,12 @@ static void science_diagram_center(GtkWidget *diagram, Tech_type_id tech)
       + gtk_adjustment_get_upper(adjust)
       - gtk_adjustment_get_page_size(adjust)) / width * x;
     gtk_adjustment_set_value(adjust, value);
-    gtk_adjustment_value_changed(adjust);
 
     adjust = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sw));
     value = (gtk_adjustment_get_lower(adjust)
       + gtk_adjustment_get_upper(adjust)
       - gtk_adjustment_get_page_size(adjust)) / height * y;
     gtk_adjustment_set_value(adjust, value);
-    gtk_adjustment_value_changed(adjust);
   }
 }
 
@@ -592,7 +590,8 @@ static void science_report_init(struct science_report *preport)
   /* TRANS: Research report title */
   gui_dialog_set_title(preport->shell, _("Research"));
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  gui_dialog_add_button(preport->shell, "window-close", _("_Close"),
+                        GTK_RESPONSE_CLOSE);
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
 
   vbox = GTK_CONTAINER(preport->shell->vbox);
@@ -1024,7 +1023,6 @@ static void economy_report_command_callback(struct gui_dialog *pdialog,
       if (ERD_RES_DISBAND_UNITS == response) {
         const struct unit_type *putype = selected.value.utype;
         gint count;
-
         gtk_tree_model_get(model, &iter, ERD_COL_COUNT, &count, -1);
 
         shell = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL
@@ -1191,23 +1189,24 @@ static void economy_report_init(struct economy_report *preport)
 
   label = gtk_label_new(NULL);
   gtk_container_add(vbox, label);
-  gtk_widget_set_margin_left(label, 5);
-  gtk_widget_set_margin_right(label, 5);
+  gtk_widget_set_margin_start(label, 5);
+  gtk_widget_set_margin_end(label, 5);
   gtk_widget_set_margin_top(label, 5);
   gtk_widget_set_margin_bottom(label, 5);
   preport->label = GTK_LABEL(label);
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  gui_dialog_add_button(preport->shell, "window-close", _("_Close"),
+                        GTK_RESPONSE_CLOSE);
 
-  button = gui_dialog_add_button(preport->shell, _("_Disband"),
+  button = gui_dialog_add_button(preport->shell, NULL, _("_Disband"),
                                  ERD_RES_DISBAND_UNITS);
   gtk_widget_set_sensitive(button, FALSE);
 
-  button = gui_dialog_add_button(preport->shell, _("Sell _All"),
+  button = gui_dialog_add_button(preport->shell, NULL, _("Sell _All"),
                                  ERD_RES_SELL_ALL);
   gtk_widget_set_sensitive(button, FALSE);
 
-  button = gui_dialog_add_button(preport->shell, _("Sell _Redundant"),
+  button = gui_dialog_add_button(preport->shell, NULL, _("Sell _Redundant"),
                                  ERD_RES_SELL_REDUNDANT);
   gtk_widget_set_sensitive(button, FALSE);
 
@@ -1725,14 +1724,15 @@ static void units_report_init(struct units_report *preport)
     }
   }
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  gui_dialog_add_button(preport->shell, "window-close", _("_Close"),
+                        GTK_RESPONSE_CLOSE);
 
-  button = gui_dialog_add_button(preport->shell, _("_Upgrade"),
+  button = gui_dialog_add_button(preport->shell, NULL, _("_Upgrade"),
                                  URD_RES_UPGRADE);
   gtk_widget_set_sensitive(button, FALSE);
 
-  button = gui_dialog_add_stockbutton(preport->shell, GTK_STOCK_FIND,
-                                      _("Find _Nearest"), URD_RES_NEAREST);
+  button = gui_dialog_add_button(preport->shell, "edit-find",
+                                 _("Find _Nearest"), URD_RES_NEAREST);
   gtk_widget_set_sensitive(button, FALSE);
 
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
