@@ -35,7 +35,7 @@
 #include "tilespec.h"
 #include "unitselect_common.h"
 
-/* client/gui-gtk-4.0 */
+/* client/gui-gtk-3.22 */
 #include "graphics.h"
 #include "gui_stuff.h"
 #include "gui_main.h"
@@ -152,7 +152,7 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
                              const char *title,
                              enum unit_select_location_mode loc);
 static GtkTreeStore *usdlg_tab_store_new(void);
-static bool usdlg_tab_update(struct unit_select_dialog *pdialog,
+static bool usdlg_tab_update(struct unit_select_dialog *pdialog, 
                              struct usdata_hash *ushash,
                              enum unit_select_location_mode loc);
 static void usdlg_tab_append_utype(GtkTreeStore *store,
@@ -177,7 +177,7 @@ static void usdlg_cmd_sentry(GObject *object, gpointer data);
 static void usdlg_cmd_select(GObject *object, gpointer data);
 static void usdlg_cmd_deselect(GObject *object, gpointer data);
 static void usdlg_cmd_exec(GObject *object, gpointer mode_data,
-                           enum usdlg_cmd cmd);
+			   enum usdlg_cmd cmd);
 static void usdlg_cmd_exec_unit(struct unit *punit, enum usdlg_cmd cmd);
 static void usdlg_cmd_center(GObject *object, gpointer data);
 static void usdlg_cmd_focus(GObject *object, gpointer data);
@@ -200,7 +200,7 @@ void unit_select_dialog_popup_main(struct tile *ptile, bool create)
   /* Present the unit selection dialog if it exists. */
   if (pdialog) {
     /* Show all. */
-    gtk_widget_show(GTK_WIDGET(pdialog->shell));
+    gtk_widget_show_all(GTK_WIDGET(pdialog->shell));
     /* Update tile. */
     usdlg_tile(pdialog, ptile);
     /* Refresh data and hide unused tabs. */
@@ -256,6 +256,7 @@ static struct unit_select_dialog *usdlg_create(void)
   setup_dialog(pdialog->shell, toplevel);
   g_signal_connect(pdialog->shell, "destroy",
                    G_CALLBACK(usdlg_destroy_callback), pdialog);
+  gtk_window_set_position(GTK_WINDOW(pdialog->shell), GTK_WIN_POS_MOUSE);
   gtk_widget_realize(pdialog->shell);
 
   vbox = gtk_dialog_get_content_area(GTK_DIALOG(pdialog->shell));
@@ -264,7 +265,7 @@ static struct unit_select_dialog *usdlg_create(void)
   pdialog->notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(pdialog->notebook),
                            GTK_POS_BOTTOM);
-  gtk_box_insert_child_after(GTK_BOX(vbox), pdialog->notebook, NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), pdialog->notebook, TRUE, TRUE, 0);
 
   /* Append pages. */
   usdlg_tab_select(pdialog, _("_Units"), SELLOC_UNITS);
@@ -292,7 +293,7 @@ static struct unit_select_dialog *usdlg_create(void)
 static void usdlg_destroy(void)
 {
   if (unit_select_dlg) {
-    gtk_window_destroy(GTK_WINDOW(unit_select_dlg->shell));
+    gtk_widget_destroy(GTK_WIDGET(unit_select_dlg->shell));
     free(unit_select_dlg);
   }
   unit_select_dlg = NULL;
@@ -387,28 +388,22 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
                              const char *title,
                              enum unit_select_location_mode loc)
 {
-  GtkWidget *page, *label, *hgrid, *vgrid, *view, *sw;
+  GtkWidget *page, *label, *hbox, *vbox, *view, *sw;
   GtkTreeStore *store;
   static bool titles_done;
   int i;
-  int page_row = 0;
-  int grid_col = 0;
-  int grid_row = 0;
 
   page = gtk_grid_new();
   gtk_orientable_set_orientation(GTK_ORIENTABLE(page),
                                  GTK_ORIENTATION_VERTICAL);
-  gtk_widget_set_margin_start(page, 8);
-  gtk_widget_set_margin_end(page, 8);
-  gtk_widget_set_margin_top(page, 8);
-  gtk_widget_set_margin_bottom(page, 8);
+  gtk_container_set_border_width(GTK_CONTAINER(page), 8);
   pdialog->tabs[loc].page = page;
 
   label = gtk_label_new_with_mnemonic(title);
   gtk_notebook_append_page(GTK_NOTEBOOK(pdialog->notebook), page, label);
 
-  hgrid = gtk_grid_new();
-  gtk_grid_attach(GTK_GRID(page), hgrid, 0, page_row++, 1, 1);
+  hbox = gtk_grid_new();
+  gtk_container_add(GTK_CONTAINER(page), hbox);
 
   store = usdlg_tab_store_new();
   pdialog->tabs[loc].store = store;
@@ -460,26 +455,26 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
     gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
   }
 
-  sw = gtk_scrolled_window_new();
+  sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(sw), 300);
-  gtk_scrolled_window_set_has_frame(GTK_SCROLLED_WINDOW(sw), TRUE);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+                                      GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw), view);
-  gtk_grid_attach(GTK_GRID(hgrid), sw, grid_col++, 0, 1, 1);
+  gtk_container_add(GTK_CONTAINER(sw), view);
+  gtk_container_add(GTK_CONTAINER(hbox), sw);
 
-  vgrid = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid),
+  vbox = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
                                  GTK_ORIENTATION_VERTICAL);
-  gtk_grid_attach(GTK_GRID(hgrid), vgrid, grid_col++, 0, 1, 1);
+  gtk_container_add(GTK_CONTAINER(hbox), vbox);
 
   /* button box 1: ready, sentry */
 
   pdialog->tabs[loc].cmd[USDLG_CMD_READY]
     = gtk_button_new_with_mnemonic(_("Ready"));
-  gtk_grid_attach(GTK_GRID(vgrid),
-                  pdialog->tabs[loc].cmd[USDLG_CMD_READY],
-                  0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox),
+                    pdialog->tabs[loc].cmd[USDLG_CMD_READY]);
   g_signal_connect(pdialog->tabs[loc].cmd[USDLG_CMD_READY], "clicked",
                    G_CALLBACK(usdlg_cmd_ready), GINT_TO_POINTER(loc));
   gtk_widget_set_sensitive(
@@ -489,9 +484,8 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
     = gtk_button_new_with_mnemonic(_("Sentry"));
   gtk_widget_set_margin_bottom(
     GTK_WIDGET(pdialog->tabs[loc].cmd[USDLG_CMD_SENTRY]), 10);
-  gtk_grid_attach(GTK_GRID(vgrid),
-                  pdialog->tabs[loc].cmd[USDLG_CMD_SENTRY],
-                  0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox),
+                    pdialog->tabs[loc].cmd[USDLG_CMD_SENTRY]);
   g_signal_connect(pdialog->tabs[loc].cmd[USDLG_CMD_SENTRY], "clicked",
                    G_CALLBACK(usdlg_cmd_sentry), GINT_TO_POINTER(loc));
   gtk_widget_set_sensitive(
@@ -501,9 +495,8 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
 
   pdialog->tabs[loc].cmd[USDLG_CMD_SELECT]
     = gtk_button_new_with_mnemonic(_("_Select"));
-  gtk_grid_attach(GTK_GRID(vgrid),
-                  pdialog->tabs[loc].cmd[USDLG_CMD_SELECT],
-                  0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox),
+                    pdialog->tabs[loc].cmd[USDLG_CMD_SELECT]);
   g_signal_connect(pdialog->tabs[loc].cmd[USDLG_CMD_SELECT], "clicked",
                    G_CALLBACK(usdlg_cmd_select), GINT_TO_POINTER(loc));
   gtk_widget_set_sensitive(
@@ -513,9 +506,8 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
     = gtk_button_new_with_mnemonic(_("_Deselect"));
   gtk_widget_set_margin_bottom(
     GTK_WIDGET(pdialog->tabs[loc].cmd[USDLG_CMD_DESELECT]), 10);
-  gtk_grid_attach(GTK_GRID(vgrid),
-                  pdialog->tabs[loc].cmd[USDLG_CMD_DESELECT],
-                  0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox),
+                    pdialog->tabs[loc].cmd[USDLG_CMD_DESELECT]);
   g_signal_connect(pdialog->tabs[loc].cmd[USDLG_CMD_DESELECT], "clicked",
                    G_CALLBACK(usdlg_cmd_deselect), GINT_TO_POINTER(loc));
   gtk_widget_set_sensitive(
@@ -525,9 +517,8 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
 
   pdialog->tabs[loc].cmd[USDLG_CMD_CENTER]
     = gtk_button_new_with_mnemonic(_("C_enter"));
-  gtk_grid_attach(GTK_GRID(vgrid),
-                  pdialog->tabs[loc].cmd[USDLG_CMD_CENTER],
-                  0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox),
+                    pdialog->tabs[loc].cmd[USDLG_CMD_CENTER]);
   g_signal_connect(pdialog->tabs[loc].cmd[USDLG_CMD_CENTER], "clicked",
                    G_CALLBACK(usdlg_cmd_center), GINT_TO_POINTER(loc));
   gtk_widget_set_sensitive(
@@ -535,9 +526,8 @@ static void usdlg_tab_select(struct unit_select_dialog *pdialog,
 
   pdialog->tabs[loc].cmd[USDLG_CMD_FOCUS]
     = gtk_button_new_with_mnemonic(_("_Focus"));
-  gtk_grid_attach(GTK_GRID(vgrid),
-                  pdialog->tabs[loc].cmd[USDLG_CMD_FOCUS],
-                  0, grid_row++, 1, 1);
+  gtk_container_add(GTK_CONTAINER(vbox),
+                    pdialog->tabs[loc].cmd[USDLG_CMD_FOCUS]);
   g_signal_connect(pdialog->tabs[loc].cmd[USDLG_CMD_FOCUS], "clicked",
                    G_CALLBACK(usdlg_cmd_focus), GINT_TO_POINTER(loc));
   gtk_widget_set_sensitive(
@@ -1011,7 +1001,7 @@ static void usdlg_cmd_exec(GObject *object, gpointer mode_data,
       ushash = usdlg_data_new(pdialog->ptile);
 
       usdata_hash_lookup(ushash, utid, &data);
-      if (data != NULL
+      if (data != NULL 
           && unit_list_size(data->units[loc][act]) != 0) {
         unit_list_iterate(data->units[loc][act], punit) {
           usdlg_cmd_exec_unit(punit, cmd);
