@@ -43,7 +43,7 @@
 #include "options.h"
 #include "text.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-4.0 */
 #include "chatline.h"
 #include "dialogs.h"
 #include "gui_main.h"
@@ -58,11 +58,11 @@
 struct gui_dialog *players_dialog_shell;
 static GtkWidget *players_list;
 static GtkTreeSelection *players_selection;
-static GtkWidget *players_int_command;
-static GtkWidget *players_meet_command;
-static GtkWidget *players_war_command;
-static GtkWidget *players_vision_command;
-static GtkWidget *players_sship_command;
+static GSimpleAction *players_int_command;
+static GSimpleAction *players_meet_command;
+static GSimpleAction *players_war_command;
+static GSimpleAction *players_vision_command;
+static GSimpleAction *players_sship_command;
 
 static GtkListStore *players_dialog_store;
 #define PLR_DLG_COL_STYLE       (0 + num_player_dlg_columns)
@@ -73,18 +73,27 @@ static GtkListStore *players_dialog_store;
 #define PLR_DLG_COL_TOTAL       (PLR_DLG_COL_TOOLTIP + 1)
 
 static void create_players_dialog(void);
-static void players_meet_callback(GtkMenuItem *item, gpointer data);
-static void players_war_callback(GtkMenuItem *item, gpointer data);
-static void players_vision_callback(GtkMenuItem *item, gpointer data);
-static void players_intel_callback(GtkMenuItem *item, gpointer data);
-static void players_intel_wonder_callback(GtkMenuItem *item, gpointer data);
-static void players_sship_callback(GtkMenuItem *item, gpointer data);
-static void players_ai_toggle_callback(GtkMenuItem *item, gpointer data);
-static void players_ai_skill_callback(GtkMenuItem *item, gpointer data);
-
+static void players_meet_callback(GSimpleAction *action, GVariant *parameter,
+                                  gpointer data);
+static void players_war_callback(GSimpleAction *action, GVariant *parameter,
+                                 gpointer data);
+static void players_vision_callback(GSimpleAction *action, GVariant *parameter,
+                                    gpointer data);
+static void players_intel_callback(GSimpleAction *action, GVariant *parameter,
+                                   gpointer data);
+static void players_intel_wonder_callback(GSimpleAction *action,
+                                          GVariant *parameter, gpointer data);
+static void players_sship_callback(GSimpleAction *action, GVariant *parameter,
+                                   gpointer data);
+static void players_ai_toggle_callback(GSimpleAction *action, GVariant *parameter,
+                                       gpointer data);
+static void players_ai_skill_callback(GSimpleAction *action, GVariant *parameter,
+                                      gpointer data);
 
 static void update_views(void);
 
+static GMenu *display_menu;
+
 /**********************************************************************//**
   Popup the dialog 10% inside the main-window, and optionally raise it.
 **************************************************************************/
@@ -116,7 +125,8 @@ void popdown_players_dialog(void)
 **************************************************************************/
 GdkPixbuf *create_player_icon(const struct player *plr)
 {
-  int width, height;
+  int width = 20;
+  int height = 20;
   GdkPixbuf *tmp;
   cairo_surface_t *surface;
   struct color *color;
@@ -126,7 +136,6 @@ GdkPixbuf *create_player_icon(const struct player *plr)
     return NULL;
   }
 
-  gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
   surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
 
   cr = cairo_create(surface);
@@ -164,35 +173,35 @@ static void update_players_menu(void)
     plr = player_by_number(plrno);
 
     if (plr->spaceship.state != SSHIP_NONE) {
-      gtk_widget_set_sensitive(players_sship_command, TRUE);
+      g_simple_action_set_enabled(players_sship_command, TRUE);
     } else {
-      gtk_widget_set_sensitive(players_sship_command, FALSE);
+      g_simple_action_set_enabled(players_sship_command, FALSE);
     }
 
     if (NULL != client.conn.playing) {
       /* We keep button sensitive in case of DIPL_SENATE_BLOCKING, so that player
        * can request server side to check requirements of those effects with omniscience */
-      gtk_widget_set_sensitive(players_war_command,
-                               can_client_issue_orders()
-                               && pplayer_can_cancel_treaty(client_player(),
-                                                            player_by_number(plrno))
-                               != DIPL_ERROR);
+      g_simple_action_set_enabled(players_war_command,
+                                  can_client_issue_orders()
+                                  && pplayer_can_cancel_treaty(client_player(),
+                                                               player_by_number(plrno))
+                                  != DIPL_ERROR);
     } else {
-      gtk_widget_set_sensitive(players_war_command, FALSE);
+      g_simple_action_set_enabled(players_war_command, FALSE);
     }
 
-    gtk_widget_set_sensitive(players_vision_command,
-			     can_client_issue_orders()
-			     && gives_shared_vision(client.conn.playing, plr)
-                             && !players_on_same_team(client.conn.playing, plr));
+    g_simple_action_set_enabled(players_vision_command,
+                                can_client_issue_orders()
+                                && gives_shared_vision(client.conn.playing, plr)
+                                && !players_on_same_team(client.conn.playing, plr));
 
-    gtk_widget_set_sensitive(players_meet_command, can_meet_with_player(plr));
-    gtk_widget_set_sensitive(players_int_command, can_intel_with_player(plr));
+    g_simple_action_set_enabled(players_meet_command, can_meet_with_player(plr));
+    g_simple_action_set_enabled(players_int_command, can_intel_with_player(plr));
     return;
   }
 
-  gtk_widget_set_sensitive(players_meet_command, FALSE);
-  gtk_widget_set_sensitive(players_int_command, FALSE);
+  g_simple_action_set_enabled(players_meet_command, FALSE);
+  g_simple_action_set_enabled(players_int_command, FALSE);
 }
 
 /**********************************************************************//**
@@ -204,11 +213,15 @@ static void selection_callback(GtkTreeSelection *selection, gpointer data)
 }
 
 /**********************************************************************//**
-  Button pressed on player list
+  Left button pressed on player list
 **************************************************************************/
-static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
+static gboolean left_button_press_callback(GtkGestureClick *gesture, int n_press,
+                                           double x, double y)
 {
-  if (ev->type == GDK_2BUTTON_PRESS) {
+  GtkTreeView *view
+    = GTK_TREE_VIEW(gtk_event_controller_get_widget(GTK_EVENT_CONTROLLER(gesture)));
+
+  if (n_press == 2) {
     GtkTreePath *path;
 
     gtk_tree_view_get_cursor(view, &path, NULL);
@@ -224,15 +237,46 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
       gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &id, -1);
       plr = player_by_number(id);
 
-      if (ev->button == 1) {
-        if (can_intel_with_player(plr)) {
-          popup_intel_dialog(plr);
-        }
-      } else if (can_meet_with_player(plr)) {
+      if (can_intel_with_player(plr)) {
+        popup_intel_dialog(plr);
+      }
+    }
+  }
+
+  return FALSE;
+}
+
+/**********************************************************************//**
+  Right button pressed on player list
+**************************************************************************/
+static gboolean right_button_press_callback(GtkGestureClick *gesture, int n_press,
+                                            double x, double y)
+{
+  GtkTreeView *view
+    = GTK_TREE_VIEW(gtk_event_controller_get_widget(GTK_EVENT_CONTROLLER(gesture)));
+
+  if (n_press == 2) {
+    GtkTreePath *path;
+
+    gtk_tree_view_get_cursor(view, &path, NULL);
+    if (path) {
+      GtkTreeModel *model = gtk_tree_view_get_model(view);
+      GtkTreeIter it;
+      gint id;
+      struct player *plr;
+
+      gtk_tree_model_get_iter(model, &it, path);
+      gtk_tree_path_free(path);
+
+      gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &id, -1);
+      plr = player_by_number(id);
+
+      if (can_meet_with_player(plr)) {
         dsend_packet_diplomacy_init_meeting_req(&client.conn, id);
       }
     }
   }
+
   return FALSE;
 }
 
@@ -240,7 +284,7 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
   Sorting function for plr dlg.
 **************************************************************************/
 static gint plrdlg_sort_func(GtkTreeModel *model,
-			      GtkTreeIter *a, GtkTreeIter *b, gpointer data)
+                             GtkTreeIter *a, GtkTreeIter *b, gpointer data)
 {
   GValue value = { 0, };
   struct player *player1;
@@ -252,11 +296,11 @@ static gint plrdlg_sort_func(GtkTreeModel *model,
   gtk_tree_model_get_value(model, a, PLR_DLG_COL_ID, &value);
   player1 = player_by_number(g_value_get_int(&value));
   g_value_unset(&value);
-  
+
   gtk_tree_model_get_value(model, b, PLR_DLG_COL_ID, &value);
   player2 = player_by_number(g_value_get_int(&value));
   g_value_unset(&value);
-  
+
   return player_dlg_columns[n].sort_func(player1, player2);
 }
 
@@ -307,54 +351,113 @@ static GtkListStore *players_dialog_store_new(void)
   return store;
 }
 
+/************************************************************************//**
+  Create up-to-date menu item for the plrdlg display menu.
+  Caller need to g_object_unref() returned item.
+****************************************************************************/
+static GMenuItem *create_plrdlg_display_menu_item(int pos)
+{
+  GMenuItem *item;
+  char act_name[50];
+  struct player_dlg_column *pcol;
+
+  pcol = &player_dlg_columns[pos];
+
+  fc_snprintf(act_name, sizeof(act_name), "win.display%d(%s)",
+              pos, pcol->show ? "true" : "false");
+  item = g_menu_item_new(pcol->title, NULL);
+  g_menu_item_set_detailed_action(item, act_name);
+
+  return item;
+}
+
+/************************************************************************//**
+  Create up-to-date menu item for "Dead Players" menu entry.
+  Caller need to g_object_unref() returned item.
+****************************************************************************/
+static GMenuItem *create_dead_players_menu_item(void)
+{
+  GMenuItem *item;
+  char act_name[50];
+
+  /* TODO: Should this be gui-specific option as there's no universal
+   *       player dlg setup? */
+  fc_snprintf(act_name, sizeof(act_name), "win.show_dead(%s)",
+              gui_options.player_dlg_show_dead_players ? "true" : "false");
+  item = g_menu_item_new(Q_("?show:Dead Players"), NULL);
+  g_menu_item_set_detailed_action(item, act_name);
+
+  return item;
+}
+
 /**********************************************************************//**
   Toggled column visibility
 **************************************************************************/
-static void toggle_view(GtkCheckMenuItem* item, gpointer data)
+static void toggle_view(GSimpleAction *act, GVariant *value, gpointer data)
 {
-  struct player_dlg_column* pcol = data;
+  int idx = GPOINTER_TO_INT(data);
+  struct player_dlg_column *pcol = &player_dlg_columns[idx];
 
-  pcol->show = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
+  pcol->show ^= 1;
   update_views();
+
+  /* The menu has no 'playername' in the beginning, so menu index is one smaller
+   * then column index. */
+  g_menu_remove(display_menu, idx - 1);
+
+  menu_item_insert_unref(display_menu, idx - 1, create_plrdlg_display_menu_item(idx));
 }
 
 /**********************************************************************//**
   Called whenever player toggles the 'Show/Dead Players' menu item
 **************************************************************************/
-static void toggle_dead_players(GtkCheckMenuItem* item, gpointer data)
+static void toggle_dead_players(GSimpleAction *act, GVariant *value,
+                                gpointer data)
 {
-  gui_options.player_dlg_show_dead_players = 
-    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
+  int idx = GPOINTER_TO_INT(data);
+
+  gui_options.player_dlg_show_dead_players ^= 1;
   real_players_dialog_update(NULL);
+
+  /* The menu has no 'playername' in the beginning, so menu index is one smaller
+   * then column index - applies also to this even though this is not
+   * a column. */
+  g_menu_remove(display_menu, idx - 1);
+
+  menu_item_insert_unref(display_menu, idx - 1,
+                         create_dead_players_menu_item());
 }
 
 /**********************************************************************//**
   Create and return the "diplomacy" menu for the player report. This menu
   contains diplomacy actions the current player can use on other nations.
 **************************************************************************/
-static GtkWidget *create_diplomacy_menu(void)
+static GMenu *create_diplomacy_menu(GActionGroup *group)
 {
-  GtkWidget *menu, *item;
+  GMenu *menu;
+  GSimpleAction *act;
 
-  menu = gtk_menu_new();
+  menu = g_menu_new();
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Meet"));
-  g_signal_connect(item, "activate",
-                   G_CALLBACK(players_meet_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  players_meet_command = item;
+  act = g_simple_action_new("meet", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate", G_CALLBACK(players_meet_callback), NULL);
+  menu_item_append_unref(menu, g_menu_item_new(_("_Meet"), "win.meet"));
+  players_meet_command = act;
 
-  item = gtk_menu_item_new_with_mnemonic(_("Cancel _Treaty"));
-  g_signal_connect(item, "activate",
-                   G_CALLBACK(players_war_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  players_war_command = item;
+  act = g_simple_action_new("cancel_treaty", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate", G_CALLBACK(players_war_callback), NULL);
+  menu_item_append_unref(menu, g_menu_item_new(_("Cancel _Treaty"),
+                                               "win.cancel_treaty"));
+  players_war_command = act;
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Withdraw Vision"));
-  g_signal_connect(item, "activate",
-                   G_CALLBACK(players_vision_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  players_vision_command = item;
+  act = g_simple_action_new("withdraw_vision", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate", G_CALLBACK(players_vision_callback), NULL);
+  menu_item_append_unref(menu, g_menu_item_new(_("_Withdraw Vision"),
+                                               "win.withdraw_vision"));
+  players_vision_command = act;
 
   return menu;
 }
@@ -363,28 +466,29 @@ static GtkWidget *create_diplomacy_menu(void)
   Create and return the "intelligence" menu. The items in this menu are
   used by the player to see more detailed information about other nations.
 **************************************************************************/
-static GtkWidget *create_intelligence_menu(void)
+static GMenu *create_intelligence_menu(GActionGroup *group)
 {
-  GtkWidget *menu, *item;
+  GMenu *menu;
+  GSimpleAction *act;
 
-  menu = gtk_menu_new();
+  menu = g_menu_new();
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Report"));
-  g_signal_connect(item, "activate",
-                   G_CALLBACK(players_intel_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  players_int_command = item;
+  act = g_simple_action_new("report", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate", G_CALLBACK(players_intel_callback), NULL);
+  menu_item_append_unref(menu, g_menu_item_new(_("_Report"), "win.report"));
+  players_int_command = act;
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Wonders"));
-  g_signal_connect(item, "activate",
-                   G_CALLBACK(players_intel_wonder_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  act = g_simple_action_new("wonders", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate", G_CALLBACK(players_intel_wonder_callback), NULL);
+  menu_item_append_unref(menu, g_menu_item_new(_("_Wonders"), "win.wonders"));
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Spaceship"));
-  g_signal_connect(item, "activate",
-                   G_CALLBACK(players_sship_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  players_sship_command = item;
+  act = g_simple_action_new("spaceship", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate", G_CALLBACK(players_sship_callback), NULL);
+  menu_item_append_unref(menu, g_menu_item_new(_("_Spaceship"), "win.spaceship"));
+  players_sship_command = act;
 
   return menu;
 }
@@ -392,31 +496,77 @@ static GtkWidget *create_intelligence_menu(void)
 /**********************************************************************//**
   Create 'show' menu for player dialog
 **************************************************************************/
-static GtkWidget* create_show_menu(void)
+static GMenu *create_show_menu(GActionGroup *group)
 {
+  GVariantType *bvart = g_variant_type_new("b");
   int i;
-  GtkWidget *menu = gtk_menu_new();
-  GtkWidget *item;
+  GSimpleAction *act;
+  GVariant *var;
+
+  display_menu = g_menu_new();
 
   /* Index starting at one (1) here to force playername to always be shown */
-  for (i = 1; i < num_player_dlg_columns; i++) {
-    struct player_dlg_column *pcol;
+  for (i = 1; i < num_player_dlg_columns - 1; i++) {
+    char act_name[50];
 
-    pcol = &player_dlg_columns[i];
-    item = gtk_check_menu_item_new_with_label(pcol->title);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), pcol->show);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    g_signal_connect(item, "toggled", G_CALLBACK(toggle_view), pcol);
+    var = g_variant_new("b", TRUE);
+    fc_snprintf(act_name, sizeof(act_name), "display%d", i);
+    act = g_simple_action_new_stateful(act_name, bvart, var);
+    g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+    g_signal_connect(act, "change-state", G_CALLBACK(toggle_view), GINT_TO_POINTER(i));
+
+    menu_item_insert_unref(display_menu, i, create_plrdlg_display_menu_item(i));
   }
 
-  item = gtk_separator_menu_item_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  var = g_variant_new("b", TRUE);
+  act = g_simple_action_new_stateful("show_dead", bvart, var);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
 
-  item = gtk_check_menu_item_new_with_label(Q_("?show:Dead Players"));
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
-                                 gui_options.player_dlg_show_dead_players);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  g_signal_connect(item, "toggled", G_CALLBACK(toggle_dead_players), NULL);
+  menu_item_insert_unref(display_menu, i, create_dead_players_menu_item());
+  g_signal_connect(act, "change-state", G_CALLBACK(toggle_dead_players),
+                   GINT_TO_POINTER(i));
+
+  g_variant_type_free(bvart);
+
+  return display_menu;
+}
+
+/**********************************************************************//**
+  Create and return the "AI" menu, to adjust difficulty levels
+  of players.
+**************************************************************************/
+static GMenu *create_ai_menu(GActionGroup *group)
+{
+  GMenu *menu;
+  GSimpleAction *act;
+  enum ai_level level;
+
+  menu = g_menu_new();
+
+  act = g_simple_action_new("ai_toggle", NULL);
+  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+  g_signal_connect(act, "activate",
+                   G_CALLBACK(players_ai_toggle_callback), NULL);
+
+  menu_item_append_unref(menu, g_menu_item_new(_("_Toggle AI Mode"),
+                                               "win.ai_toggle"));
+
+  for (level = 0; level < AI_LEVEL_COUNT; level++) {
+    if (is_settable_ai_level(level)) {
+      const char *level_name = ai_level_translated_name(level);
+      char act_name[50];
+
+      fc_snprintf(act_name, sizeof(act_name), "ai_level%d", level);
+      act = g_simple_action_new(act_name, NULL);
+      g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
+      g_signal_connect(act, "activate",
+                       G_CALLBACK(players_ai_skill_callback),
+                       GUINT_TO_POINTER(level));
+
+      fc_snprintf(act_name, sizeof(act_name), "win.ai_level%d", level);
+      menu_item_append_unref(menu, g_menu_item_new(level_name, act_name));
+    }
+  }
 
   return menu;
 }
@@ -428,8 +578,14 @@ void create_players_dialog(void)
 {
   int i;
   GtkWidget *sep, *sw;
-  GtkWidget *menubar, *menu, *item, *vbox;
-  enum ai_level level;
+  GtkWidget *aux_menu;
+  GMenu *topmenu, *submenu;
+  GActionGroup *group;
+  GtkWidget *vgrid;
+  GtkEventController *left_controller;
+  GtkEventController *right_controller;
+  GtkGesture *gesture;
+  int grid_row = 0;
 
   gui_dialog_new(&players_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL,
                  TRUE);
@@ -455,8 +611,17 @@ void create_players_dialog(void)
   players_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(players_list));
   g_signal_connect(players_selection, "changed",
       G_CALLBACK(selection_callback), NULL);
-  g_signal_connect(players_list, "button_press_event",
-      G_CALLBACK(button_press_callback), NULL);
+  gesture = gtk_gesture_click_new();
+  left_controller = GTK_EVENT_CONTROLLER(gesture);
+  g_signal_connect(left_controller, "pressed",
+                   G_CALLBACK(left_button_press_callback), NULL);
+  gtk_widget_add_controller(players_list, left_controller);
+  gesture = gtk_gesture_click_new();
+  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 3);
+  right_controller = GTK_EVENT_CONTROLLER(gesture);
+  g_signal_connect(right_controller, "pressed",
+                   G_CALLBACK(right_button_press_callback), NULL);
+  gtk_widget_add_controller(players_list, right_controller);
 
   for (i = 0; i < num_player_dlg_columns; i++) {
     struct player_dlg_column *pcol;
@@ -519,77 +684,48 @@ void create_players_dialog(void)
   gtk_tree_view_set_search_column(GTK_TREE_VIEW(players_list),
                                   player_dlg_default_sort_column());
 
-  sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-                                      GTK_SHADOW_ETCHED_IN);
+  sw = gtk_scrolled_window_new();
+  gtk_scrolled_window_set_has_frame(GTK_SCROLLED_WINDOW(sw), TRUE);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-  gtk_container_add(GTK_CONTAINER(sw), players_list);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw), players_list);
 
-  gtk_container_add(GTK_CONTAINER(players_dialog_shell->vbox), sw);
+  gui_dialog_add_content_widget(players_dialog_shell, sw);
 
-  vbox = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
+  vgrid = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid),
                                  GTK_ORIENTATION_VERTICAL);
 
   sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
-  gtk_container_add(GTK_CONTAINER(vbox), sep);
+  gtk_grid_attach(GTK_GRID(vgrid), sep, 0, grid_row++, 1, 1);
 
-  menubar = gtk_aux_menu_bar_new();
-  gtk_container_add(GTK_CONTAINER(vbox), menubar);
+  aux_menu = aux_menu_new();
+  gtk_grid_attach(GTK_GRID(vgrid), aux_menu, 0, grid_row++, 1, 1);
 
+  gui_dialog_add_action_widget(players_dialog_shell, vgrid);
 
-  gui_dialog_add_widget(players_dialog_shell, vbox);
+  group = G_ACTION_GROUP(g_simple_action_group_new());
+  topmenu = g_menu_new();
 
-  item = gtk_menu_item_new_with_mnemonic(_("Di_plomacy"));
-  menu = create_diplomacy_menu();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  submenu = create_diplomacy_menu(group);
+  submenu_append_unref(topmenu, _("Di_plomacy"), G_MENU_MODEL(submenu));
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Intelligence"));
-  menu = create_intelligence_menu();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  submenu = create_intelligence_menu(group);
+  submenu_append_unref(topmenu, _("_Intelligence"), G_MENU_MODEL(submenu));
 
-  item = gtk_menu_item_new_with_mnemonic(_("_Display"));
-  menu = create_show_menu();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  submenu = create_show_menu(group);
+  submenu_append_unref(topmenu, _("_Display"), G_MENU_MODEL(submenu));
 
-  item = gtk_menu_item_new_with_mnemonic(_("_AI"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  submenu = create_ai_menu(group);
+  submenu_append_unref(topmenu, _("_AI"), G_MENU_MODEL(submenu));
 
-  menu = gtk_menu_new();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-
-  item = gtk_menu_item_new_with_mnemonic(_("_Toggle AI Mode"));
-  g_signal_connect(item, "activate",
-      G_CALLBACK(players_ai_toggle_callback), NULL);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-
-  sep = gtk_separator_menu_item_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep);
-
-  for (level = 0; level < AI_LEVEL_COUNT; level++) {
-    if (is_settable_ai_level(level)) {
-      const char *level_name = ai_level_translated_name(level);
-
-      item = gtk_menu_item_new_with_label(level_name);
-      g_signal_connect(item, "activate",
-                       G_CALLBACK(players_ai_skill_callback),
-                       GUINT_TO_POINTER(level));
-      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    }
-  }
-  gtk_widget_show_all(menu);
+  gtk_widget_insert_action_group(aux_menu, "win", group);
+  gtk_menu_button_set_menu_model(GTK_MENU_BUTTON(aux_menu), G_MENU_MODEL(topmenu));
 
   gui_dialog_show_all(players_dialog_shell);
 
   real_players_dialog_update(NULL);
 
-  gui_dialog_set_default_response(players_dialog_shell,
-    GTK_RESPONSE_CLOSE);
-
   gtk_tree_view_focus(GTK_TREE_VIEW(players_list));
 }
 
@@ -600,9 +736,9 @@ void create_players_dialog(void)
 #define MIN_DIMENSION 5
 
 /**********************************************************************//**
- Builds the flag pixmap. May return NULL if there is not enough memory.
- You must call g_object_unref on the returned pixbuf when it is no
- longer needed.
+  Builds the flag pixmap. May return NULL if there is not enough memory.
+  You must call g_object_unref on the returned pixbuf when it is no
+  longer needed.
 **************************************************************************/
 GdkPixbuf *get_flag(const struct nation_type *nation)
 {
@@ -640,7 +776,6 @@ GdkPixbuf *get_flag(const struct nation_type *nation)
   return im;
 }
 
-
 /**********************************************************************//**
   Fills the player list with the information for 'pplayer' at the row
   given by 'it'.
@@ -648,7 +783,7 @@ GdkPixbuf *get_flag(const struct nation_type *nation)
 static void fill_row(GtkListStore *store, GtkTreeIter *it,
                      const struct player *pplayer)
 {
-  struct player_dlg_column* pcol;
+  struct player_dlg_column *pcol;
   GdkPixbuf *pixbuf;
   int style = PANGO_STYLE_NORMAL, weight = PANGO_WEIGHT_NORMAL;
   int k;
@@ -764,7 +899,8 @@ void real_players_dialog_update(void *unused)
   Callback for diplomatic meetings button. This button is enabled iff
   we can meet with the other player.
 **************************************************************************/
-void players_meet_callback(GtkMenuItem *item, gpointer data)
+void players_meet_callback(GSimpleAction *action, GVariant *parameter,
+                           gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -794,10 +930,10 @@ static void confirm_cancel_pact(enum clause_type clause, int plrno,
   setup_dialog(shell, toplevel);
   gtk_dialog_set_default_response(GTK_DIALOG(shell), GTK_RESPONSE_NO);
 
-  if (gtk_dialog_run(GTK_DIALOG(shell)) == GTK_RESPONSE_YES) {
+  if (blocking_dialog(shell) == GTK_RESPONSE_YES) {
     dsend_packet_diplomacy_cancel_pact(&client.conn, plrno, clause);
   }
-  gtk_widget_destroy(shell);
+  gtk_window_destroy(GTK_WINDOW(shell));
   FC_FREE(title);
   FC_FREE(question);
 }
@@ -805,7 +941,8 @@ static void confirm_cancel_pact(enum clause_type clause, int plrno,
 /**********************************************************************//**
   Pact cancellation requested
 **************************************************************************/
-void players_war_callback(GtkMenuItem *item, gpointer data)
+void players_war_callback(GSimpleAction *action, GVariant *parameter,
+                          gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -846,7 +983,8 @@ void players_war_callback(GtkMenuItem *item, gpointer data)
 /**********************************************************************//**
   Withdrawing shared vision
 **************************************************************************/
-void players_vision_callback(GtkMenuItem *item, gpointer data)
+void players_vision_callback(GSimpleAction *action, GVariant *parameter,
+                             gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -873,7 +1011,8 @@ void players_vision_callback(GtkMenuItem *item, gpointer data)
 /**********************************************************************//**
   Intelligence report query
 **************************************************************************/
-void players_intel_callback(GtkMenuItem *item, gpointer data)
+void players_intel_callback(GSimpleAction *action, GVariant *parameter,
+                            gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -892,7 +1031,8 @@ void players_intel_callback(GtkMenuItem *item, gpointer data)
 /**********************************************************************//**
   Wonders list report query
 **************************************************************************/
-void players_intel_wonder_callback(GtkMenuItem *item, gpointer data)
+void players_intel_wonder_callback(GSimpleAction *action, GVariant *parameter,
+                                   gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -909,7 +1049,8 @@ void players_intel_wonder_callback(GtkMenuItem *item, gpointer data)
 /**********************************************************************//**
   Spaceship query callback
 **************************************************************************/
-void players_sship_callback(GtkMenuItem *item, gpointer data)
+void players_sship_callback(GSimpleAction *action, GVariant *parameter,
+                            gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -925,7 +1066,9 @@ void players_sship_callback(GtkMenuItem *item, gpointer data)
 /**********************************************************************//**
   AI toggle callback.
 **************************************************************************/
-static void players_ai_toggle_callback(GtkMenuItem *item, gpointer data)
+static void players_ai_toggle_callback(GSimpleAction *action,
+                                       GVariant *parameter,
+                                       gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
@@ -942,7 +1085,9 @@ static void players_ai_toggle_callback(GtkMenuItem *item, gpointer data)
 /**********************************************************************//**
   AI skill level setting callback.
 **************************************************************************/
-static void players_ai_skill_callback(GtkMenuItem *item, gpointer data)
+static void players_ai_skill_callback(GSimpleAction *action,
+                                      GVariant *parameter,
+                                      gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
