@@ -155,8 +155,8 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   } else {
     plr2_label->setText("FLAG MISSING");
   }
-  text2 = ruler_title_for_player(player_by_number(counterpart),
-                                 plr_buf, sizeof(plr_buf));
+  text2 = title_for_player(player_by_number(counterpart),
+                           plr_buf, sizeof(plr_buf));
   text2 = "<b><center>" + text2.toHtmlEscaped() + "</center></b>";
   label2->setText(text2);
   plr2_accept = new QLabel;
