@@ -23,4 +23,12 @@ QApplication *current_app();
 #define CAPTURE_DEFAULT_THIS [=]
 #endif // FREECIV_HAVE_CXX20_CAPTURE_THIS
 
+// Compatibility layer between Qt5 and Qt6 while we support
+// also the former.
+#ifndef FC_QT5_MODE
+#define mevent_gpos(__ev__) (__ev__)->globalPosition().toPoint()
+#else  // FC_QT5_MODE
+#define mevent_gpos(__ev__) (__ev__)->globalPos()
+#endif // FC_QT5_MODE
+
 #endif // FC__GUI_MAIN_H
