@@ -30,14 +30,18 @@ bool client_handle_packet(enum packet_type type, const void *packet)
 
   case PACKET_INVESTIGATE_STARTED:
     handle_investigate_started(
-      ((const struct packet_investigate_started *)packet)->unit_id,
-      ((const struct packet_investigate_started *)packet)->city_id);
+      ((const struct packet_investigate_started *)packet)->unit_id16,
+      ((const struct packet_investigate_started *)packet)->unit_id32,
+      ((const struct packet_investigate_started *)packet)->city_id16,
+      ((const struct packet_investigate_started *)packet)->city_id32);
     return TRUE;
 
   case PACKET_INVESTIGATE_FINISHED:
     handle_investigate_finished(
-      ((const struct packet_investigate_finished *)packet)->unit_id,
-      ((const struct packet_investigate_finished *)packet)->city_id);
+      ((const struct packet_investigate_finished *)packet)->unit_id16,
+      ((const struct packet_investigate_finished *)packet)->unit_id32,
+      ((const struct packet_investigate_finished *)packet)->city_id16,
+      ((const struct packet_investigate_finished *)packet)->city_id32);
     return TRUE;
 
   case PACKET_SERVER_JOIN_REPLY:
@@ -134,7 +138,8 @@ bool client_handle_packet(enum packet_type type, const void *packet)
 
   case PACKET_CITY_REMOVE:
     handle_city_remove(
-      ((const struct packet_city_remove *)packet)->city_id);
+      ((const struct packet_city_remove *)packet)->city_id16,
+      ((const struct packet_city_remove *)packet)->city_id32);
     return TRUE;
 
   case PACKET_CITY_INFO:
@@ -155,17 +160,13 @@ bool client_handle_packet(enum packet_type type, const void *packet)
 
   case PACKET_CITY_NAME_SUGGESTION_INFO:
     handle_city_name_suggestion_info(
-      ((const struct packet_city_name_suggestion_info *)packet)->unit_id,
+      ((const struct packet_city_name_suggestion_info *)packet)->unit_id16,
+      ((const struct packet_city_name_suggestion_info *)packet)->unit_id32,
       ((const struct packet_city_name_suggestion_info *)packet)->name);
     return TRUE;
 
   case PACKET_CITY_SABOTAGE_LIST:
-    handle_city_sabotage_list(
-      ((const struct packet_city_sabotage_list *)packet)->actor_id,
-      ((const struct packet_city_sabotage_list *)packet)->city_id,
-      ((const struct packet_city_sabotage_list *)packet)->improvements,
-      ((const struct packet_city_sabotage_list *)packet)->act_id,
-      ((const struct packet_city_sabotage_list *)packet)->request_kind);
+    handle_city_sabotage_list(packet);
     return TRUE;
 
   case PACKET_CITY_RALLY_POINT:
@@ -204,7 +205,8 @@ bool client_handle_packet(enum packet_type type, const void *packet)
 
   case PACKET_UNIT_REMOVE:
     handle_unit_remove(
-      ((const struct packet_unit_remove *)packet)->unit_id);
+      ((const struct packet_unit_remove *)packet)->unit_id16,
+      ((const struct packet_unit_remove *)packet)->unit_id32);
     return TRUE;
 
   case PACKET_UNIT_INFO:
@@ -220,12 +222,7 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     return TRUE;
 
   case PACKET_UNIT_ACTION_ANSWER:
-    handle_unit_action_answer(
-      ((const struct packet_unit_action_answer *)packet)->actor_id,
-      ((const struct packet_unit_action_answer *)packet)->target_id,
-      ((const struct packet_unit_action_answer *)packet)->cost,
-      ((const struct packet_unit_action_answer *)packet)->action_type,
-      ((const struct packet_unit_action_answer *)packet)->request_kind);
+    handle_unit_action_answer(packet);
     return TRUE;
 
   case PACKET_UNIT_ACTIONS:
@@ -605,7 +602,8 @@ bool client_handle_packet(enum packet_type type, const void *packet)
 
   case PACKET_WEB_CITY_INFO_ADDITION:
     handle_web_city_info_addition(
-      ((const struct packet_web_city_info_addition *)packet)->id,
+      ((const struct packet_web_city_info_addition *)packet)->id16,
+      ((const struct packet_web_city_info_addition *)packet)->id32,
       ((const struct packet_web_city_info_addition *)packet)->granary_size,
       ((const struct packet_web_city_info_addition *)packet)->granary_turns);
     return TRUE;
