@@ -776,7 +776,8 @@ bool action_enabler_utype_possible_actor(const struct action_enabler *ae,
                                          const struct unit_type *act_utype);
 bool action_enabler_possible_actor(const struct action_enabler *ae);
 
-struct action *action_is_blocked_by(const struct action *act,
+struct action *action_is_blocked_by(const struct civ_map *nmap,
+                                    const struct action *act,
                                     const struct unit *actor_unit,
                                     const struct tile *target_tile,
                                     const struct city *target_city,
@@ -840,10 +841,12 @@ struct act_prob action_prob_vs_extras(const struct civ_map *nmap,
                                       const struct tile *target,
                                       const struct extra_type *tgt_extra);
 
-struct act_prob action_prob_self(const struct unit *actor,
+struct act_prob action_prob_self(const struct civ_map *nmap,
+                                 const struct unit *actor,
                                  const action_id act_id);
 
-struct act_prob action_prob_unit_vs_tgt(const struct action *paction,
+struct act_prob action_prob_unit_vs_tgt(const struct civ_map *nmap,
+                                        const struct action *paction,
                                         const struct unit *act_unit,
                                         const struct city *tgt_city,
                                         const struct unit *tgt_unit,
@@ -851,7 +854,8 @@ struct act_prob action_prob_unit_vs_tgt(const struct action *paction,
                                         const struct extra_type *sub_tgt);
 
 struct act_prob
-action_speculate_unit_on_city(action_id act_id,
+action_speculate_unit_on_city(const struct civ_map *nmap,
+                              action_id act_id,
                               const struct unit *actor,
                               const struct city *actor_home,
                               const struct tile *actor_tile,
@@ -859,7 +863,8 @@ action_speculate_unit_on_city(action_id act_id,
                               const struct city* target);
 
 struct act_prob
-action_speculate_unit_on_unit(action_id act_id,
+action_speculate_unit_on_unit(const struct civ_map *nmap,
+                              action_id act_id,
                               const struct unit *actor,
                               const struct city *actor_home,
                               const struct tile *actor_tile,
@@ -867,7 +872,8 @@ action_speculate_unit_on_unit(action_id act_id,
                               const struct unit *target);
 
 struct act_prob
-action_speculate_unit_on_units(action_id act_id,
+action_speculate_unit_on_units(const struct civ_map *nmap,
+                               action_id act_id,
                                const struct unit *actor,
                                const struct city *actor_home,
                                const struct tile *actor_tile,
@@ -875,7 +881,8 @@ action_speculate_unit_on_units(action_id act_id,
                                const struct tile *target);
 
 struct act_prob
-action_speculate_unit_on_tile(action_id act_id,
+action_speculate_unit_on_tile(const struct civ_map *nmap,
+                              action_id act_id,
                               const struct unit *actor,
                               const struct city *actor_home,
                               const struct tile *actor_tile,
@@ -884,7 +891,8 @@ action_speculate_unit_on_tile(action_id act_id,
                               const struct extra_type *target_extra);
 
 struct act_prob
-action_speculate_unit_on_extras(action_id act_id,
+action_speculate_unit_on_extras(const struct civ_map *nmap,
+                                action_id act_id,
                                 const struct unit *actor,
                                 const struct city *actor_home,
                                 const struct tile *actor_tile,
@@ -893,7 +901,8 @@ action_speculate_unit_on_extras(action_id act_id,
                                 const struct extra_type *target_extra);
 
 struct act_prob
-action_speculate_unit_on_self(action_id act_id,
+action_speculate_unit_on_self(const struct civ_map *nmap,
+                              action_id act_id,
                               const struct unit *actor,
                               const struct city *actor_home,
                               const struct tile *actor_tile,
