@@ -152,7 +152,7 @@ void caravan_result_init_zero(struct caravan_result *result)
 
 /************************************************************************//**
   Initialize the result to go from src to dest with the given amount
-  of time.  This is useful for calling get_discounted_reward and the such.
+  of time. This is useful for calling get_discounted_reward() and the such.
 ****************************************************************************/
 static void caravan_result_init(struct caravan_result *result,
                                 const struct city *src,
@@ -188,7 +188,7 @@ int caravan_result_compare(const struct caravan_result *a,
   } else if (a->value < b->value) {
     return -1;
   } else {
-    /* faster time is better, so reverse-sort on time. */
+    /* Faster time is better, so reverse-sort on time. */
     return b->arrival_time - a->arrival_time ;
   }
 }
@@ -201,10 +201,12 @@ int caravan_result_compare(const struct caravan_result *a,
   The callback should return TRUE if it wants to stop searching,
   FALSE otherwise.
 ****************************************************************************/
-typedef bool (*search_callback) (void *data, const struct city *pcity,
+typedef bool (*search_callback) (const struct civ_map *nmap,
+                                 void *data, const struct city *pcity,
                                  int arrival_turn, int arrival_moves_left);
 
-static void caravan_search_from(const struct unit *caravan,
+static void caravan_search_from(const struct civ_map *nmap,
+                                const struct unit *caravan,
                                 const struct caravan_parameter *param,
                                 struct tile *start_tile,
                                 int turns_before, int moves_left_before,
@@ -215,7 +217,6 @@ static void caravan_search_from(const struct unit *caravan,
   struct pf_map *pfm;
   struct pf_parameter pfparam;
   int end_time;
-  const struct civ_map *nmap = &(wld.map);
 
   end_time = param->horizon - turns_before;
 
@@ -240,8 +241,9 @@ static void caravan_search_from(const struct unit *caravan,
     }
 
     pcity = tile_city(pos.tile);
-    if (pcity && callback(callback_data, pcity, turns_before + pos.turn,
-                          pos.moves_left)) {
+    if (pcity != NULL
+        && callback(nmap, callback_data, pcity, turns_before + pos.turn,
+                    pos.moves_left)) {
       break;
     }
   } pf_map_positions_iterate_end;
@@ -483,8 +485,9 @@ static bool get_discounted_reward(const struct unit *caravan,
   bool consider_wonder;
   bool consider_trade;
   bool consider_windfall;
+  const struct civ_map *nmap = &(wld.map);
 
-  /* if no foreign trade is allowed, just quit. */
+  /* If no foreign trade is allowed, just quit. */
   if (!does_foreign_trade_param_allow(parameter, pplayer_src, pplayer_dest)) {
     caravan_result_init_zero(result);
     return FALSE;
@@ -492,17 +495,17 @@ static bool get_discounted_reward(const struct unit *caravan,
 
   consider_wonder = parameter->consider_wonders
     && action_prob_possible(
-        action_speculate_unit_on_city(ACTION_HELP_WONDER,
+        action_speculate_unit_on_city(nmap, ACTION_HELP_WONDER,
                                       caravan, src, city_tile(dest),
                                       TRUE, dest));
   consider_trade = parameter->consider_trade
     && action_prob_possible(
-        action_speculate_unit_on_city(ACTION_TRADE_ROUTE,
+        action_speculate_unit_on_city(nmap, ACTION_TRADE_ROUTE,
                                       caravan, src, city_tile(dest),
                                       TRUE, dest));
   consider_windfall = parameter->consider_windfall
     && action_prob_possible(
-        action_speculate_unit_on_city(ACTION_MARKETPLACE,
+        action_speculate_unit_on_city(nmap, ACTION_MARKETPLACE,
                                       caravan, src, city_tile(dest),
                                       TRUE, dest));
 
@@ -544,7 +547,7 @@ static bool get_discounted_reward(const struct unit *caravan,
 
   if ((consider_trade
        || (consider_windfall
-           && (!parameter->consider_trade /* can only enter marketplaces */
+           && (!parameter->consider_trade /* Can only enter marketplaces */
                /* (FIXME: test any extra restrictions for trade routes) */
                /* or the result is so big that we are still in plus */
                /* (consider having produced IF_GOLD instead) */
@@ -598,7 +601,8 @@ struct cewt_data {
   const struct caravan_parameter *param;
 };
 
-static bool cewt_callback(void *vdata, const struct city *dest,
+static bool cewt_callback(const struct civ_map *nmap,
+                          void *vdata, const struct city *dest,
                           int arrival_time, int moves_left)
 {
   struct cewt_data *data = vdata;
@@ -618,7 +622,8 @@ static bool cewt_callback(void *vdata, const struct city *dest,
   Using the caravan_search function to take transit time into account,
   evaluate the benefit of sending the caravan to dest.
 ****************************************************************************/
-static void caravan_evaluate_withtransit(const struct unit *caravan,
+static void caravan_evaluate_withtransit(const struct civ_map *nmap,
+                                         const struct unit *caravan,
                                          const struct city *dest,
                                          const struct caravan_parameter *param,
                                          struct caravan_result *result,
@@ -631,7 +636,7 @@ static void caravan_evaluate_withtransit(const struct unit *caravan,
   caravan_result_init(result, game_city_by_number(caravan->homecity),
                       dest, 0);
   data.result = result;
-  caravan_search_from(caravan, param, unit_tile(caravan), 0,
+  caravan_search_from(nmap, caravan, param, unit_tile(caravan), 0,
                       caravan->moves_left, omniscient, cewt_callback, &data);
 }
 
@@ -643,10 +648,12 @@ void caravan_evaluate(const struct unit *caravan,
                       const struct caravan_parameter *param,
                       struct caravan_result *result, bool omniscient)
 {
+  const struct civ_map *nmap = &(wld.map);
+
   if (param->ignore_transit_time) {
     caravan_evaluate_notransit(caravan, dest, param, result);
   } else {
-    caravan_evaluate_withtransit(caravan, dest, param, result, omniscient);
+    caravan_evaluate_withtransit(nmap, caravan, dest, param, result, omniscient);
   }
 }
 
@@ -669,9 +676,8 @@ static void caravan_find_best_destination_notransit(const struct unit *caravan,
     if (does_foreign_trade_param_allow(param, src_owner, dest_owner)) {
       city_list_iterate(dest_owner->cities, dest) {
         caravan_result_init(&current, pcity, dest, 0);
-        get_discounted_reward(caravan, param, &current);
-
-        if (caravan_result_compare(&current, best) > 0) {
+        if (get_discounted_reward(caravan, param, &current)
+            && caravan_result_compare(&current, best) > 0) {
           *best = current;
         }
       } city_list_iterate_end;
@@ -689,7 +695,8 @@ struct cfbdw_data {
   struct caravan_result *best;
 };
 
-static bool cfbdw_callback(void *vdata, const struct city *dest,
+static bool cfbdw_callback(const struct civ_map *nmap,
+                           void *vdata, const struct city *dest,
                            int arrival_time, int moves_left)
 {
   struct cfbdw_data *data = vdata;
@@ -697,18 +704,19 @@ static bool cfbdw_callback(void *vdata, const struct city *dest,
 
   caravan_result_init(&current, data->best->src, dest, arrival_time);
 
-  get_discounted_reward(data->caravan, data->param, &current);
-  if (caravan_result_compare(&current, data->best) > 0) {
+  if (get_discounted_reward(data->caravan, data->param, &current)
+      && caravan_result_compare(&current, data->best) > 0) {
     *data->best = current;
   }
 
-  return FALSE; /* don't stop. */
+  return FALSE; /* Don't stop. */
 }
 
 /************************************************************************//**
   Using caravan_search, find the best destination.
 ****************************************************************************/
 static void caravan_find_best_destination_withtransit(
+    const struct civ_map *nmap,
     const struct unit *caravan,
     const struct caravan_parameter *param,
     const struct city *src,
@@ -731,7 +739,7 @@ static void caravan_find_best_destination_withtransit(
     start_tile = unit_tile(caravan);
   }
 
-  caravan_search_from(caravan, param, start_tile, turns_before,
+  caravan_search_from(nmap, caravan, param, start_tile, turns_before,
                       caravan->moves_left, omniscient, cfbdw_callback, &data);
 }
 
@@ -740,7 +748,8 @@ static void caravan_find_best_destination_withtransit(
   Store it in *destout (if destout is non-null); return the value of the
   trade route.
 ****************************************************************************/
-void caravan_find_best_destination(const struct unit *caravan,
+void caravan_find_best_destination(const struct civ_map *nmap,
+                                   const struct unit *caravan,
                                    const struct caravan_parameter *parameter,
                                    struct caravan_result *result, bool omniscient)
 {
@@ -751,7 +760,7 @@ void caravan_find_best_destination(const struct unit *caravan,
 
     fc_assert(src != NULL);
 
-    caravan_find_best_destination_withtransit(caravan, parameter, src, 0,
+    caravan_find_best_destination_withtransit(nmap, caravan, parameter, src, 0,
                                               caravan->moves_left, omniscient, result);
   }
 }
@@ -798,12 +807,13 @@ struct cowt_data {
 
 /************************************************************************//**
   Callback for the caravan_search invocation in
-  caravan_optimize_withtransit.
+  caravan_optimize_withtransit().
 
-  For every city we can reach, use caravan_find_best_destination as a
-  subroutine.
+  For every city we can reach, use caravan_find_best_destination_withtransit()
+  as a subroutine.
 ****************************************************************************/
-static bool cowt_callback(void *vdata, const struct city *pcity,
+static bool cowt_callback(const struct civ_map *nmap, void *vdata,
+                          const struct city *pcity,
                           int arrival_time, int moves_left)
 {
   struct cowt_data *data = vdata;
@@ -822,7 +832,7 @@ static bool cowt_callback(void *vdata, const struct city *pcity,
   /* next, try changing home city (if we're allowed to) */
   if (city_owner(pcity) == unit_owner(caravan)) {
     caravan_find_best_destination_withtransit(
-                caravan, data->param, pcity, arrival_time, moves_left, data->omniscient,
+                nmap, caravan, data->param, pcity, arrival_time, moves_left, data->omniscient,
                 &current);
     if (caravan_result_compare(&current, data->best) > 0) {
       *data->best = current;
@@ -836,7 +846,8 @@ static bool cowt_callback(void *vdata, const struct city *pcity,
   Find the best src/dest pair (including possibly changing home city), taking
   account of the trip time.
 ****************************************************************************/
-static void caravan_optimize_withtransit(const struct unit *caravan,
+static void caravan_optimize_withtransit(const struct civ_map *nmap,
+                                         const struct unit *caravan,
                                          const struct caravan_parameter *param,
                                          struct caravan_result *result,
                                          bool omniscient)
@@ -848,7 +859,7 @@ static void caravan_optimize_withtransit(const struct unit *caravan,
   data.best = result;
   data.omniscient = omniscient;
   caravan_result_init_zero(data.best);
-  caravan_search_from(caravan, param, unit_tile(caravan), 0,
+  caravan_search_from(nmap, caravan, param, unit_tile(caravan), 0,
                       caravan->moves_left, omniscient, cowt_callback, &data);
 }
 
@@ -861,9 +872,11 @@ void caravan_optimize_allpairs(const struct unit *caravan,
                                const struct caravan_parameter *param,
                                struct caravan_result *result, bool omniscient)
 {
+  const struct civ_map *nmap = &(wld.map);
+
   if (param->ignore_transit_time) {
     caravan_optimize_notransit(caravan, param, result);
   } else {
-    caravan_optimize_withtransit(caravan, param, result, omniscient);
+    caravan_optimize_withtransit(nmap, caravan, param, result, omniscient);
   }
 }
