@@ -262,7 +262,7 @@ extern "C" {
 /* MC for an impossible step. If this value is returned by get_MC it
  * is treated like TB_IGNORE for this step. This won't change the TB
  * for any other step to this tile. This is assumed to be negative,
- * i.e., to be catched by "if (cost < 0)" */
+ * i.e., to be caught by "if (cost < 0)" */
 #define PF_IMPOSSIBLE_MC -1
 
 /* The factor which is used to multiple total_EC in the total_CC
@@ -475,11 +475,10 @@ struct pf_parameter {
 /* The map itself. Opaque type. */
 struct pf_map;
 
-/* The reverse map strucure. Opaque type. */
+/* The reverse map structure. Opaque type. */
 struct pf_reverse_map;
 
 
-
 /* ========================= Public Interface ============================ */
 
 /* Create and free. */
@@ -523,15 +522,15 @@ void pf_path_print_real(const struct pf_path *path, enum log_level level,
 
 
 /* Reverse map functions (Costs to go to start tile). */
-struct pf_reverse_map *pf_reverse_map_new(const struct player *pplayer,
+struct pf_reverse_map *pf_reverse_map_new(const struct civ_map *nmap,
+                                          const struct player *pplayer,
                                           struct tile *start_tile,
-                                          int max_turns, bool omniscient,
-                                          const struct civ_map *nmap)
+                                          int max_turns, bool omniscient)
                        fc__warn_unused_result;
-struct pf_reverse_map *pf_reverse_map_new_for_city(const struct city *pcity,
+struct pf_reverse_map *pf_reverse_map_new_for_city(const struct civ_map *nmap,
+                                                   const struct city *pcity,
                                                    const struct player *attacker,
-                                                   int max_turns, bool omniscient,
-                                                   const struct civ_map *nmap)
+                                                   int max_turns, bool omniscient)
                        fc__warn_unused_result;
 void pf_reverse_map_destroy(struct pf_reverse_map *prfm);
 
