@@ -285,7 +285,7 @@ enum output_loss {
 };
 
 /* This enumerators are used at client side only (so changing it doesn't
- * break the compability) to mark that the city need specific gui updates
+ * break the compatibility) to mark that the city need specific gui updates
  * (e.g. city dialog, or city report). */
 enum city_updates {
   CU_NO_UPDATE          = 0,
@@ -564,11 +564,12 @@ citizens player_angry_citizens(const struct player *pplayer);
 int city_population(const struct city *pcity);
 int city_total_impr_gold_upkeep(const struct city *pcity);
 int city_total_unit_gold_upkeep(const struct city *pcity);
-int city_unit_unhappiness(struct unit *punit, int *free_unhappy);
-bool city_happy(const struct city *pcity);  /* generally use celebrating instead */
-bool city_unhappy(const struct city *pcity);                /* anarchy??? */
+int city_unit_unhappiness(const struct civ_map *nmap,
+                          struct unit *punit, int *free_unhappy);
+bool city_happy(const struct city *pcity);  /* Generally use celebrating instead */
+bool city_unhappy(const struct city *pcity);                /* Anarchy??? */
 bool base_city_celebrating(const struct city *pcity);
-bool city_celebrating(const struct city *pcity);            /* love the king ??? */
+bool city_celebrating(const struct city *pcity);            /* Love the king ??? */
 bool city_rapture_grow(const struct city *pcity);
 bool city_is_occupied(const struct city *pcity);
 
@@ -594,11 +595,14 @@ bool can_city_build_unit_now(const struct civ_map *nmap,
                              const struct city *pcity,
                              const struct unit_type *punittype);
 
-bool can_city_build_direct(const struct city *pcity,
+bool can_city_build_direct(const struct civ_map *nmap,
+                           const struct city *pcity,
                            const struct universal *target);
-bool can_city_build_later(const struct city *pcity,
+bool can_city_build_later(const struct civ_map *nmap,
+                          const struct city *pcity,
                           const struct universal *target);
-bool can_city_build_now(const struct city *pcity,
+bool can_city_build_now(const struct civ_map *nmap,
+                        const struct city *pcity,
                         const struct universal *target);
 
 int city_unit_slots_available(const struct city *pcity);
@@ -633,7 +637,7 @@ int city_turns_to_grow(const struct city *pcity);
 bool city_can_grow_to(const struct city *pcity, int pop_size);
 bool city_can_change_build(const struct city *pcity);
 
-void city_choose_build_default(struct city *pcity);
+void city_choose_build_default(const struct civ_map *nmap, struct city *pcity);
 
 /* Textual representation of buildings */
 
@@ -752,7 +756,8 @@ bool is_unit_near_a_friendly_city(const struct civ_map *nmap,
 bool is_friendly_city_near(const struct civ_map *nmap,
                            const struct player *owner,
                            const struct tile *ptile);
-bool city_exists_within_max_city_map(const struct tile *ptile,
+bool city_exists_within_max_city_map(const struct civ_map *nmap,
+                                     const struct tile *ptile,
                                      bool may_be_on_center);
 
 /* Granary size as a function of city size */
@@ -763,8 +768,9 @@ void city_add_improvement(struct city *pcity,
 void city_remove_improvement(struct city *pcity,
 			     const struct impr_type *pimprove);
 
-/* city update functions */
-void city_refresh_from_main_map(struct city *pcity, bool *workers_map);
+/* City update functions */
+void city_refresh_from_main_map(const struct civ_map *nmap,
+                                struct city *pcity, bool *workers_map);
 
 int city_waste(const struct city *pcity, Output_type_id otype, int total,
                int *breakdown);
@@ -773,13 +779,13 @@ Specialist_type_id best_specialist(Output_type_id otype,
 int get_final_city_output_bonus(const struct city *pcity, Output_type_id otype);
 bool city_built_last_turn(const struct city *pcity);
 
-/* city creation / destruction */
+/* City creation / destruction */
 struct city *create_city_virtual(struct player *pplayer,
 				 struct tile *ptile, const char *name);
 void destroy_city_virtual(struct city *pcity);
 bool city_is_virtual(const struct city *pcity);
 
-/* misc */
+/* Misc */
 bool is_city_option_set(const struct city *pcity, enum city_options option);
 void city_styles_alloc(int num);
 void city_styles_free(void);
