@@ -78,7 +78,7 @@ const struct ft_color ftc_server        = FT_COLOR("#8B0000",   NULL);
 const struct ft_color ftc_client        = FT_COLOR("#EF7F00",   NULL);
 const struct ft_color ftc_editor        = FT_COLOR("#0000FF",   NULL);
 const struct ft_color ftc_command       = FT_COLOR("#006400",   NULL);
-      struct ft_color ftc_changed       = FT_COLOR("#FF0000",   NULL);
+VAR_ARG_CONST struct ft_color ftc_changed = FT_COLOR("#FF0000", NULL);
 const struct ft_color ftc_server_prompt = FT_COLOR("#FF0000",   "#BEBEBE");
 const struct ft_color ftc_player_lost   = FT_COLOR("#FFFFFF",   "#000000");
 const struct ft_color ftc_game_start    = FT_COLOR("#00FF00",   "#115511");
