@@ -371,7 +371,7 @@ void conn_list_do_unbuffer(struct conn_list *dest)
 
 /**********************************************************************//**
   Find connection by exact user name, from game.all_connections,
-  case-insensitve.  Returns NULL if not found.
+  case-insensitive.  Returns NULL if not found.
 **************************************************************************/
 struct connection *conn_by_user(const char *user_name)
 {
@@ -385,7 +385,7 @@ struct connection *conn_by_user(const char *user_name)
 }
 
 /**********************************************************************//**
-  Like conn_by_username(), but allow unambigous prefix (i.e. abbreviation).
+  Like conn_by_username(), but allow unambiguous prefix (i.e. abbreviation).
   Returns NULL if could not match, or if ambiguous or other problem, and
   fills *result with characterisation of match/non-match (see
   "utility/shared.[ch]").
