@@ -29,13 +29,17 @@ struct packet_processing_finished {
 };
 
 struct packet_investigate_started {
-  int unit_id;
-  int city_id;
+  int unit_id16;
+  int unit_id32;
+  int city_id16;
+  int city_id32;
 };
 
 struct packet_investigate_finished {
-  int unit_id;
-  int city_id;
+  int unit_id16;
+  int unit_id32;
+  int city_id16;
+  int city_id32;
 };
 
 struct packet_server_join_req {
@@ -101,7 +105,8 @@ struct packet_tile_info {
   enum known_type known;
   int owner;
   int extras_owner;
-  int worked;
+  int worked16;
+  int worked32;
   Terrain_type_id terrain;
   Resource_type_id resource;
   bv_extras extras;
@@ -314,11 +319,13 @@ struct packet_server_info {
 };
 
 struct packet_city_remove {
-  int city_id;
+  int city_id16;
+  int city_id32;
 };
 
 struct packet_city_info {
-  int id;
+  int id16;
+  int id32;
   int tile;
   int owner;
   int original;
@@ -371,14 +378,16 @@ struct packet_city_info {
 };
 
 struct packet_city_nationalities {
-  int id;
+  int id16;
+  int id32;
   int nationalities_count;
   int nation_id[MAX_CITY_NATIONALITIES];
   int nation_citizens[MAX_CITY_NATIONALITIES];
 };
 
 struct packet_city_short_info {
-  int id;
+  int id16;
+  int id32;
   int tile;
   int owner;
   int size;
@@ -394,83 +403,100 @@ struct packet_city_short_info {
 };
 
 struct packet_trade_route_info {
-  int city;
+  int city16;
+  int city32;
   int index;
-  int partner;
+  int partner16;
+  int partner32;
   int value;
   enum route_direction direction;
   Goods_type_id goods;
 };
 
 struct packet_city_sell {
-  int city_id;
+  int city_id16;
+  int city_id32;
   int build_id;
 };
 
 struct packet_city_buy {
-  int city_id;
+  int city_id16;
+  int city_id32;
 };
 
 struct packet_city_change {
-  int city_id;
+  int city_id16;
+  int city_id32;
   int production_kind;
   int production_value;
 };
 
 struct packet_city_worklist {
-  int city_id;
+  int city_id16;
+  int city_id32;
   struct worklist worklist;
 };
 
 struct packet_city_make_specialist {
-  int city_id;
+  int city_id16;
+  int city_id32;
   int tile_id;
 };
 
 struct packet_city_make_worker {
-  int city_id;
+  int city_id16;
+  int city_id32;
   int tile_id;
 };
 
 struct packet_city_change_specialist {
-  int city_id;
+  int city_id16;
+  int city_id32;
   Specialist_type_id from;
   Specialist_type_id to;
 };
 
 struct packet_city_rename {
-  int city_id;
+  int city_id16;
+  int city_id32;
   char name[MAX_LEN_CITYNAME];
 };
 
 struct packet_city_options_req {
-  int city_id;
+  int city_id16;
+  int city_id32;
   bv_city_options options;
 };
 
 struct packet_city_refresh {
-  int city_id;
+  int city_id16;
+  int city_id32;
 };
 
 struct packet_city_name_suggestion_req {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
 };
 
 struct packet_city_name_suggestion_info {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
   char name[MAX_LEN_CITYNAME];
 };
 
 struct packet_city_sabotage_list {
-  int actor_id;
-  int city_id;
+  int actor_id16;
+  int actor_id32;
+  int city_id16;
+  int city_id32;
   bv_imprs improvements;
   action_id act_id;
   int request_kind;
 };
 
 struct packet_city_rally_point {
-  int city_id;
+  int city_id16;
+  int city_id32;
   int length;
   bool persistent;
   bool vigilant;
@@ -478,7 +504,8 @@ struct packet_city_rally_point {
 };
 
 struct packet_worker_task {
-  int city_id;
+  int city_id16;
+  int city_id32;
   int tile_id;
   enum unit_activity activity;
   int tgt;
@@ -608,16 +635,19 @@ struct packet_player_tech_goal {
 };
 
 struct packet_unit_remove {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
 };
 
 struct packet_unit_info {
-  int id;
+  int id16;
+  int id32;
   int owner;
   int nationality;
   int tile;
   enum direction8 facing;
-  int homecity;
+  int homecity16;
+  int homecity32;
   int upkeep[O_LAST];
   int veteran;
   enum server_side_agent ssa_controller;
@@ -627,7 +657,8 @@ struct packet_unit_info {
   bool done_moving;
   bool stay;
   Unit_type_id type;
-  int transported_by;
+  int transported_by16;
+  int transported_by32;
   int carrying;
   int movesleft;
   int hp;
@@ -651,7 +682,8 @@ struct packet_unit_info {
 };
 
 struct packet_unit_short_info {
-  int id;
+  int id16;
+  int id32;
   int owner;
   int tile;
   enum direction8 facing;
@@ -662,14 +694,18 @@ struct packet_unit_short_info {
   int hp;
   int activity;
   int activity_tgt;
-  int transported_by;
+  int transported_by16;
+  int transported_by32;
   int packet_use;
-  int info_city_id;
+  int info_city_id16;
+  int info_city_id32;
 };
 
 struct packet_unit_combat_info {
-  int attacker_unit_id;
-  int defender_unit_id;
+  int attacker_unit_id16;
+  int attacker_unit_id32;
+  int defender_unit_id16;
+  int defender_unit_id32;
   int attacker_hp;
   int defender_hp;
   bool make_att_veteran;
@@ -677,13 +713,15 @@ struct packet_unit_combat_info {
 };
 
 struct packet_unit_sscs_set {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
   enum unit_ss_data_type type;
   int value;
 };
 
 struct packet_unit_orders {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
   int src_tile;
   int length;
   bool repeat;
@@ -693,12 +731,14 @@ struct packet_unit_orders {
 };
 
 struct packet_unit_server_side_agent_set {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
   enum server_side_agent agent;
 };
 
 struct packet_unit_action_query {
-  int actor_id;
+  int actor_id16;
+  int actor_id32;
   int target_id;
   action_id action_type;
   int request_kind;
@@ -709,7 +749,8 @@ struct packet_unit_type_upgrade {
 };
 
 struct packet_unit_do_action {
-  int actor_id;
+  int actor_id16;
+  int actor_id32;
   int target_id;
   int sub_tgt_id;
   char name[MAX_LEN_NAME];
@@ -717,7 +758,8 @@ struct packet_unit_do_action {
 };
 
 struct packet_unit_action_answer {
-  int actor_id;
+  int actor_id16;
+  int actor_id32;
   int target_id;
   int cost;
   action_id action_type;
@@ -725,17 +767,22 @@ struct packet_unit_action_answer {
 };
 
 struct packet_unit_get_actions {
-  int actor_unit_id;
-  int target_unit_id;
+  int actor_unit_id16;
+  int actor_unit_id32;
+  int target_unit_id16;
+  int target_unit_id32;
   int target_tile_id;
   int target_extra_id;
   int request_kind;
 };
 
 struct packet_unit_actions {
-  int actor_unit_id;
-  int target_unit_id;
-  int target_city_id;
+  int actor_unit_id16;
+  int actor_unit_id32;
+  int target_unit_id16;
+  int target_unit_id32;
+  int target_city_id16;
+  int target_city_id32;
   int target_tile_id;
   int target_extra_id;
   int request_kind;
@@ -743,7 +790,8 @@ struct packet_unit_actions {
 };
 
 struct packet_unit_change_activity {
-  int unit_id;
+  int unit_id16;
+  int unit_id32;
   enum unit_activity activity;
   int target;
 };
@@ -1676,14 +1724,17 @@ struct packet_edit_unit_remove {
 };
 
 struct packet_edit_unit_remove_by_id {
-  int id;
+  int id16;
+  int id32;
 };
 
 struct packet_edit_unit {
-  int id;
+  int id16;
+  int id32;
   Unit_type_id utype;
   int owner;
-  int homecity;
+  int homecity16;
+  int homecity32;
   int moves_left;
   int hp;
   int veteran;
@@ -1695,7 +1746,8 @@ struct packet_edit_unit {
   bool moved;
   bool paradropped;
   bool done_moving;
-  int transported_by;
+  int transported_by16;
+  int transported_by32;
   bool stay;
 };
 
@@ -1707,11 +1759,13 @@ struct packet_edit_city_create {
 };
 
 struct packet_edit_city_remove {
-  int id;
+  int id16;
+  int id32;
 };
 
 struct packet_edit_city {
-  int id;
+  int id16;
+  int id32;
   char name[MAX_LEN_CITYNAME];
   int owner;
   int original;
@@ -1810,7 +1864,8 @@ struct packet_play_music {
 };
 
 struct packet_web_city_info_addition {
-  int id;
+  int id16;
+  int id32;
   int granary_size;
   int granary_turns;
 };
@@ -2028,10 +2083,10 @@ int send_packet_processing_started(struct connection *pc);
 int send_packet_processing_finished(struct connection *pc);
 
 int send_packet_investigate_started(struct connection *pc, const struct packet_investigate_started *packet);
-int dsend_packet_investigate_started(struct connection *pc, int unit_id, int city_id);
+int dsend_packet_investigate_started(struct connection *pc, int unit_id16, int unit_id32, int city_id16, int city_id32);
 
 int send_packet_investigate_finished(struct connection *pc, const struct packet_investigate_finished *packet);
-int dsend_packet_investigate_finished(struct connection *pc, int unit_id, int city_id);
+int dsend_packet_investigate_finished(struct connection *pc, int unit_id16, int unit_id32, int city_id16, int city_id32);
 
 int send_packet_server_join_req(struct connection *pc, const struct packet_server_join_req *packet);
 int dsend_packet_server_join_req(struct connection *pc, const char *username, const char *capability, const char *version_label, int major_version, int minor_version, int patch_version);
@@ -2098,8 +2153,8 @@ int dsend_packet_server_info(struct connection *pc, const char *version_label, i
 
 int send_packet_city_remove(struct connection *pc, const struct packet_city_remove *packet);
 void lsend_packet_city_remove(struct conn_list *dest, const struct packet_city_remove *packet);
-int dsend_packet_city_remove(struct connection *pc, int city_id);
-void dlsend_packet_city_remove(struct conn_list *dest, int city_id);
+int dsend_packet_city_remove(struct connection *pc, int city_id16, int city_id32);
+void dlsend_packet_city_remove(struct conn_list *dest, int city_id16, int city_id32);
 
 int send_packet_city_info(struct connection *pc, const struct packet_city_info *packet, bool force_to_send);
 void lsend_packet_city_info(struct conn_list *dest, const struct packet_city_info *packet, bool force_to_send);
@@ -2114,42 +2169,42 @@ int send_packet_trade_route_info(struct connection *pc, const struct packet_trad
 void lsend_packet_trade_route_info(struct conn_list *dest, const struct packet_trade_route_info *packet);
 
 int send_packet_city_sell(struct connection *pc, const struct packet_city_sell *packet);
-int dsend_packet_city_sell(struct connection *pc, int city_id, int build_id);
+int dsend_packet_city_sell(struct connection *pc, int city_id16, int city_id32, int build_id);
 
 int send_packet_city_buy(struct connection *pc, const struct packet_city_buy *packet);
-int dsend_packet_city_buy(struct connection *pc, int city_id);
+int dsend_packet_city_buy(struct connection *pc, int city_id16, int city_id32);
 
 int send_packet_city_change(struct connection *pc, const struct packet_city_change *packet);
-int dsend_packet_city_change(struct connection *pc, int city_id, int production_kind, int production_value);
+int dsend_packet_city_change(struct connection *pc, int city_id16, int city_id32, int production_kind, int production_value);
 
 int send_packet_city_worklist(struct connection *pc, const struct packet_city_worklist *packet);
-int dsend_packet_city_worklist(struct connection *pc, int city_id, const struct worklist *worklist);
+int dsend_packet_city_worklist(struct connection *pc, int city_id16, int city_id32, const struct worklist *worklist);
 
 int send_packet_city_make_specialist(struct connection *pc, const struct packet_city_make_specialist *packet);
-int dsend_packet_city_make_specialist(struct connection *pc, int city_id, int tile_id);
+int dsend_packet_city_make_specialist(struct connection *pc, int city_id16, int city_id32, int tile_id);
 
 int send_packet_city_make_worker(struct connection *pc, const struct packet_city_make_worker *packet);
-int dsend_packet_city_make_worker(struct connection *pc, int city_id, int tile_id);
+int dsend_packet_city_make_worker(struct connection *pc, int city_id16, int city_id32, int tile_id);
 
 int send_packet_city_change_specialist(struct connection *pc, const struct packet_city_change_specialist *packet);
-int dsend_packet_city_change_specialist(struct connection *pc, int city_id, Specialist_type_id from, Specialist_type_id to);
+int dsend_packet_city_change_specialist(struct connection *pc, int city_id16, int city_id32, Specialist_type_id from, Specialist_type_id to);
 
 int send_packet_city_rename(struct connection *pc, const struct packet_city_rename *packet);
-int dsend_packet_city_rename(struct connection *pc, int city_id, const char *name);
+int dsend_packet_city_rename(struct connection *pc, int city_id16, int city_id32, const char *name);
 
 int send_packet_city_options_req(struct connection *pc, const struct packet_city_options_req *packet);
-int dsend_packet_city_options_req(struct connection *pc, int city_id, bv_city_options options);
+int dsend_packet_city_options_req(struct connection *pc, int city_id16, int city_id32, bv_city_options options);
 
 int send_packet_city_refresh(struct connection *pc, const struct packet_city_refresh *packet);
-int dsend_packet_city_refresh(struct connection *pc, int city_id);
+int dsend_packet_city_refresh(struct connection *pc, int city_id16, int city_id32);
 
 int send_packet_city_name_suggestion_req(struct connection *pc, const struct packet_city_name_suggestion_req *packet);
-int dsend_packet_city_name_suggestion_req(struct connection *pc, int unit_id);
+int dsend_packet_city_name_suggestion_req(struct connection *pc, int unit_id16, int unit_id32);
 
 int send_packet_city_name_suggestion_info(struct connection *pc, const struct packet_city_name_suggestion_info *packet);
 void lsend_packet_city_name_suggestion_info(struct conn_list *dest, const struct packet_city_name_suggestion_info *packet);
-int dsend_packet_city_name_suggestion_info(struct connection *pc, int unit_id, const char *name);
-void dlsend_packet_city_name_suggestion_info(struct conn_list *dest, int unit_id, const char *name);
+int dsend_packet_city_name_suggestion_info(struct connection *pc, int unit_id16, int unit_id32, const char *name);
+void dlsend_packet_city_name_suggestion_info(struct conn_list *dest, int unit_id16, int unit_id32, const char *name);
 
 int send_packet_city_sabotage_list(struct connection *pc, const struct packet_city_sabotage_list *packet);
 void lsend_packet_city_sabotage_list(struct conn_list *dest, const struct packet_city_sabotage_list *packet);
@@ -2198,8 +2253,8 @@ int dsend_packet_player_tech_goal(struct connection *pc, int tech);
 
 int send_packet_unit_remove(struct connection *pc, const struct packet_unit_remove *packet);
 void lsend_packet_unit_remove(struct conn_list *dest, const struct packet_unit_remove *packet);
-int dsend_packet_unit_remove(struct connection *pc, int unit_id);
-void dlsend_packet_unit_remove(struct conn_list *dest, int unit_id);
+int dsend_packet_unit_remove(struct connection *pc, int unit_id16, int unit_id32);
+void dlsend_packet_unit_remove(struct conn_list *dest, int unit_id16, int unit_id32);
 
 int send_packet_unit_info(struct connection *pc, const struct packet_unit_info *packet);
 void lsend_packet_unit_info(struct conn_list *dest, const struct packet_unit_info *packet);
@@ -2211,33 +2266,33 @@ int send_packet_unit_combat_info(struct connection *pc, const struct packet_unit
 void lsend_packet_unit_combat_info(struct conn_list *dest, const struct packet_unit_combat_info *packet);
 
 int send_packet_unit_sscs_set(struct connection *pc, const struct packet_unit_sscs_set *packet);
-int dsend_packet_unit_sscs_set(struct connection *pc, int unit_id, enum unit_ss_data_type type, int value);
+int dsend_packet_unit_sscs_set(struct connection *pc, int unit_id16, int unit_id32, enum unit_ss_data_type type, int value);
 
 int send_packet_unit_orders(struct connection *pc, const struct packet_unit_orders *packet);
 
 int send_packet_unit_server_side_agent_set(struct connection *pc, const struct packet_unit_server_side_agent_set *packet);
-int dsend_packet_unit_server_side_agent_set(struct connection *pc, int unit_id, enum server_side_agent agent);
+int dsend_packet_unit_server_side_agent_set(struct connection *pc, int unit_id16, int unit_id32, enum server_side_agent agent);
 
 int send_packet_unit_action_query(struct connection *pc, const struct packet_unit_action_query *packet);
-int dsend_packet_unit_action_query(struct connection *pc, int actor_id, int target_id, action_id action_type, int request_kind);
+int dsend_packet_unit_action_query(struct connection *pc, int actor_id16, int actor_id32, int target_id, action_id action_type, int request_kind);
 
 int send_packet_unit_type_upgrade(struct connection *pc, const struct packet_unit_type_upgrade *packet);
 int dsend_packet_unit_type_upgrade(struct connection *pc, Unit_type_id type);
 
 int send_packet_unit_do_action(struct connection *pc, const struct packet_unit_do_action *packet);
-int dsend_packet_unit_do_action(struct connection *pc, int actor_id, int target_id, int sub_tgt_id, const char *name, action_id action_type);
+int dsend_packet_unit_do_action(struct connection *pc, int actor_id16, int actor_id32, int target_id, int sub_tgt_id, const char *name, action_id action_type);
 
 int send_packet_unit_action_answer(struct connection *pc, const struct packet_unit_action_answer *packet);
-int dsend_packet_unit_action_answer(struct connection *pc, int actor_id, int target_id, int cost, action_id action_type, int request_kind);
+int dsend_packet_unit_action_answer(struct connection *pc, int actor_id16, int actor_id32, int target_id, int cost, action_id action_type, int request_kind);
 
 int send_packet_unit_get_actions(struct connection *pc, const struct packet_unit_get_actions *packet);
-int dsend_packet_unit_get_actions(struct connection *pc, int actor_unit_id, int target_unit_id, int target_tile_id, int target_extra_id, int request_kind);
+int dsend_packet_unit_get_actions(struct connection *pc, int actor_unit_id16, int actor_unit_id32, int target_unit_id16, int target_unit_id32, int target_tile_id, int target_extra_id, int request_kind);
 
 int send_packet_unit_actions(struct connection *pc, const struct packet_unit_actions *packet);
-int dsend_packet_unit_actions(struct connection *pc, int actor_unit_id, int target_unit_id, int target_city_id, int target_tile_id, int target_extra_id, int request_kind, const struct act_prob *action_probabilities);
+int dsend_packet_unit_actions(struct connection *pc, int actor_unit_id16, int actor_unit_id32, int target_unit_id16, int target_unit_id32, int target_city_id16, int target_city_id32, int target_tile_id, int target_extra_id, int request_kind, const struct act_prob *action_probabilities);
 
 int send_packet_unit_change_activity(struct connection *pc, const struct packet_unit_change_activity *packet);
-int dsend_packet_unit_change_activity(struct connection *pc, int unit_id, enum unit_activity activity, int target);
+int dsend_packet_unit_change_activity(struct connection *pc, int unit_id16, int unit_id32, enum unit_activity activity, int target);
 
 int send_packet_diplomacy_init_meeting_req(struct connection *pc, const struct packet_diplomacy_init_meeting_req *packet);
 int dsend_packet_diplomacy_init_meeting_req(struct connection *pc, int counterpart);
@@ -2539,7 +2594,7 @@ int send_packet_edit_unit_remove(struct connection *pc, const struct packet_edit
 int dsend_packet_edit_unit_remove(struct connection *pc, int owner, int tile, Unit_type_id type, int count);
 
 int send_packet_edit_unit_remove_by_id(struct connection *pc, const struct packet_edit_unit_remove_by_id *packet);
-int dsend_packet_edit_unit_remove_by_id(struct connection *pc, int id);
+int dsend_packet_edit_unit_remove_by_id(struct connection *pc, int id16, int id32);
 
 int send_packet_edit_unit(struct connection *pc, const struct packet_edit_unit *packet);
 
@@ -2547,7 +2602,7 @@ int send_packet_edit_city_create(struct connection *pc, const struct packet_edit
 int dsend_packet_edit_city_create(struct connection *pc, int owner, int tile, int size, int tag);
 
 int send_packet_edit_city_remove(struct connection *pc, const struct packet_edit_city_remove *packet);
-int dsend_packet_edit_city_remove(struct connection *pc, int id);
+int dsend_packet_edit_city_remove(struct connection *pc, int id16, int id32);
 
 int send_packet_edit_city(struct connection *pc, const struct packet_edit_city *packet);
 
