@@ -650,8 +650,12 @@ void player_set_color(struct player *pplayer,
 }
 
 /*******************************************************************//**
-  Clear all player data. If full is set, then the nation and the team will
-  be cleared too.
+  Clear all player data.
+
+  If full is set, also
+  - The nation is cleared
+  - The team is cleared
+  - Nationality information of remaining units is adjusted
 ***********************************************************************/
 void player_clear(struct player *pplayer, bool full)
 {
@@ -708,6 +712,15 @@ void player_clear(struct player *pplayer, bool full)
   if (full) {
     team_remove_player(pplayer);
 
+    players_iterate_alive(owner) {
+      unit_list_iterate(owner->units, owned) {
+        if (unit_nationality(owned) == pplayer) {
+          /* Switch nationality to that of current owner. */
+          owned->nationality = owner;
+        }
+      } unit_list_iterate_end;
+    } players_iterate_alive_end;
+
     /* This comes last because log calls in the above functions
      * may use it. */
     if (pplayer->nation != NULL) {
@@ -879,7 +892,7 @@ const char *player_name(const struct player *pplayer)
 }
 
 /*******************************************************************//**
-  Find player by name, allowing unambigous prefix (ie abbreviation).
+  Find player by name, allowing unambiguous prefix (i.e. abbreviation).
   Returns NULL if could not match, or if ambiguous or other
   problem, and fills *result with characterisation of match/non-match
   (see shared.[ch])
@@ -1207,12 +1220,14 @@ struct unit *player_unit_by_number(const struct player *pplayer, int unit_id)
 }
 
 /*******************************************************************//**
-  Return true iff x,y is inside any of the player's city map.
+  Return true iff tile is inside any of the player's city map.
 ***********************************************************************/
 bool player_in_city_map(const struct player *pplayer,
                         const struct tile *ptile)
 {
-  city_tile_iterate(CITY_MAP_MAX_RADIUS_SQ, ptile, ptile1) {
+  const struct civ_map *nmap = &(wld.map);
+
+  city_tile_iterate(nmap, CITY_MAP_MAX_RADIUS_SQ, ptile, ptile1) {
     struct city *pcity = tile_city(ptile1);
 
     if (pcity
