@@ -923,7 +923,7 @@ bool api_methods_tile_city_exists_within_max_city_map(lua_State *L,
   LUASCRIPT_CHECK_STATE(L, FALSE);
   LUASCRIPT_CHECK_SELF(L, ptile, FALSE);
 
-  return city_exists_within_max_city_map(ptile, may_be_on_center);
+  return city_exists_within_max_city_map(&(wld.map), ptile, may_be_on_center);
 }
 
 /**********************************************************************//**
@@ -1174,10 +1174,12 @@ int api_methods_tile_sq_distance(lua_State *L, Tile *ptile1, Tile *ptile2)
 **************************************************************************/
 bool api_methods_unit_city_can_be_built_here(lua_State *L, Unit *punit)
 {
+  const struct civ_map *nmap = &(wld.map);
+
   LUASCRIPT_CHECK_STATE(L, FALSE);
   LUASCRIPT_CHECK_SELF(L, punit, FALSE);
 
-  return city_can_be_built_here(unit_tile(punit), punit, TRUE);
+  return city_can_be_built_here(nmap, unit_tile(punit), punit, TRUE);
 }
 
 /**********************************************************************//**
