@@ -551,7 +551,7 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
             if (adv_city_worker_act_get(pcity, cindex,
                                         action_id_get_activity(act)) >= 0
                 && action_prob_possible(
-                  action_speculate_unit_on_tile(act,
+                  action_speculate_unit_on_tile(nmap, act,
                                                 punit, unit_home(punit),
                                                 ptile,
                                                 parameter.omniscience,
@@ -590,11 +590,12 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
             if (removing) {
               as_rmextra_action_iterate(try_act) {
                 struct action *taction = action_by_number(try_act);
+
                 if (is_extra_removed_by_action(pextra, taction)) {
                   /* We do not even evaluate actions we can't do.
                    * Removal is not considered prerequisite for anything */
                   if (action_prob_possible(
-                        action_speculate_unit_on_tile(try_act,
+                        action_speculate_unit_on_tile(nmap, try_act,
                                                       punit,
                                                       unit_home(punit),
                                                       ptile,
@@ -609,10 +610,11 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
             } else {
               as_extra_action_iterate(try_act) {
                 struct action *taction = action_by_number(try_act);
+
                 if (is_extra_caused_by_action(pextra, taction)) {
                   eval_act = action_id_get_activity(try_act);
                   if (action_prob_possible(
-                        action_speculate_unit_on_tile(try_act,
+                        action_speculate_unit_on_tile(nmap, try_act,
                                                       punit,
                                                       unit_home(punit),
                                                       ptile,
@@ -716,7 +718,7 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
                   dep_tgt = road_extra_get(pdep);
 
                   if (action_prob_possible(
-                        action_speculate_unit_on_tile(ACTION_ROAD,
+                        action_speculate_unit_on_tile(nmap, ACTION_ROAD,
                                                       punit, unit_home(punit), ptile,
                                                       parameter.omniscience,
                                                       ptile, dep_tgt))) {
@@ -761,7 +763,7 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
 
                     if (eval_dep_act != ACTIVITY_LAST) {
                       if (action_prob_possible(
-                            action_speculate_unit_on_tile(eval_dep_action,
+                            action_speculate_unit_on_tile(nmap, eval_dep_action,
                                                           punit, unit_home(punit), ptile,
                                                           parameter.omniscience,
                                                           ptile, pdep))) {
@@ -934,7 +936,7 @@ void auto_settler_findwork(const struct civ_map *nmap,
   /* Time it will take worker to complete its given task */
   int completion_time = 0;
 
-  /* Terminate what might be an inifite recursion of two units
+  /* Terminate what might be an infinite recursion of two units
    * displacing each other, but leave enough space for
    * finite recursion. */
   if (recursion > 5
@@ -1287,6 +1289,8 @@ bool auto_settlers_speculate_can_act_at(const struct unit *punit,
                                         struct extra_type *target,
                                         const struct tile *ptile)
 {
+  const struct civ_map *nmap = &(wld.map);
+
   action_by_activity_iterate(paction, activity) {
     if (action_get_actor_kind(paction) != AAK_UNIT) {
       /* Not relevant. */
@@ -1296,7 +1300,7 @@ bool auto_settlers_speculate_can_act_at(const struct unit *punit,
     switch (action_get_target_kind(paction)) {
     case ATK_CITY:
       return action_prob_possible(action_speculate_unit_on_city(
-                                    paction->id,
+                                    nmap, paction->id,
                                     punit, unit_home(punit), ptile,
                                     omniscient_cheat,
                                     tile_city(ptile)));
@@ -1306,25 +1310,25 @@ bool auto_settlers_speculate_can_act_at(const struct unit *punit,
       break;
     case ATK_UNITS:
       return action_prob_possible(action_speculate_unit_on_units(
-                                    paction->id,
+                                    nmap, paction->id,
                                     punit, unit_home(punit), ptile,
                                     omniscient_cheat,
                                     ptile));
     case ATK_TILE:
       return action_prob_possible(action_speculate_unit_on_tile(
-                                    paction->id,
+                                    nmap, paction->id,
                                     punit, unit_home(punit), ptile,
                                     omniscient_cheat,
                                     ptile, target));
     case ATK_EXTRAS:
       return action_prob_possible(action_speculate_unit_on_extras(
-                                    paction->id,
+                                    nmap, paction->id,
                                     punit, unit_home(punit), ptile,
                                     omniscient_cheat,
                                     ptile, target));
     case ATK_SELF:
       return action_prob_possible(action_speculate_unit_on_self(
-                                    paction->id,
+                                    nmap, paction->id,
                                     punit, unit_home(punit), ptile,
                                     omniscient_cheat));
     case ATK_COUNT:
