@@ -61,6 +61,7 @@ void handle_city_name_suggestion_req(struct player *pplayer, int unit_id16,
                                      int unit_id32)
 {
   struct unit *punit;
+  const struct civ_map *nmap = &(wld.map);
 
   if (!has_capability("ids32", pplayer->current_conn->capability)) {
     unit_id32 = unit_id16;
@@ -75,7 +76,7 @@ void handle_city_name_suggestion_req(struct player *pplayer, int unit_id16,
     return;
   }
 
-  if (action_prob_possible(action_prob_vs_tile(punit, ACTION_FOUND_CITY,
+  if (action_prob_possible(action_prob_vs_tile(nmap, punit, ACTION_FOUND_CITY,
                                                unit_tile(punit), NULL))) {
     log_verbose("handle_city_name_suggest_req(unit_pos (%d, %d))",
                 TILE_XY(unit_tile(punit)));
@@ -480,6 +481,7 @@ void handle_city_change(struct player *pplayer, int city_id16, int city_id32,
 {
   struct universal prod;
   struct city *pcity;
+  const struct civ_map *nmap = &(wld.map);
 
   if (!has_capability("ids32", pplayer->current_conn->capability)) {
     city_id32 = city_id16;
@@ -511,7 +513,7 @@ void handle_city_change(struct player *pplayer, int city_id16, int city_id32,
     return;
   }
 
-  if (!can_city_build_now(pcity, &prod)) {
+  if (!can_city_build_now(nmap, pcity, &prod)) {
     return;
   }
   if (!city_can_change_build(pcity)) {
