@@ -1029,6 +1029,9 @@ void map_change_seen(struct player *pplayer,
     if (pcity != NULL && city_owner(pcity) == pplayer) {
       city_map_update_empty(pcity, ptile);
       pcity->specialists[DEFAULT_SPECIALIST]++;
+      if (pcity->server.needs_arrange == CNA_NOT) {
+        pcity->server.needs_arrange = CNA_NORMAL;
+      }
     }
 
     update_player_tile_last_seen(pplayer, ptile);
@@ -1860,23 +1863,27 @@ static void terrain_change_bounce_single_unit(struct unit *punit,
 /**********************************************************************//**
   Helper function for bounce_units_on_terrain_change() that checks units
   on a single tile.
+
+  @param ptile Tile where the units to check are located
 **************************************************************************/
 static void check_units_single_tile(struct tile *ptile)
 {
+  const struct civ_map *nmap = &(wld.map);
+
   unit_list_iterate_safe(ptile->units, punit) {
     int id = punit->id;
 
     /* Top-level transports only. Each handle their own cargo */
     if (!unit_transported(punit)
-        && !can_unit_exist_at_tile(&(wld.map), punit, ptile)) {
+        && !can_unit_exist_at_tile(nmap, punit, ptile)) {
       terrain_change_bounce_single_unit(punit, ptile);
 
       if (unit_is_alive(id) && unit_tile(punit) == ptile) {
-        log_verbose("Disbanded %s %s due to changing land "
-                    " to sea at (%d, %d).",
+        log_verbose("Disbanded %s %s due to changing terrain "
+                    "at (%d, %d).",
                     nation_rule_name(nation_of_unit(punit)),
-                    unit_rule_name(punit), TILE_XY(unit_tile(punit)));
-        notify_player(unit_owner(punit), unit_tile(punit),
+                    unit_rule_name(punit), TILE_XY(ptile));
+        notify_player(unit_owner(punit), ptile,
                       E_UNIT_LOST_MISC, ftc_server,
                       _("Disbanded your %s due to changing terrain."),
                       unit_tile_link(punit));
@@ -1890,14 +1897,19 @@ static void check_units_single_tile(struct tile *ptile)
   Check ptile and nearby tiles to see if all units can remain at their
   current locations, and move or disband any that cannot. Call this after
   terrain or specials change on ptile.
+
+  @param ptile Tile where the terrain may have changed
 **************************************************************************/
 void bounce_units_on_terrain_change(struct tile *ptile)
 {
+  const struct civ_map *nmap = &(wld.map);
+
   /* Check this tile for direct effect on its units */
   check_units_single_tile(ptile);
   /* We have to check adjacent tiles too, in case units in cities are now
-   * illegal (e.g., boat in a city that has become landlocked). */
-  adjc_iterate(&(wld.map), ptile, ptile2) {
+   * illegal (e.g., boat in a city that has become landlocked),
+   * and in case of CoastStrict units losing their adjacent coast. */
+  adjc_iterate(nmap, ptile, ptile2) {
     check_units_single_tile(ptile2);
   } adjc_iterate_end;
 }
@@ -2036,9 +2048,19 @@ void check_terrain_change(struct tile *ptile, struct terrain *oldter)
   }
 
   claimer = tile_claimer(ptile);
-  if (claimer != NULL && is_ocean_tile(ptile)) {
-    if (!is_claimable_ocean(ptile, claimer, tile_owner(ptile))) {
-      map_clear_border(ptile);
+  if (claimer != NULL) {
+    /* Make sure map_claim_border() conditions are still satisfied */
+    if (is_ocean_tile(ptile)) {
+      /* Only certain water tiles are claimable */
+      if (!is_claimable_ocean(ptile, claimer, tile_owner(ptile))) {
+        map_clear_border(ptile);
+      }
+    } else {
+      /* Only land tiles on the same island as the border source
+       * are claimable */
+      if (tile_continent(ptile) != tile_continent(claimer)) {
+        map_clear_border(ptile);
+      }
     }
   }
 
@@ -2060,13 +2082,12 @@ static bool is_claimable_ocean(struct tile *ptile, struct tile *source,
                                struct player *pplayer)
 {
   Continent_id cont = tile_continent(ptile);
-  Continent_id cont1 = tile_continent(source);
-  Continent_id cont2;
+  Continent_id source_cont = tile_continent(source);
   int ocean_tiles;
   bool other_continent;
 
   if (get_ocean_size(-cont) <= MAXIMUM_CLAIMED_OCEAN_SIZE
-      && get_lake_surrounders(cont) == cont1) {
+      && get_lake_surrounders(cont) == source_cont) {
     return TRUE;
   }
 
@@ -2076,26 +2097,23 @@ static bool is_claimable_ocean(struct tile *ptile, struct tile *source,
   }
 
   if (num_known_tech_with_flag(pplayer, TF_CLAIM_OCEAN) > 0
-      || (cont1 < 0 && num_known_tech_with_flag(pplayer, TF_CLAIM_OCEAN_LIMITED) > 0)) {
+      || (source_cont < 0 && num_known_tech_with_flag(pplayer, TF_CLAIM_OCEAN_LIMITED) > 0)) {
     return TRUE;
   }
 
   ocean_tiles = 0;
   other_continent = FALSE;
-  adjc_iterate(&(wld.map), ptile, tile2) {
-    cont2 = tile_continent(tile2);
-    if (tile2 == source) {
+  adjc_iterate(&(wld.map), ptile, adj_tile) {
+    Continent_id adj_cont = tile_continent(adj_tile);
+    if (adj_tile == source) {
       /* Water next to border source is always claimable */
       return TRUE;
     }
-    if (cont2 == cont) {
+    if (adj_cont == cont) {
       ocean_tiles++;
-    } else if (cont1 <= 0) {
-      /* First adjacent land (only if border source is not on land) */
-      cont1 = cont2;
-    } else if (cont2 != cont1) {
-      /* This water has two land continents adjacent, or land adjacent
-       * that is of a different continent from the border source */
+    } else if (adj_cont != source_cont) {
+      /* This water is adjacent to a continent different from the one
+       * the border source is on */
       other_continent = TRUE;
     }
   } adjc_iterate_end;
@@ -2501,6 +2519,19 @@ void vision_clear_sight(struct vision *vision)
   const v_radius_t vision_radius_sq = V_RADIUS(-1, -1, -1);
 
   vision_change_sight(vision, vision_radius_sq);
+
+  /* Owner of some city might have lost vision of a tile previously worked */
+  players_iterate(pplayer) {
+    city_list_iterate(pplayer->cities, pcity) {
+      /* We are not interested about CNA_BROADCAST_PENDING, as the vision loss has
+       * not set it, and whatever set it should take care of it. */
+      if (pcity->server.needs_arrange == CNA_NORMAL) {
+        city_refresh(pcity);
+        auto_arrange_workers(pcity);
+        pcity->server.needs_arrange = CNA_NOT;
+      }
+    } city_list_iterate_end;
+  } players_iterate_end;
 }
 
 /**********************************************************************//**
