@@ -417,6 +417,9 @@ bool tab_tech::initialize_new_tech(struct advance *padv)
   padv->require[AR_ROOT] = none;
   name_set(&(padv->name), 0, "New Tech");
   BV_CLR_ALL(padv->flags);
+  if (padv->helptext != nullptr) {
+    strvec_clear(padv->helptext);
+  }
 
   return true;
 }
