@@ -414,7 +414,7 @@ void remove_leading_spaces(char *s)
 }
 
 /************************************************************************//**
-  Terminates string pointed to by 's' to remove traling spaces;
+  Terminates string pointed to by 's' to remove trailing spaces;
   Note 's' must point to writeable memory!
 ****************************************************************************/
 void remove_trailing_spaces(char *s)
@@ -1493,11 +1493,14 @@ void init_nls(void)
 
     if (lc->grouping[0] == '\0') {
       /* This actually indicates no grouping at all. */
-      char *m = malloc(sizeof(char));
+      char *m = fc_malloc(sizeof(char));
+
+      free(grouping);
       *m = CHAR_MAX;
       grouping = m;
     } else {
       size_t len;
+
       for (len = 0;
 	   lc->grouping[len] != '\0' && lc->grouping[len] != CHAR_MAX; len++) {
 	/* nothing */
